@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2007, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -36,7 +36,7 @@
 
 #include <limits>
 
-CPL_CVSID("$Id: ersdataset.cpp f43b2d1751a0a5add7998a1f861cdbf67744bc0b 2018-06-05 14:28:14 +0200 Even Rouault $")
+CPL_CVSID("$Id: ersdataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -46,7 +46,7 @@ CPL_CVSID("$Id: ersdataset.cpp f43b2d1751a0a5add7998a1f861cdbf67744bc0b 2018-06-
 
 class ERSRasterBand;
 
-class ERSDataset : public RawDataset
+class ERSDataset final: public RawDataset
 {
     friend class ERSRasterBand;
 
@@ -92,15 +92,29 @@ class ERSDataset : public RawDataset
     void FlushCache(void) override;
     CPLErr GetGeoTransform( double * padfTransform ) override;
     CPLErr SetGeoTransform( double *padfTransform ) override;
-    const char *GetProjectionRef(void) override;
-    CPLErr SetProjection( const char * ) override;
+    const char *_GetProjectionRef(void) override;
+    CPLErr _SetProjection( const char * ) override;
+    const OGRSpatialReference* GetSpatialRef() const override {
+        return GetSpatialRefFromOldGetProjectionRef();
+    }
+    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override {
+        return OldSetProjectionFromSetSpatialRef(poSRS);
+    }
     char **GetFileList(void) override;
 
     int GetGCPCount() override;
-    const char *GetGCPProjection() override;
+    const char *_GetGCPProjection() override;
+    const OGRSpatialReference* GetGCPSpatialRef() const override {
+        return GetGCPSpatialRefFromOldGetGCPProjection();
+    }
     const GDAL_GCP *GetGCPs() override;
-    CPLErr SetGCPs( int nGCPCount, const GDAL_GCP *pasGCPList,
+    CPLErr _SetGCPs( int nGCPCount, const GDAL_GCP *pasGCPList,
                     const char *pszGCPProjection ) override;
+    using RawDataset::SetGCPs;
+    CPLErr SetGCPs( int nGCPCountIn, const GDAL_GCP *pasGCPListIn,
+                    const OGRSpatialReference* poSRS ) override {
+        return OldSetGCPsFromNew(nGCPCountIn, pasGCPListIn, poSRS);
+    }
 
     char **GetMetadataDomainList() override;
     const char *GetMetadataItem( const char * pszName,
@@ -111,7 +125,7 @@ class ERSDataset : public RawDataset
     static int Identify( GDALOpenInfo * );
     static GDALDataset *Create( const char * pszFilename,
                                 int nXSize, int nYSize, int nBands,
-                                GDALDataType eType, char ** papszParmList );
+                                GDALDataType eType, char ** papszParamList );
 };
 
 /************************************************************************/
@@ -146,14 +160,14 @@ ERSDataset::ERSDataset() :
 ERSDataset::~ERSDataset()
 
 {
-    FlushCache();
+    ERSDataset::FlushCache();
 
     if( fpImage != nullptr )
     {
         VSIFCloseL( fpImage );
     }
 
-    CloseDependentDatasets();
+    ERSDataset::CloseDependentDatasets();
 
     CPLFree( pszProjection );
 
@@ -230,7 +244,7 @@ char **ERSDataset::GetMetadataDomainList()
 {
     return BuildMetadataDomainList(GDALPamDataset::GetMetadataDomainList(),
                                    TRUE,
-                                   "ERS", NULL);
+                                   "ERS", nullptr);
 }
 
 /************************************************************************/
@@ -288,7 +302,7 @@ int ERSDataset::GetGCPCount()
 /*                          GetGCPProjection()                          */
 /************************************************************************/
 
-const char *ERSDataset::GetGCPProjection()
+const char *ERSDataset::_GetGCPProjection()
 
 {
     return pszGCPProjection;
@@ -308,7 +322,7 @@ const GDAL_GCP *ERSDataset::GetGCPs()
 /*                              SetGCPs()                               */
 /************************************************************************/
 
-CPLErr ERSDataset::SetGCPs( int nGCPCountIn, const GDAL_GCP *pasGCPListIn,
+CPLErr ERSDataset::_SetGCPs( int nGCPCountIn, const GDAL_GCP *pasGCPListIn,
                             const char *pszGCPProjectionIn )
 
 {
@@ -404,11 +418,11 @@ CPLErr ERSDataset::SetGCPs( int nGCPCountIn, const GDAL_GCP *pasGCPListIn,
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *ERSDataset::GetProjectionRef()
+const char *ERSDataset::_GetProjectionRef()
 
 {
     // try xml first
-    const char* pszPrj = GDALPamDataset::GetProjectionRef();
+    const char* pszPrj = GDALPamDataset::_GetProjectionRef();
     if(pszPrj && strlen(pszPrj) > 0)
         return pszPrj;
 
@@ -419,7 +433,7 @@ const char *ERSDataset::GetProjectionRef()
 /*                           SetProjection()                            */
 /************************************************************************/
 
-CPLErr ERSDataset::SetProjection( const char *pszSRS )
+CPLErr ERSDataset::_SetProjection( const char *pszSRS )
 
 {
     if( pszProjection && EQUAL(pszSRS,pszProjection) )
@@ -731,14 +745,13 @@ void ERSDataset::ReadGCPs()
 /* ==================================================================== */
 /************************************************************************/
 
-class ERSRasterBand : public RawRasterBand
+class ERSRasterBand final: public RawRasterBand
 {
   public:
-    ERSRasterBand( GDALDataset *poDS, int nBand, void * fpRaw,
+    ERSRasterBand( GDALDataset *poDS, int nBand, VSILFILE * fpRaw,
                    vsi_l_offset nImgOffset, int nPixelOffset,
                    int nLineOffset,
-                   GDALDataType eDataType, int bNativeOrder,
-                   int bIsVSIL = FALSE, int bOwnsFP = FALSE );
+                   GDALDataType eDataType, int bNativeOrder );
 
     double GetNoDataValue( int *pbSuccess = nullptr ) override;
     CPLErr SetNoDataValue( double ) override;
@@ -748,14 +761,13 @@ class ERSRasterBand : public RawRasterBand
 /*                           ERSRasterBand()                            */
 /************************************************************************/
 
-ERSRasterBand::ERSRasterBand( GDALDataset *poDSIn, int nBandIn, void * fpRawIn,
+ERSRasterBand::ERSRasterBand( GDALDataset *poDSIn, int nBandIn, VSILFILE * fpRawIn,
                               vsi_l_offset nImgOffsetIn, int nPixelOffsetIn,
                               int nLineOffsetIn,
-                              GDALDataType eDataTypeIn, int bNativeOrderIn,
-                              int bIsVSILIn, int bOwnsFPIn ) :
+                              GDALDataType eDataTypeIn, int bNativeOrderIn ) :
     RawRasterBand(poDSIn, nBandIn, fpRawIn, nImgOffsetIn, nPixelOffsetIn,
-                  nLineOffsetIn, eDataTypeIn, bNativeOrderIn, bIsVSILIn,
-                  bOwnsFPIn)
+                  nLineOffsetIn, eDataTypeIn, bNativeOrderIn,
+                  RawRasterBand::OwnFP::NO)
 {}
 
 /************************************************************************/
@@ -1034,6 +1046,20 @@ GDALDataset *ERSDataset::Open( GDALOpenInfo * poOpenInfo )
                 return nullptr;
             }
 
+            if( !RAWDatasetCheckMemoryUsage(poDS->nRasterXSize,
+                                            poDS->nRasterYSize,
+                                            nBands,
+                                            iWordSize,
+                                            iWordSize,
+                                            iWordSize * nBands * poDS->nRasterXSize,
+                                            nHeaderOffset,
+                                            iWordSize * poDS->nRasterXSize,
+                                            poDS->fpImage) )
+            {
+                delete poDS;
+                return nullptr;
+            }
+
             for( int iBand = 0; iBand < nBands; iBand++ )
             {
                 // Assume pixel interleaved.
@@ -1044,7 +1070,7 @@ GDALDataset *ERSDataset::Open( GDALOpenInfo * poOpenInfo )
                                        + iWordSize * iBand * poDS->nRasterXSize,
                                        iWordSize,
                                        iWordSize * nBands * poDS->nRasterXSize,
-                                       eType, bNative, TRUE ));
+                                       eType, bNative ));
                 if( EQUAL(osCellType,"Signed8BitInteger") )
                     poDS->GetRasterBand(iBand+1)->
                         SetMetadataItem( "PIXELTYPE", "SIGNEDBYTE",
@@ -1259,7 +1285,7 @@ GDALDataset *ERSDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->TryLoadXML();
 
     // if no SR in xml, try aux
-    const char* pszPrj = poDS->GDALPamDataset::GetProjectionRef();
+    const char* pszPrj = poDS->GDALPamDataset::_GetProjectionRef();
     if( !pszPrj || strlen(pszPrj) == 0 )
     {
         // try aux
@@ -1486,7 +1512,7 @@ void GDALRegister_ERS()
     poDriver->SetDescription( "ERS" );
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "ERMapper .ers Labelled" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_ers.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/ers.html" );
     poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "ers" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
                                "Byte Int16 UInt16 Int32 UInt32 "
