@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2000, Frank Warmerdam
- * Copyright (c) 2008-2012, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2012, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -50,11 +50,15 @@
 #include "tifvsi.h"
 #include "xtiffio.h"
 
-CPL_CVSID("$Id: gt_overview.cpp 2e45a8bf0ec885228093226d18865364c9c1eba3 2018-04-14 20:35:01 +0200 Even Rouault $")
+CPL_CVSID("$Id: gt_overview.cpp db9c615f39e4da0623e12f467ff74d7946f17e24 2021-04-21 13:16:57 +0200 Even Rouault $")
 
 // TODO(schwehr): Explain why 128 and not 127.
 constexpr int knMaxOverviews = 128;
 
+#if TIFFLIB_VERSION > 20181110 // > 4.0.10
+#define SUPPORTS_GET_OFFSET_BYTECOUNT
+#endif
+
 /************************************************************************/
 /*                         GTIFFWriteDirectory()                        */
 /*                                                                      */
@@ -80,7 +84,10 @@ toff_t GTIFFWriteDirectory( TIFF *hTIFF, int nSubfileType,
                             const char *pszMetadata,
                             const char* pszJPEGQuality,
                             const char* pszJPEGTablesMode,
-                            const char* pszNoData )
+                            const char* pszNoData,
+                            CPL_UNUSED const uint32_t* panLercAddCompressionAndVersion,
+                            bool bDeferStrileArrayWriting,
+                            const char *pszWebpLevel)
 
 {
     const toff_t nBaseDirOffset = TIFFCurrentDirOffset( hTIFF );
@@ -88,10 +95,7 @@ toff_t GTIFFWriteDirectory( TIFF *hTIFF, int nSubfileType,
     // This is a bit of a hack to cause (*tif->tif_cleanup)(tif); to be called.
     // See https://trac.osgeo.org/gdal/ticket/2055
     TIFFSetField( hTIFF, TIFFTAG_COMPRESSION, COMPRESSION_NONE );
-
-#if defined(TIFFLIB_VERSION) && TIFFLIB_VERSION >= 20051201  // 3.8.0
     TIFFFreeDirectory( hTIFF );
-#endif
 
     TIFFCreateDirectory( hTIFF );
 
@@ -129,8 +133,7 @@ toff_t GTIFFWriteDirectory( TIFF *hTIFF, int nSubfileType,
                       panExtraSampleValues );
     }
 
-    if( nCompressFlag == COMPRESSION_LZW ||
-        nCompressFlag == COMPRESSION_ADOBE_DEFLATE )
+    if( GTIFFSupportsPredictor(nCompressFlag) )
         TIFFSetField( hTIFF, TIFFTAG_PREDICTOR, nPredictor );
 
 /* -------------------------------------------------------------------- */
@@ -168,6 +171,19 @@ toff_t GTIFFWriteDirectory( TIFF *hTIFF, int nSubfileType,
         }
     }
 
+    if (nCompressFlag == COMPRESSION_WEBP  && pszWebpLevel != nullptr)
+    {
+        const int nWebpLevel = atoi(pszWebpLevel);
+        if ( nWebpLevel >= 1 )
+            TIFFSetField( hTIFF, TIFFTAG_WEBP_LEVEL, nWebpLevel );
+    }
+
+    if( nCompressFlag == COMPRESSION_LERC && panLercAddCompressionAndVersion )
+    {
+        TIFFSetField(hTIFF, TIFFTAG_LERC_PARAMETERS, 2,
+                     panLercAddCompressionAndVersion);
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Write no data value if we have one.                             */
 /* -------------------------------------------------------------------- */
@@ -176,6 +192,13 @@ toff_t GTIFFWriteDirectory( TIFF *hTIFF, int nSubfileType,
         TIFFSetField( hTIFF, TIFFTAG_GDAL_NODATA, pszNoData );
     }
 
+    if( bDeferStrileArrayWriting )
+    {
+#ifdef SUPPORTS_GET_OFFSET_BYTECOUNT
+        TIFFDeferStrileArrayWriting( hTIFF );
+#endif
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Write directory, and return byte offset.                        */
 /* -------------------------------------------------------------------- */
@@ -288,10 +311,30 @@ GTIFFBuildOverviews( const char * pszFilename,
                      const char * pszResampling,
                      GDALProgressFunc pfnProgress, void * pProgressData )
 
+{
+    return GTIFFBuildOverviewsEx(pszFilename, nBands, papoBandList,
+                                 nOverviews, panOverviewList,
+                                 nullptr,
+                                 pszResampling,
+                                 nullptr,
+                                 pfnProgress, pProgressData);
+}
+
+CPLErr
+GTIFFBuildOverviewsEx( const char * pszFilename,
+                       int nBands, GDALRasterBand **papoBandList,
+                       int nOverviews,
+                       const int * panOverviewList,
+                       const std::pair<int, int>* pasOverviewSize,
+                       const char * pszResampling,
+                       const char* const* papszOptions,
+                       GDALProgressFunc pfnProgress, void * pProgressData )
 {
     if( nBands == 0 || nOverviews == 0 )
         return CE_None;
 
+    CPLAssert( (panOverviewList != nullptr) ^ (pasOverviewSize != nullptr) );
+
     if( !GTiffOneTimeInit() )
         return CE_Failure;
 
@@ -421,12 +464,15 @@ GTIFFBuildOverviews( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Use specified compression method.                               */
 /* -------------------------------------------------------------------- */
-    const char *pszCompress = CPLGetConfigOption( "COMPRESS_OVERVIEW", nullptr );
+    const char *pszCompress = papszOptions ?
+        CSLFetchNameValue(papszOptions, "COMPRESS") :
+        CPLGetConfigOption( "COMPRESS_OVERVIEW", nullptr );
 
     if( pszCompress != nullptr && pszCompress[0] != '\0' )
     {
         nCompression =
-            GTIFFGetCompressionMethod(pszCompress, "COMPRESS_OVERVIEW");
+            GTIFFGetCompressionMethod(pszCompress,
+                    papszOptions ? "COMPRESS" : "COMPRESS_OVERVIEW");
         if( nCompression < 0 )
             return CE_Failure;
     }
@@ -452,7 +498,35 @@ GTIFFBuildOverviews( const char * pszFilename,
     else
         nPlanarConfig = PLANARCONFIG_SEPARATE;
 
-    const char* pszInterleave =
+    bool bSourceIsPixelInterleaved = false;
+    bool bSourceIsJPEG2000 = false;
+    if( nBands > 1 )
+    {
+        GDALDataset* poSrcDS = papoBandList[0]->GetDataset();
+        if( poSrcDS )
+        {
+            const char* pszSrcInterleave = poSrcDS->GetMetadataItem("INTERLEAVE",
+                                                                "IMAGE_STRUCTURE");
+            if( pszSrcInterleave && EQUAL(pszSrcInterleave, "PIXEL") )
+            {
+                bSourceIsPixelInterleaved = true;
+            }
+        }
+
+        const char* pszSrcCompression = papoBandList[0]->GetMetadataItem("COMPRESSION",
+                                                                "IMAGE_STRUCTURE");
+        if( pszSrcCompression )
+        {
+            bSourceIsJPEG2000 = EQUAL(pszSrcCompression, "JPEG2000");
+        }
+        if( bSourceIsPixelInterleaved && bSourceIsJPEG2000 )
+        {
+            nPlanarConfig = PLANARCONFIG_CONTIG;
+        }
+    }
+
+    const char* pszInterleave = papszOptions ?
+        CSLFetchNameValue(papszOptions, "INTERLEAVE") :
         CPLGetConfigOption( "INTERLEAVE_OVERVIEW", nullptr );
     if( pszInterleave != nullptr && pszInterleave[0] != '\0' )
     {
@@ -476,6 +550,8 @@ GTIFFBuildOverviews( const char * pszFilename,
     if( nBands == 3 )
         nPhotometric = PHOTOMETRIC_RGB;
     else if( papoBandList[0]->GetColorTable() != nullptr
+             && (papoBandList[0]->GetRasterDataType() == GDT_Byte ||
+                 papoBandList[0]->GetRasterDataType() == GDT_UInt16)
              && !STARTS_WITH_CI(pszResampling, "AVERAGE_BIT2") )
     {
         nPhotometric = PHOTOMETRIC_PALETTE;
@@ -491,7 +567,8 @@ GTIFFBuildOverviews( const char * pszFilename,
     else
         nPhotometric = PHOTOMETRIC_MINISBLACK;
 
-    const char* pszPhotometric =
+    const char* pszPhotometric = papszOptions ?
+        CSLFetchNameValue(papszOptions, "PHOTOMETRIC") :
         CPLGetConfigOption( "PHOTOMETRIC_OVERVIEW", nullptr );
     if( pszPhotometric != nullptr && pszPhotometric[0] != '\0' )
     {
@@ -576,10 +653,10 @@ GTIFFBuildOverviews( const char * pszFilename,
 /*      Figure out the predictor value to use.                          */
 /* -------------------------------------------------------------------- */
     int nPredictor = PREDICTOR_NONE;
-    if( nCompression == COMPRESSION_LZW ||
-        nCompression == COMPRESSION_ADOBE_DEFLATE )
+    if( GTIFFSupportsPredictor(nCompression) )
     {
-        const char* pszPredictor =
+        const char* pszPredictor = papszOptions ?
+            CSLFetchNameValue(papszOptions, "PREDICTOR") :
             CPLGetConfigOption( "PREDICTOR_OVERVIEW", nullptr );
         if( pszPredictor != nullptr )
         {
@@ -593,6 +670,7 @@ GTIFFBuildOverviews( const char * pszFilename,
     VSIStatBufL sStatBuf;
     VSILFILE* fpL = nullptr;
 
+    bool bCreateBigTIFF = false;
     if( VSIStatExL( pszFilename, &sStatBuf, VSI_STAT_EXISTS_FLAG ) != 0 )
     {
     /* -------------------------------------------------------------------- */
@@ -604,37 +682,31 @@ GTIFFBuildOverviews( const char * pszFilename,
 
         for( iOverview = 0; iOverview < nOverviews; iOverview++ )
         {
-            const int nOXSize = (nXSize + panOverviewList[iOverview] - 1)
-                / panOverviewList[iOverview];
-            const int nOYSize = (nYSize + panOverviewList[iOverview] - 1)
-                / panOverviewList[iOverview];
+            const int nOXSize = panOverviewList ?
+                (nXSize + panOverviewList[iOverview] - 1)
+                    / panOverviewList[iOverview] :
+                // cppcheck-suppress nullPointer
+                pasOverviewSize[iOverview].first;
+            const int nOYSize = panOverviewList ?
+                (nYSize + panOverviewList[iOverview] - 1)
+                    / panOverviewList[iOverview] :
+                // cppcheck-suppress nullPointer
+                pasOverviewSize[iOverview].second;
 
             dfUncompressedOverviewSize +=
                 nOXSize * static_cast<double>(nOYSize) * nBands * nDataTypeSize;
         }
 
-        if( nCompression == COMPRESSION_NONE
-            && dfUncompressedOverviewSize > 4200000000.0 )
-        {
-    #ifndef BIGTIFF_SUPPORT
-            CPLError(
-                CE_Failure, CPLE_NotSupported,
-                "The overview file would be larger than 4GB, "
-                "but this is the largest size a TIFF can be, "
-                "and BigTIFF is unavailable.  "
-                "Creation failed." );
-            return CE_Failure;
-    #endif
-        }
     /* -------------------------------------------------------------------- */
     /*      Should the file be created as a bigtiff file?                   */
     /* -------------------------------------------------------------------- */
-        const char *pszBIGTIFF = CPLGetConfigOption( "BIGTIFF_OVERVIEW", nullptr );
+        const char *pszBIGTIFF = papszOptions ?
+            CSLFetchNameValue(papszOptions, "BIGTIFF") :
+            CPLGetConfigOption( "BIGTIFF_OVERVIEW", nullptr );
 
         if( pszBIGTIFF == nullptr )
             pszBIGTIFF = "IF_SAFER";
 
-        bool bCreateBigTIFF = false;
         if( EQUAL(pszBIGTIFF,"IF_NEEDED") )
         {
             if( nCompression == COMPRESSION_NONE
@@ -668,17 +740,6 @@ GTIFFBuildOverviews( const char * pszFilename,
             }
         }
 
-    #ifndef BIGTIFF_SUPPORT
-        if( bCreateBigTIFF )
-        {
-            CPLError(
-                CE_Warning, CPLE_NotSupported,
-                "BigTIFF requested, but GDAL built without BigTIFF "
-                "enabled libtiff, request ignored." );
-            bCreateBigTIFF = false;
-        }
-    #endif
-
         if( bCreateBigTIFF )
             CPLDebug( "GTiff", "File being created as a BigTIFF." );
 
@@ -709,7 +770,13 @@ GTIFFBuildOverviews( const char * pszFilename,
         if( fpL == nullptr )
             hOTIFF = nullptr;
         else
+        {
+            GByte abyBuffer[4] = { 0 };
+            VSIFReadL(abyBuffer, 1, 4, fpL);
+            VSIFSeekL(fpL, 0, SEEK_SET);
+            bCreateBigTIFF = abyBuffer[2] == 43 || abyBuffer[3] == 43;
             hOTIFF = VSI_TIFFOpen( pszFilename, "r+", fpL );
+        }
         if( hOTIFF == nullptr )
         {
             if( CPLGetLastErrorNo() == 0 )
@@ -765,15 +832,17 @@ GTIFFBuildOverviews( const char * pszFilename,
 /* -------------------------------------------------------------------- */
     CPLString osMetadata;
     GDALDataset *poBaseDS = papoBandList[0]->GetDataset();
-
-    GTIFFBuildOverviewMetadata( pszResampling, poBaseDS, osMetadata );
+    if( poBaseDS )
+    {
+        GTIFFBuildOverviewMetadata( pszResampling, poBaseDS, osMetadata );
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Loop, creating overviews.                                       */
 /* -------------------------------------------------------------------- */
     int nOvrBlockXSize = 0;
     int nOvrBlockYSize = 0;
-    GTIFFGetOverviewBlockSize(&nOvrBlockXSize, &nOvrBlockYSize);
+    GTIFFGetOverviewBlockSize(papoBandList[0], &nOvrBlockXSize, &nOvrBlockYSize);
 
     CPLString osNoData; // don't move this in inner scope
     const char* pszNoData = nullptr;
@@ -785,14 +854,17 @@ GTIFFBuildOverviews( const char * pszFilename,
         pszNoData = osNoData.c_str();
     }
 
-    std::vector<uint16> anExtraSamples;
+    std::vector<uint16_t> anExtraSamples;
     for( int i = GTIFFGetMaxColorChannels(nPhotometric)+1; i <= nBands; i++ )
     {
         if( papoBandList[i-1]->GetColorInterpretation() == GCI_AlphaBand )
         {
             anExtraSamples.push_back(
-                GTiffGetAlphaValue(CPLGetConfigOption("GTIFF_ALPHA", nullptr),
-                                   DEFAULT_ALPHA_TYPE));
+                GTiffGetAlphaValue(
+                    papszOptions ?
+                        CSLFetchNameValue(papszOptions, "ALPHA") :
+                        CPLGetConfigOption("GTIFF_ALPHA", nullptr),
+                    DEFAULT_ALPHA_TYPE));
         }
         else
         {
@@ -802,12 +874,31 @@ GTIFFBuildOverviews( const char * pszFilename,
 
     for( iOverview = 0; iOverview < nOverviews; iOverview++ )
     {
-        const int nOXSize = (nXSize + panOverviewList[iOverview] - 1)
-            / panOverviewList[iOverview];
-        const int nOYSize = (nYSize + panOverviewList[iOverview] - 1)
-            / panOverviewList[iOverview];
+        const int nOXSize = panOverviewList ?
+                (nXSize + panOverviewList[iOverview] - 1)
+                    / panOverviewList[iOverview] :
+                // cppcheck-suppress nullPointer
+                pasOverviewSize[iOverview].first;
+        const int nOYSize = panOverviewList ?
+                (nYSize + panOverviewList[iOverview] - 1)
+                    / panOverviewList[iOverview] :
+                // cppcheck-suppress nullPointer
+                pasOverviewSize[iOverview].second;
 
-        GTIFFWriteDirectory( hOTIFF, FILETYPE_REDUCEDIMAGE,
+        unsigned nTileXCount = DIV_ROUND_UP(nOXSize, nOvrBlockXSize);
+        unsigned nTileYCount = DIV_ROUND_UP(nOYSize, nOvrBlockYSize);
+        // libtiff implementation limitation
+        if( nTileXCount > 0x80000000U / (bCreateBigTIFF ? 8 : 4) / nTileYCount )
+        {
+            CPLError(CE_Failure, CPLE_NotSupported,
+                     "File too large regarding tile size. This would result "
+                     "in a file with tile arrays larger than 2GB");
+            XTIFFClose( hOTIFF );
+            VSIFCloseL(fpL);
+            return CE_Failure;
+        }
+
+        if( GTIFFWriteDirectory( hOTIFF, FILETYPE_REDUCEDIMAGE,
                              nOXSize, nOYSize, nBitsPerPixel,
                              nPlanarConfig, nBands,
                              nOvrBlockXSize, nOvrBlockYSize, TRUE, nCompression,
@@ -816,10 +907,24 @@ GTIFFBuildOverviews( const char * pszFilename,
                              static_cast<int>(anExtraSamples.size()),
                              anExtraSamples.empty() ? nullptr : anExtraSamples.data(),
                              osMetadata,
-                             CPLGetConfigOption( "JPEG_QUALITY_OVERVIEW", nullptr ),
-                             CPLGetConfigOption( "JPEG_TABLESMODE_OVERVIEW", nullptr ),
-                             pszNoData
-                           );
+                             papszOptions ?
+                                CSLFetchNameValue(papszOptions, "JPEG_QUALITY") :
+                                CPLGetConfigOption( "JPEG_QUALITY_OVERVIEW", nullptr ),
+                             papszOptions ?
+                                CSLFetchNameValue(papszOptions, "JPEG_TABLESMODE") :
+                                CPLGetConfigOption( "JPEG_TABLESMODE_OVERVIEW", nullptr ),
+                             pszNoData,
+                             nullptr,
+                             false,
+                             papszOptions ?
+                                CSLFetchNameValue(papszOptions, "WEBP_LEVEL") :
+                                CPLGetConfigOption( "WEBP_LEVEL_OVERVIEW", nullptr )
+                           ) == 0 )
+        {
+            XTIFFClose( hOTIFF );
+            VSIFCloseL(fpL);
+            return CE_Failure;
+        }
     }
 
     if( panRed )
@@ -841,8 +946,17 @@ GTIFFBuildOverviews( const char * pszFilename,
 /*      Open the overview dataset so that we can get at the overview    */
 /*      bands.                                                          */
 /* -------------------------------------------------------------------- */
-    GDALDataset *hODS =
-        static_cast<GDALDataset *>( GDALOpen( pszFilename, GA_Update ) );
+    CPLStringList aosOpenOptions;
+    aosOpenOptions.SetNameValue("NUM_THREADS",
+                                CSLFetchNameValue(papszOptions, "NUM_THREADS"));
+    aosOpenOptions.SetNameValue("SPARSE_OK",
+                                CSLFetchNameValue(papszOptions, "SPARSE_OK"));
+    aosOpenOptions.SetNameValue("@MASK_OVERVIEW_DATASET",
+                                CSLFetchNameValue(papszOptions, "MASK_OVERVIEW_DATASET"));
+    GDALDataset *hODS = GDALDataset::Open( pszFilename,
+                                           GDAL_OF_RASTER | GDAL_OF_UPDATE,
+                                           nullptr,
+                                           aosOpenOptions.List() );
     if( hODS == nullptr )
         return CE_Failure;
 
@@ -851,51 +965,72 @@ GTIFFBuildOverviews( const char * pszFilename,
 /* -------------------------------------------------------------------- */
     TIFF *hTIFF = static_cast<TIFF *>( hODS->GetInternalHandle(nullptr) );
 
-    if( nCompression == COMPRESSION_JPEG
-        && CPLGetConfigOption( "JPEG_QUALITY_OVERVIEW", nullptr ) != nullptr )
+    const char* pszJPEGQuality =
+        papszOptions ?
+            CSLFetchNameValue(papszOptions, "JPEG_QUALITY") :
+            CPLGetConfigOption( "JPEG_QUALITY_OVERVIEW", nullptr );
+    if( nCompression == COMPRESSION_JPEG && pszJPEGQuality != nullptr )
     {
-        const int nJpegQuality =
-            atoi(CPLGetConfigOption("JPEG_QUALITY_OVERVIEW","75"));
+        const int nJpegQuality = atoi(pszJPEGQuality);
         TIFFSetField( hTIFF, TIFFTAG_JPEGQUALITY,
                       nJpegQuality );
-        GTIFFSetJpegQuality((GDALDatasetH)hODS, nJpegQuality);
+        GTIFFSetJpegQuality(GDALDataset::ToHandle(hODS), nJpegQuality);
+    }
+    const char* pszWebpLevel =
+        papszOptions ?
+            CSLFetchNameValue(papszOptions, "WEBP_LEVEL") :
+            CPLGetConfigOption( "WEBP_LEVEL_OVERVIEW", nullptr );
+    if( nCompression == COMPRESSION_WEBP && pszWebpLevel != nullptr )
+    {
+        const int nWebpLevel = atoi(pszWebpLevel);
+        if ( nWebpLevel >= 1 )
+        {
+            TIFFSetField( hTIFF, TIFFTAG_WEBP_LEVEL, nWebpLevel );
+            GTIFFSetWebPLevel(GDALDataset::ToHandle(hODS), nWebpLevel);
+        }
     }
 
-    if( nCompression == COMPRESSION_JPEG
-        && CPLGetConfigOption( "JPEG_TABLESMODE_OVERVIEW", nullptr ) != nullptr )
+    const char* pszJPEGTablesMode =
+        papszOptions ?
+            CSLFetchNameValue(papszOptions, "JPEG_TABLESMODE") :
+            CPLGetConfigOption( "JPEG_TABLESMODE_OVERVIEW", nullptr );
+    if( nCompression == COMPRESSION_JPEG && pszJPEGTablesMode != nullptr )
     {
-        const int nJpegTablesMode =
-            atoi(CPLGetConfigOption("JPEG_TABLESMODE_OVERVIEW",
-                            CPLSPrintf("%d", knGTIFFJpegTablesModeDefault)));
+        const int nJpegTablesMode = atoi(pszJPEGTablesMode);
         TIFFSetField( hTIFF, TIFFTAG_JPEGTABLESMODE,
                       nJpegTablesMode );
-        GTIFFSetJpegTablesMode((GDALDatasetH)hODS, nJpegTablesMode);
+        GTIFFSetJpegTablesMode(GDALDataset::ToHandle(hODS), nJpegTablesMode);
     }
 
 /* -------------------------------------------------------------------- */
 /*      Loop writing overview data.                                     */
 /* -------------------------------------------------------------------- */
 
-    int *panOverviewListSorted =
-        static_cast<int*>(CPLMalloc(sizeof(int) * nOverviews));
-    memcpy( panOverviewListSorted, panOverviewList, sizeof(int) * nOverviews);
-    std::sort(panOverviewListSorted, panOverviewListSorted + nOverviews);
+    int *panOverviewListSorted = nullptr;
+    if( panOverviewList )
+    {
+        panOverviewListSorted = static_cast<int*>(CPLMalloc(sizeof(int) * nOverviews));
+        memcpy( panOverviewListSorted, panOverviewList, sizeof(int) * nOverviews);
+        std::sort(panOverviewListSorted, panOverviewListSorted + nOverviews);
+    }
 
     GTIFFSetInExternalOvr(true);
 
     CPLErr eErr = CE_None;
 
-    if( nCompression != COMPRESSION_NONE &&
-        nPlanarConfig == PLANARCONFIG_CONTIG &&
-        !GDALDataTypeIsComplex(papoBandList[0]->GetRasterDataType()) &&
-        papoBandList[0]->GetColorTable() == nullptr &&
-        (STARTS_WITH_CI(pszResampling, "NEAR") ||
-         EQUAL(pszResampling, "AVERAGE") ||
-         EQUAL(pszResampling, "GAUSS") ||
-         EQUAL(pszResampling, "CUBIC") ||
-         EQUAL(pszResampling, "CUBICSPLINE") ||
-         EQUAL(pszResampling, "LANCZOS") ||
-         EQUAL(pszResampling, "BILINEAR")))
+    if(  ((bSourceIsPixelInterleaved && bSourceIsJPEG2000) ||
+          (nCompression != COMPRESSION_NONE)) &&
+         nPlanarConfig == PLANARCONFIG_CONTIG &&
+         !GDALDataTypeIsComplex(papoBandList[0]->GetRasterDataType()) &&
+         papoBandList[0]->GetColorTable() == nullptr &&
+         (STARTS_WITH_CI(pszResampling, "NEAR") ||
+          EQUAL(pszResampling, "AVERAGE") ||
+          EQUAL(pszResampling, "RMS") ||
+          EQUAL(pszResampling, "GAUSS") ||
+          EQUAL(pszResampling, "CUBIC") ||
+          EQUAL(pszResampling, "CUBICSPLINE") ||
+          EQUAL(pszResampling, "LANCZOS") ||
+          EQUAL(pszResampling, "BILINEAR")) )
     {
         // In the case of pixel interleaved compressed overviews, we want to
         // generate the overviews for all the bands block by block, and not
@@ -916,12 +1051,20 @@ GTIFFBuildOverviews( const char * pszFilename,
             const double noDataValue = poSrcBand->GetNoDataValue(&bHasNoData);
             if( bHasNoData )
                 poDstBand->SetNoDataValue(noDataValue);
+            std::vector<bool> abDstOverviewAssigned(1 + poDstBand->GetOverviewCount());
 
             for( int i = 0; i < nOverviews && eErr == CE_None; i++ )
             {
+                const bool bDegenerateOverview =
+                    panOverviewListSorted != nullptr &&
+                    (poSrcBand->GetXSize() >> panOverviewListSorted[i]) == 0 &&
+                    (poSrcBand->GetYSize() >> panOverviewListSorted[i]) == 0;
+
                 for( int j = -1; j < poDstBand->GetOverviewCount() &&
                                  eErr == CE_None; j++ )
                 {
+                    if( abDstOverviewAssigned[1+j] )
+                        continue;
                     GDALRasterBand * poOverview =
                             (j < 0 ) ? poDstBand : poDstBand->GetOverview( j );
                     if( poOverview == nullptr )
@@ -930,34 +1073,61 @@ GTIFFBuildOverviews( const char * pszFilename,
                         continue;
                     }
 
-                    const int nOvFactor =
-                        GDALComputeOvFactor(poOverview->GetXSize(),
-                                            poSrcBand->GetXSize(),
-                                            poOverview->GetYSize(),
-                                            poSrcBand->GetYSize());
-
-                    if( nOvFactor == panOverviewListSorted[i]
-                        || nOvFactor == GDALOvLevelAdjust2(
-                                            panOverviewListSorted[i],
-                                            poSrcBand->GetXSize(),
-                                            poSrcBand->GetYSize() ) )
+                    bool bMatch;
+                    if( panOverviewListSorted )
                     {
+                        const int nOvFactor =
+                            GDALComputeOvFactor(poOverview->GetXSize(),
+                                                poSrcBand->GetXSize(),
+                                                poOverview->GetYSize(),
+                                                poSrcBand->GetYSize());
+
+                        bMatch = nOvFactor == panOverviewListSorted[i]
+                            || nOvFactor == GDALOvLevelAdjust2(
+                                                panOverviewListSorted[i],
+                                                poSrcBand->GetXSize(),
+                                                poSrcBand->GetYSize() )
+                            // Deal with edge cases where overview levels lead to
+                            // degenerate 1x1 overviews
+                            || (bDegenerateOverview &&
+                                poOverview->GetXSize() == 1 &&
+                                poOverview->GetYSize() == 1 );
+                    }
+                    else
+                    {
+                        bMatch = (
+                            // cppcheck-suppress nullPointer
+                            poOverview->GetXSize() == pasOverviewSize[i].first &&
+                            // cppcheck-suppress nullPointer
+                            poOverview->GetYSize() == pasOverviewSize[i].second );
+                    }
+                    if( bMatch )
+                    {
+                        abDstOverviewAssigned[j+1] = true;
                         papapoOverviewBands[iBand][i] = poOverview;
                         if( bHasNoData )
                             poOverview->SetNoDataValue(noDataValue);
                         break;
                     }
                 }
+
                 CPLAssert( papapoOverviewBands[iBand][i] != nullptr );
             }
         }
 
-        if( eErr == CE_None )
-            eErr =
-                GDALRegenerateOverviewsMultiBand(
-                    nBands, papoBandList,
-                    nOverviews, papapoOverviewBands,
-                    pszResampling, pfnProgress, pProgressData );
+        {
+            CPLConfigOptionSetter oSetter(
+                "GDAL_NUM_THREADS",
+                CSLFetchNameValue(papszOptions, "NUM_THREADS"),
+                true);
+
+            if( eErr == CE_None )
+                eErr =
+                    GDALRegenerateOverviewsMultiBand(
+                        nBands, papoBandList,
+                        nOverviews, papapoOverviewBands,
+                        pszResampling, pfnProgress, pProgressData );
+        }
 
         for( int iBand = 0; iBand < nBands; iBand++ )
         {
@@ -1010,15 +1180,23 @@ GTIFFBuildOverviews( const char * pszFilename,
                     (iBand + 1) / static_cast<double>( nBands ),
                     pfnProgress, pProgressData );
 
-            if( eErr == CE_None )
-                eErr =
-                    GDALRegenerateOverviews(
-                        hSrcBand,
-                        nDstOverviews,
-                        reinterpret_cast<GDALRasterBandH *>( papoOverviews ),
-                        pszResampling,
-                        GDALScaledProgress,
-                        pScaledProgressData );
+
+            {
+                CPLConfigOptionSetter oSetter(
+                    "GDAL_NUM_THREADS",
+                    CSLFetchNameValue(papszOptions, "NUM_THREADS"),
+                    true);
+
+                if( eErr == CE_None )
+                    eErr =
+                        GDALRegenerateOverviews(
+                            hSrcBand,
+                            nDstOverviews,
+                            reinterpret_cast<GDALRasterBandH *>( papoOverviews ),
+                            pszResampling,
+                            GDALScaledProgress,
+                            pScaledProgressData );
+            }
 
             GDALDestroyScaledProgress( pScaledProgressData );
         }
