@@ -48,7 +48,7 @@ LT_USE_LIDAR_NAMESPACE
 #include "gdal_pam.h"
 // #include "gdal_alg.h" // 1.6 and later have gridding algorithms
 
-CPL_CVSID("$Id: gdal_MG4Lidar.cpp e13dcd4dc171dfeed63f912ba06b9374ce4f3bb2 2018-03-18 21:37:41Z Even Rouault $")
+CPL_CVSID("$Id: gdal_MG4Lidar.cpp 2dc350b1c738a002da1ec5c4cbe3e42fec4069dc 2021-03-04 16:15:13 +0100 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -77,7 +77,7 @@ IMPLEMENT_OBJECT_CREATE(CropableMG4PointReader)
 
 static double MaxRasterSize = 2048.0;
 static double MaxBlockSideSize = 1024.0;
-class MG4LidarDataset : public GDALPamDataset
+class MG4LidarDataset final: public GDALPamDataset
 {
 friend class MG4LidarRasterBand;
 
@@ -86,7 +86,10 @@ public:
    ~MG4LidarDataset();
    static GDALDataset *Open( GDALOpenInfo * );
    CPLErr         GetGeoTransform( double * padfTransform ) override;
-   const char *GetProjectionRef() override;
+   const char *_GetProjectionRef() override;
+    const OGRSpatialReference* GetSpatialRef() const override {
+        return GetSpatialRefFromOldGetProjectionRef();
+    }
 
 protected:
    MG4PointReader *reader;
@@ -106,7 +109,7 @@ protected:
 /*                            MG4LidarRasterBand                                          */
 /* ========================================  */
 
-class MG4LidarRasterBand : public GDALPamRasterBand
+class MG4LidarRasterBand final: public GDALPamRasterBand
 {
    friend class MG4LidarDataset;
 
@@ -577,7 +580,7 @@ CPLErr MG4LidarDataset::GetGeoTransform( double * padfTransform )
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *MG4LidarDataset::GetProjectionRef()
+const char *MG4LidarDataset::_GetProjectionRef()
 
 {
    const char * wkt = CPLGetXMLValue(poXMLPCView, "GeoReference", nullptr);
@@ -724,6 +727,9 @@ GDALDataset *MG4LidarDataset::Open( GDALOpenInfo * poOpenInfo )
           return nullptr;
    }
 
+  if( !GDALIsDriverDeprecatedForGDAL35StillEnabled("MG4LIDAR") )
+      return nullptr;
+
    CPLXMLNode *psInputFile = CPLGetXMLNode( pxmlPCView, "InputFile" );
    if( psInputFile == nullptr )
    {
@@ -924,7 +930,7 @@ void GDALRegister_MG4Lidar()
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                "MrSID Generation 4 / Lidar (.sid)" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,  "frmt_mrsid_lidar.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,  "drivers/raster/mg4lidar.html" );
 
     poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "view" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Float64" );
