@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2006, Waypoint Information Technology
- * Copyright (c) 2009-2012, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009-2012, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -41,11 +41,11 @@
 #include "../../ogr/ogrsf_frmts/mitab/mitab.h"
 #endif
 
-CPL_CVSID("$Id: grddataset.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
+CPL_CVSID("$Id: grddataset.cpp edff0f6ff69efb760c7e123f7ca2e2b2038d3a7e 2021-03-27 10:49:34 +0100 Even Rouault $")
 
-#define NODATA -1.e37f
-#define SCALE16BIT 65534.0
-#define SCALE32BIT 4294967294.0
+constexpr float NODATA = -1.e37f;
+constexpr double SCALE16BIT = 65534.0;
+constexpr double SCALE32BIT = 4294967294.0;
 
 void replaceExt(std::string& s, const std::string& newExt);
 /************************************************************************/
@@ -67,7 +67,7 @@ void replaceExt(std::string& s, const std::string& newExt) {
 /************************************************************************/
 class NWT_GRDRasterBand;
 
-class NWT_GRDDataset: public GDALPamDataset {
+class NWT_GRDDataset final: public GDALPamDataset {
     friend class NWT_GRDRasterBand;
 
     VSILFILE *fp;
@@ -81,6 +81,9 @@ class NWT_GRDDataset: public GDALPamDataset {
     int UpdateHeader();
     int WriteTab();
 
+    NWT_GRDDataset(const NWT_GRDDataset&) = delete;
+    NWT_GRDDataset& operator= (const NWT_GRDDataset&) = delete;
+
 public:
     NWT_GRDDataset();
     ~NWT_GRDDataset();
@@ -88,7 +91,7 @@ public:
     static GDALDataset *Open(GDALOpenInfo *);
     static int Identify(GDALOpenInfo *);
     static GDALDataset *Create(const char * pszFilename, int nXSize, int nYSize,
-            int nBands, GDALDataType eType, char ** papszParmList);
+            int nBands, GDALDataType eType, char ** papszParamList);
     static GDALDataset *CreateCopy(const char * pszFilename,
             GDALDataset * poSrcDS, int bStrict, char **papszOptions,
             GDALProgressFunc pfnProgress, void * pProgressData);
@@ -108,7 +111,7 @@ public:
 /* ==================================================================== */
 /************************************************************************/
 
-class NWT_GRDRasterBand: public GDALPamRasterBand {
+class NWT_GRDRasterBand final: public GDALPamRasterBand {
     friend class NWT_GRDDataset;
 
     int bHaveOffsetScale;
@@ -153,11 +156,11 @@ NWT_GRDRasterBand::NWT_GRDRasterBand( NWT_GRDDataset * poDSIn, int nBandIn,
         if (poDSIn->pGrd->cFormat == 0x00) {
             eDataType = GDT_Float32;
             dfScale = (poDSIn->pGrd->fZMax - poDSIn->pGrd->fZMin)
-                    / (double) SCALE16BIT;
+                    / SCALE16BIT;
         } else {
             eDataType = GDT_Float32;
             dfScale = (poDSIn->pGrd->fZMax - poDSIn->pGrd->fZMin)
-                    / (double) SCALE32BIT;
+                    / SCALE32BIT;
         }
     }
     else
@@ -172,7 +175,7 @@ NWT_GRDRasterBand::NWT_GRDRasterBand( NWT_GRDDataset * poDSIn, int nBandIn,
 }
 
 double NWT_GRDRasterBand::GetNoDataValue(int *pbSuccess) {
-    NWT_GRDDataset *poGDS = reinterpret_cast<NWT_GRDDataset *>(poDS);
+    NWT_GRDDataset *poGDS = cpl::down_cast<NWT_GRDDataset *>(poDS);
     double dRetval;
     if ((nBand == 4) || (poGDS->nBands == 1)) {
         if (pbSuccess != nullptr)
@@ -180,7 +183,7 @@ double NWT_GRDRasterBand::GetNoDataValue(int *pbSuccess) {
         if (dfNoData != 0.0) {
             dRetval = dfNoData;
         } else {
-            dRetval = (double) NODATA;
+            dRetval = NODATA;
         }
 
         return dRetval;
@@ -203,7 +206,7 @@ CPLErr NWT_GRDRasterBand::SetNoDataValue(double dfNoDataIn) {
 }
 
 GDALColorInterp NWT_GRDRasterBand::GetColorInterpretation() {
-    NWT_GRDDataset *poGDS = reinterpret_cast<NWT_GRDDataset *>(poDS);
+    NWT_GRDDataset *poGDS = cpl::down_cast<NWT_GRDDataset *>(poDS);
     //return GCI_RGB;
     if ((nBand == 4) || (poGDS->nBands == 1))
         return GCI_GrayIndex;
@@ -225,13 +228,13 @@ CPLErr NWT_GRDRasterBand::IWriteBlock(CPL_UNUSED int nBlockXOff, int nBlockYOff,
 
     // Each block is an entire row of the dataset, so the x offset should always be 0
     CPLAssert(nBlockXOff == 0);
-    NWT_GRDDataset *poGDS = reinterpret_cast<NWT_GRDDataset *>(poDS);
+    NWT_GRDDataset *poGDS = cpl::down_cast<NWT_GRDDataset *>(poDS);
 
     if( dfScale == 0.0 )
         return CE_Failure;
 
     // Ensure the blocksize is not beyond the system limits and
-    // initialise the size of the record
+    // initialize the size of the record
     if (nBlockXSize > INT_MAX / 2) {
         return CE_Failure;
     }
@@ -243,30 +246,29 @@ CPLErr NWT_GRDRasterBand::IWriteBlock(CPL_UNUSED int nBlockXOff, int nBlockYOff,
             SEEK_SET);
 
     // Cast pImage to float
-    float *pfImage = reinterpret_cast<float *>(pImage);
+    const float *pfImage = static_cast<const float *>(pImage);
 
-    // Initialise output array
-    GByte *pabyRecord = reinterpret_cast<GByte *>(VSI_MALLOC_VERBOSE(
+    // Initialize output array
+    GByte *pabyRecord = static_cast<GByte *>(VSI_MALLOC_VERBOSE(
                     nRecordSize));
     if (pabyRecord == nullptr)
-    return CE_Failure;
+        return CE_Failure;
 
     // We only ever write to band 4; RGB bands are basically 'virtual'
     // (i.e. the RGB colour is computed from the raw data).
     // For all intents and purposes, there is essentially 1 band on disk.
-    float fValue;// A single pixel value
-    unsigned short nWrite;// The stretched value to be written
     if (nBand == 1) {
         for (int i = 0; i < nBlockXSize; i++) {
-            fValue = pfImage[i];
+            const float fValue = pfImage[i];
+            unsigned short nWrite;// The stretched value to be written
 
             // We allow data to be interpreted by a user-defined null value
             // (a 'virtual' value, since it is always 0 on disk) or
             // if not defined we default to the GRD standard of -1E37.
             // We allow a little bit of flexibility in that if it is below -1E37
             // it is in all probability still intended as a null value.
-            if ((fValue == dfNoData) || (fValue <= (double)NODATA)) {
-                nWrite = (unsigned short) 0;
+            if ((fValue == dfNoData) || (fValue <= NODATA)) {
+                nWrite = 0;
             }
             else {
                 if (fValue < poGDS->pGrd->fZMin) {
@@ -278,16 +280,16 @@ CPLErr NWT_GRDRasterBand::IWriteBlock(CPL_UNUSED int nBlockXOff, int nBlockYOff,
                 // Data on disk is stretched within the unsigned short range so
                 // we must convert (the inverse of what is done in IReadBlock),
                 // based on the Z value range
-                nWrite = (unsigned short) (((fValue - dfOffset) / dfScale) + 1);
+                nWrite = static_cast<unsigned short>(((fValue - dfOffset) / dfScale) + 1);
             }
+            CPL_LSBPTR16(&nWrite);
             // Copy the result to the byte array (2 bytes per value)
-            memcpy(reinterpret_cast<void *>(pabyRecord + 2 * i),
-                    reinterpret_cast<void *>(&nWrite), 2);
+            memcpy(pabyRecord + 2 * i, &nWrite, 2);
         }
 
         // Write the buffer to disk
         if (VSIFWriteL(pabyRecord, 1, nRecordSize, poGDS->fp)
-                != (size_t) (nRecordSize)) {
+                != static_cast<size_t>(nRecordSize)) {
             CPLError(CE_Failure, CPLE_FileIO,
                     "Failed to write scanline %d to file.\n", nBlockYOff);
             CPLFree(pabyRecord);
@@ -308,24 +310,23 @@ CPLErr NWT_GRDRasterBand::IWriteBlock(CPL_UNUSED int nBlockXOff, int nBlockYOff,
 /************************************************************************/
 CPLErr NWT_GRDRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
         void *pImage) {
-    NWT_GRDDataset *poGDS = reinterpret_cast<NWT_GRDDataset *>(poDS);
+    NWT_GRDDataset *poGDS = cpl::down_cast<NWT_GRDDataset *>(poDS);
     if (nBlockXSize > INT_MAX / 2)
         return CE_Failure;
     const int nRecordSize = nBlockXSize * 2;
-    unsigned short raw1;
 
     // Seek to the data position
     VSIFSeekL(poGDS->fp,
             1024 + nRecordSize * static_cast<vsi_l_offset>(nBlockYOff),
             SEEK_SET);
 
-    GByte *pabyRecord = reinterpret_cast<GByte *>(VSI_MALLOC_VERBOSE(
+    GByte *pabyRecord = static_cast<GByte *>(VSI_MALLOC_VERBOSE(
                     nRecordSize));
     if (pabyRecord == nullptr)
-    return CE_Failure;
+        return CE_Failure;
 
     // Read the data
-    if ((int) VSIFReadL(pabyRecord, 1, nRecordSize, poGDS->fp) != nRecordSize) {
+    if (static_cast<int>(VSIFReadL(pabyRecord, 1, nRecordSize, poGDS->fp)) != nRecordSize) {
         CPLFree(pabyRecord);
         return CE_Failure;
     }
@@ -333,41 +334,41 @@ CPLErr NWT_GRDRasterBand::IReadBlock( CPL_UNUSED int nBlockXOff, int nBlockYOff,
     if ((nBand == 4) || (poGDS->nBands == 1))            //Z values
     {
         int bSuccess;
-        double dNoData = GetNoDataValue(&bSuccess);
+        const float fNoData = static_cast<float>(GetNoDataValue(&bSuccess));
         for (int i = 0; i < nBlockXSize; i++) {
-            memcpy(reinterpret_cast<void *>(&raw1),
-                    reinterpret_cast<void *>(pabyRecord + 2 * i), 2);
+            unsigned short raw1;
+            memcpy(&raw1, pabyRecord + 2 * i, 2);
             CPL_LSBPTR16(&raw1);
             if (raw1 == 0) {
-                reinterpret_cast<float *>(pImage)[i] = (float) dNoData; // null value
+                static_cast<float *>(pImage)[i] = fNoData; // null value
             } else {
-                reinterpret_cast<float *>(pImage)[i] =
-                static_cast<float>(dfOffset + ((raw1 - 1) * dfScale));
+                static_cast<float *>(pImage)[i] =
+                    static_cast<float>(dfOffset + ((raw1 - 1) * dfScale));
             }
         }
     } else if (nBand == 1)            // red values
     {
         for (int i = 0; i < nBlockXSize; i++) {
-            memcpy(reinterpret_cast<void *>(&raw1),
-                    reinterpret_cast<void *>(pabyRecord + 2 * i), 2);
+            unsigned short raw1;
+            memcpy(&raw1, pabyRecord + 2 * i, 2);
             CPL_LSBPTR16(&raw1);
-            reinterpret_cast<char *>(pImage)[i] = poGDS->ColorMap[raw1 / 16].r;
+            static_cast<unsigned char *>(pImage)[i] = poGDS->ColorMap[raw1 / 16].r;
         }
     } else if (nBand == 2)            // green
     {
         for (int i = 0; i < nBlockXSize; i++) {
-            memcpy(reinterpret_cast<void *>(&raw1),
-                    reinterpret_cast<void *>(pabyRecord + 2 * i), 2);
+            unsigned short raw1;
+            memcpy(&raw1, pabyRecord + 2 * i, 2);
             CPL_LSBPTR16(&raw1);
-            reinterpret_cast<char *>(pImage)[i] = poGDS->ColorMap[raw1 / 16].g;
+            static_cast<unsigned char *>(pImage)[i] = poGDS->ColorMap[raw1 / 16].g;
         }
     } else if (nBand == 3)            // blue
     {
         for (int i = 0; i < nBlockXSize; i++) {
-            memcpy(reinterpret_cast<void *>(&raw1),
-                    reinterpret_cast<void *>(pabyRecord + 2 * i), 2);
+            unsigned short raw1;
+            memcpy(&raw1, pabyRecord + 2 * i, 2);
             CPL_LSBPTR16(&raw1);
-            reinterpret_cast<char *>(pImage)[i] = poGDS->ColorMap[raw1 / 16].b;
+            static_cast<unsigned char *>(pImage)[i] = poGDS->ColorMap[raw1 / 16].b;
         }
     } else {
         CPLError(CE_Failure, CPLE_IllegalArg, "No band number %d", nBand);
@@ -409,7 +410,7 @@ NWT_GRDDataset::~NWT_GRDDataset() {
     // Make sure any changes to the header etc are written
     // if we are in update mode.
     if (eAccess == GA_Update) {
-        FlushCache();
+        NWT_GRDDataset::FlushCache();
     }
     pGrd->fp = nullptr;       // this prevents nwtCloseGrid from closing the fp
     nwtCloseGrid(pGrd);
@@ -576,7 +577,7 @@ GDALDataset *NWT_GRDDataset::Open(GDALOpenInfo * poOpenInfo) {
 
     poDS->pGrd->fp = poDS->fp;
 
-    if (!nwt_ParseHeader(poDS->pGrd, reinterpret_cast<char *>(poDS->abyHeader))
+    if (!nwt_ParseHeader(poDS->pGrd, poDS->abyHeader)
             || !GDALCheckDatasetDimensions(poDS->pGrd->nXSide,
                     poDS->pGrd->nYSide)) {
         delete poDS;
@@ -626,7 +627,7 @@ int NWT_GRDDataset::UpdateHeader() {
     poHeaderBlock->InitNewBlock(fp, 1024);
 
     // Write the header string
-    poHeaderBlock->WriteBytes(5, (GByte *) "HGPC1\0");
+    poHeaderBlock->WriteBytes(5, reinterpret_cast<const GByte *>("HGPC1\0"));
 
     // Version number
     poHeaderBlock->WriteFloat(pGrd->fVersion);
@@ -649,12 +650,12 @@ int NWT_GRDDataset::UpdateHeader() {
 
     // Description String
     int nChar = static_cast<int>(strlen(pGrd->cDescription));
-    poHeaderBlock->WriteBytes(nChar, (GByte*) pGrd->cDescription);
+    poHeaderBlock->WriteBytes(nChar, reinterpret_cast<const GByte*>(pGrd->cDescription));
     poHeaderBlock->WriteZeros(32 - nChar);
 
     // Unit Name String
     nChar = static_cast<int>(strlen(pGrd->cZUnits));
-    poHeaderBlock->WriteBytes(nChar, (GByte *) pGrd->cZUnits);
+    poHeaderBlock->WriteBytes(nChar, reinterpret_cast<const GByte*>(pGrd->cZUnits));
     poHeaderBlock->WriteZeros(32 - nChar);
 
     //Ignore 126 - 141 as unknown usage
@@ -662,7 +663,7 @@ int NWT_GRDDataset::UpdateHeader() {
 
     // Hill shading
     poHeaderBlock->WriteInt16(pGrd->bHillShadeExists ? 1 : 0);
-    poHeaderBlock->WriteInt16((short) 0);
+    poHeaderBlock->WriteInt16(0);
 
     poHeaderBlock->WriteByte(pGrd->cHillShadeBrightness);
     poHeaderBlock->WriteByte(pGrd->cHillShadeContrast);
@@ -671,9 +672,9 @@ int NWT_GRDDataset::UpdateHeader() {
     poHeaderBlock->WriteZeros(110);
 
     // Write spatial reference
-    poHeaderBlock->WriteBytes((int) strlen(pGrd->cMICoordSys),
-            (GByte*) pGrd->cMICoordSys);
-    poHeaderBlock->WriteZeros(256 - (int) strlen(pGrd->cMICoordSys));
+    poHeaderBlock->WriteBytes(static_cast<int>(strlen(pGrd->cMICoordSys)),
+            reinterpret_cast<const GByte*>(pGrd->cMICoordSys));
+    poHeaderBlock->WriteZeros(256 - static_cast<int>(strlen(pGrd->cMICoordSys)));
 
     // Unit code
     poHeaderBlock->WriteByte(static_cast<GByte>(pGrd->iZUnits));
@@ -688,7 +689,7 @@ int NWT_GRDDataset::UpdateHeader() {
     }
 
     poHeaderBlock->WriteByte(byDisplayStatus);
-    poHeaderBlock->WriteInt16((short) 0); //Data Type?
+    poHeaderBlock->WriteInt16(0); //Data Type?
 
     // Colour inflections
     poHeaderBlock->WriteInt16(pGrd->iNumColorInflections);
@@ -816,7 +817,7 @@ int NWT_GRDDataset::WriteTab() {
 /*                                Create()                              */
 /************************************************************************/
 GDALDataset *NWT_GRDDataset::Create(const char * pszFilename, int nXSize,
-        int nYSize, int nBands, GDALDataType eType, char ** papszParmList) {
+        int nYSize, int nBands, GDALDataType eType, char ** papszParamList) {
     if (nBands != 1) {
         CPLError(CE_Failure, CPLE_FileIO,
                 "Only single band datasets are supported for writing");
@@ -829,9 +830,9 @@ GDALDataset *NWT_GRDDataset::Create(const char * pszFilename, int nXSize,
     }
     NWT_GRDDataset *poDS = new NWT_GRDDataset();
     poDS->eAccess = GA_Update;
-    poDS->pGrd = reinterpret_cast<NWT_GRID *>(calloc(1, sizeof(NWT_GRID)));
+    poDS->pGrd = static_cast<NWT_GRID *>(calloc(1, sizeof(NWT_GRID)));
 
-    // We currently only support GRD grid types (could potentially support GRC in the papszParmList).
+    // We currently only support GRD grid types (could potentially support GRC in the papszParamList).
     // Also only support GDT_Float32 as the data type. GRD format allows for data to be stretched to
     // 32bit or 16bit integers on disk, so it would be feasible to support other data types
     poDS->pGrd->cFormat = 0x00;
@@ -854,16 +855,16 @@ GDALDataset *NWT_GRDDataset::Create(const char * pszFilename, int nXSize,
 
     float fZMin, fZMax;
     // See if the user passed the min/max values
-    if (CSLFetchNameValue(papszParmList, "ZMIN") == nullptr) {
-        fZMin = (float) -2E+37;
+    if (CSLFetchNameValue(papszParamList, "ZMIN") == nullptr) {
+        fZMin = static_cast<float>(-2E+37);
     } else {
-        fZMin = static_cast<float>(CPLAtof(CSLFetchNameValue(papszParmList, "ZMIN")));
+        fZMin = static_cast<float>(CPLAtof(CSLFetchNameValue(papszParamList, "ZMIN")));
     }
 
-    if (CSLFetchNameValue(papszParmList, "ZMAX") == nullptr) {
-        fZMax = (float) 2E+38;
+    if (CSLFetchNameValue(papszParamList, "ZMAX") == nullptr) {
+        fZMax = static_cast<float>(2E+38);
     } else {
-        fZMax = static_cast<float>(CPLAtof(CSLFetchNameValue(papszParmList, "ZMAX")));
+        fZMax = static_cast<float>(CPLAtof(CSLFetchNameValue(papszParamList, "ZMAX")));
     }
 
     poDS->pGrd->fZMin = fZMin;
@@ -906,32 +907,32 @@ GDALDataset *NWT_GRDDataset::Create(const char * pszFilename, int nXSize,
     poDS->pGrd->fHillShadeAngle = 0;
 
     // Set the raster style settings. These aren't used anywhere other than to write the TAB file
-    if (CSLFetchNameValue(papszParmList, "BRIGHTNESS") == nullptr) {
+    if (CSLFetchNameValue(papszParamList, "BRIGHTNESS") == nullptr) {
         poDS->pGrd->style.iBrightness = 50;
     } else {
         poDS->pGrd->style.iBrightness = atoi(
-                CSLFetchNameValue(papszParmList, "BRIGHTNESS"));
+                CSLFetchNameValue(papszParamList, "BRIGHTNESS"));
     }
 
-    if (CSLFetchNameValue(papszParmList, "CONTRAST") == nullptr) {
+    if (CSLFetchNameValue(papszParamList, "CONTRAST") == nullptr) {
         poDS->pGrd->style.iContrast = 50;
     } else {
         poDS->pGrd->style.iContrast = atoi(
-                CSLFetchNameValue(papszParmList, "CONTRAST"));
+                CSLFetchNameValue(papszParamList, "CONTRAST"));
     }
 
-    if (CSLFetchNameValue(papszParmList, "TRANSCOLOR") == nullptr) {
+    if (CSLFetchNameValue(papszParamList, "TRANSCOLOR") == nullptr) {
         poDS->pGrd->style.iTransColour = 0;
     } else {
         poDS->pGrd->style.iTransColour = atoi(
-                CSLFetchNameValue(papszParmList, "TRANSCOLOR"));
+                CSLFetchNameValue(papszParamList, "TRANSCOLOR"));
     }
 
-    if (CSLFetchNameValue(papszParmList, "TRANSLUCENCY") == nullptr) {
+    if (CSLFetchNameValue(papszParamList, "TRANSLUCENCY") == nullptr) {
         poDS->pGrd->style.iTranslucency = 0;
     } else {
         poDS->pGrd->style.iTranslucency = atoi(
-                CSLFetchNameValue(papszParmList, "TRANSLUCENCY"));
+                CSLFetchNameValue(papszParamList, "TRANSLUCENCY"));
     }
 
     poDS->pGrd->style.bGreyscale = FALSE;
@@ -1014,7 +1015,7 @@ GDALDataset * NWT_GRDDataset::CreateCopy(const char * pszFilename,
         tmpOptions = CSLSetNameValue(tmpOptions, "ZMIN", sMin);
     }
 
-    GDALDriver *poDriver = (GDALDriver *) GDALGetDriverByName("NWT_GRD");
+    GDALDriver *poDriver = GDALDriver::FromHandle(GDALGetDriverByName("NWT_GRD"));
     GDALDataset *poDstDS = poDriver->DefaultCreateCopy(pszFilename, poSrcDS,
             bStrict, tmpOptions, pfnProgress, pProgressData);
 
@@ -1036,7 +1037,7 @@ void GDALRegister_NWT_GRD() {
     poDriver->SetMetadataItem(GDAL_DCAP_RASTER, "YES");
     poDriver->SetMetadataItem(GDAL_DMD_LONGNAME,
             "Northwood Numeric Grid Format .grd/.tab");
-    poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "frmt_nwtgrd.html");
+    poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "drivers/raster/nwtgrd.html");
     poDriver->SetMetadataItem(GDAL_DMD_EXTENSION, "grd");
     poDriver->SetMetadataItem(GDAL_DCAP_VIRTUALIO, "YES");
     poDriver->SetMetadataItem(GDAL_DMD_CREATIONDATATYPES, "Float32");
