@@ -2,10 +2,10 @@
  *
  * Project:  PDF driver
  * Purpose:  GDALDataset driver for PDF dataset (read vector features)
- * Author:   Even Rouault, <even dot rouault at mines dash paris dot org>
+ * Author:   Even Rouault, <even dot rouault at spatialys.com>
  *
  ******************************************************************************
- * Copyright (c) 2010-2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,10 +28,12 @@
 
 #include "gdal_pdf.h"
 
+#include <array>
+
 #define SQUARE(x) ((x)*(x))
 #define EPSILON 1e-5
 
-CPL_CVSID("$Id: pdfreadvectors.cpp 5b629d218bb9b280261daf8f4d1b22f904729505 2019-10-19 19:57:30 +0200 Even Rouault $")
+CPL_CVSID("$Id: pdfreadvectors.cpp 6a73451ff0b40272a30aa9470d5493f6970ab096 2021-03-28 15:28:29 +0200 Even Rouault $")
 
 #ifdef HAVE_PDF_READ_SUPPORT
 
@@ -79,7 +81,10 @@ int PDFDataset::OpenVectorLayers(GDALPDFDictionary* poPageDict)
     }
     else
     {
-        ExploreContents(poContents, poResources);
+        int nDepth = 0;
+        int nVisited = 0;
+        bool bStop = false;
+        ExploreContents(poContents, poResources, nDepth, nVisited, bStop);
         std::set< std::pair<int,int> > aoSetAlreadyVisited;
         ExploreTree(poStructTreeRoot, aoSetAlreadyVisited, 0);
     }
@@ -348,9 +353,9 @@ OGRGeometry* PDFDataset::GetGeometryFromMCID(int nMCID)
 class GraphicState
 {
     public:
-        double adfCM[6];
-        double adfStrokeColor[3];
-        double adfFillColor[3];
+        std::array<double,6> adfCM;
+        std::array<double,3> adfStrokeColor;
+        std::array<double,3> adfFillColor;
 
         GraphicState()
         {
@@ -513,6 +518,8 @@ static OGRPoint* PDFGetStarCenter(OGRLineString* poLS)
     double dfSqD13 = SQUARE(poLS->getX(1) - poLS->getX(3)) +
                       SQUARE(poLS->getY(1) - poLS->getY(3));
     const double dfSin18divSin126 = 0.38196601125;
+    if( dfSqD02 == 0 )
+        return nullptr;
     int bOK = fabs(dfSqD13 / dfSqD02 - SQUARE(dfSin18divSin126)) < EPSILON;
     for(int i=1;i<10 && bOK;i++)
     {
@@ -1212,7 +1219,7 @@ OGRGeometry* PDFDataset::ParseContent(const char* pszContent,
                 }
                 else if( EQUAL2(szToken, "RG") || EQUAL2(szToken, "rg") )
                 {
-                    double* padf = ( EQUAL2(szToken, "RG") ) ? oGS.adfStrokeColor : oGS.adfFillColor;
+                    double* padf = ( EQUAL2(szToken, "RG") ) ? &oGS.adfStrokeColor[0] : &oGS.adfFillColor[0];
                     if (!UnstackTokens(szToken, 3, aszTokenStack, nTokenStackSize, padf))
                     {
                         CPLDebug("PDF", "Should not happen at line %d", __LINE__);
@@ -1407,8 +1414,8 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
 
         if (poCenter == nullptr && poMLS != nullptr && poMLS->getNumGeometries() == 2)
         {
-            OGRLineString* poLS1 = poMLS->getGeometryRef(0)->toLineString();
-            OGRLineString* poLS2 = poMLS->getGeometryRef(1)->toLineString();
+            const OGRLineString* poLS1 = poMLS->getGeometryRef(0);
+            const OGRLineString* poLS2 = poMLS->getGeometryRef(1);
 
             // Recognize points as written by GDAL (ogr-sym-0: cross (+) ).
             if (poLS1->getNumPoints() == 2 && poLS2->getNumPoints() == 2 &&
@@ -1468,14 +1475,14 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
                 {
                     poLS->closeRings();
 
-                    OGRPoint* poCenter = nullptr;
+                    std::unique_ptr<OGRPoint> poCenter;
 
                    if (nPolys == 0 &&
                         poLS &&
                         poLS->getNumPoints() == 1 + BEZIER_STEPS * 4 )
                    {
                         // Recognize points as written by GDAL (ogr-sym-3 : circle (filled))
-                        poCenter = PDFGetCircleCenter(poLS);
+                        poCenter.reset(PDFGetCircleCenter(poLS));
                    }
 
                     if (nPolys == 0 &&
@@ -1484,7 +1491,7 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
                         poLS->getNumPoints() == 5)
                     {
                         // Recognize points as written by GDAL (ogr-sym-5: square (filled))
-                        poCenter = PDFGetSquareCenter(poLS);
+                        poCenter.reset(PDFGetSquareCenter(poLS));
 
                         /* ESRI points */
                         if (poCenter == nullptr &&
@@ -1494,8 +1501,8 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
                             poLS->getY(2) == poLS->getY(3) &&
                             poLS->getX(3) == poLS->getX(0))
                         {
-                            poCenter = new OGRPoint((poLS->getX(0) + poLS->getX(1)) / 2,
-                                                    (poLS->getY(0) + poLS->getY(2)) / 2);
+                            poCenter.reset(new OGRPoint((poLS->getX(0) + poLS->getX(1)) / 2,
+                                                    (poLS->getY(0) + poLS->getY(2)) / 2));
                         }
                     }
                     // Recognize points as written by GDAL (ogr-sym-7: triangle (filled))
@@ -1503,19 +1510,20 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
                              poLS &&
                              poLS->getNumPoints() == 4)
                     {
-                        poCenter = PDFGetTriangleCenter(poLS);
+                        poCenter.reset(PDFGetTriangleCenter(poLS));
                     }
                     // Recognize points as written by GDAL (ogr-sym-9: star (filled))
                     else if (nPolys == 0 &&
                              poLS &&
                              poLS->getNumPoints() == 11)
                     {
-                        poCenter = PDFGetStarCenter(poLS);
+                        poCenter.reset(PDFGetStarCenter(poLS));
                     }
 
                     if (poCenter)
                     {
-                        poGeom = poCenter;
+                        delete poGeom;
+                        poGeom = poCenter.release();
                         break;
                     }
 
@@ -1597,9 +1605,21 @@ OGRGeometry* PDFDataset::BuildGeometry(std::vector<double>& oCoords,
 /************************************************************************/
 
 void PDFDataset::ExploreContents(GDALPDFObject* poObj,
-                                       GDALPDFObject* poResources)
+                                 GDALPDFObject* poResources,
+                                 int nDepth,
+                                 int& nVisited,
+                                 bool& bStop)
 {
     std::map<CPLString, OGRPDFLayer*> oMapPropertyToLayer;
+    if( nDepth == 10 || nVisited == 1000 )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "ExploreContents(): too deep exploration or too many items");
+        bStop = true;
+        return;
+    }
+    if( bStop )
+        return;
 
     if (poObj->GetType() == PDFObjectType_Array)
     {
@@ -1609,7 +1629,10 @@ void PDFDataset::ExploreContents(GDALPDFObject* poObj,
             GDALPDFObject* poSubObj = poArray->Get(i);
             if( poSubObj )
             {
-                ExploreContents(poSubObj, poResources);
+                nVisited ++;
+                ExploreContents(poSubObj, poResources, nDepth + 1, nVisited, bStop);
+                if( bStop )
+                    return;
             }
         }
     }
@@ -1739,25 +1762,10 @@ void PDFDataset::ExploreContentsNonStructured(GDALPDFObject* poContents,
         if (poProperties != nullptr &&
             poProperties->GetType() == PDFObjectType_Dictionary)
         {
-            char** papszLayersWithRef = osLayerWithRefList.List();
-            char** papszIter = papszLayersWithRef;
             std::map< std::pair<int, int>, OGRPDFLayer *> oMapNumGenToLayer;
-            while(papszIter && *papszIter)
+            for(const auto& oLayerWithref: aoLayerWithRef )
             {
-                char** papszTokens = CSLTokenizeString(*papszIter);
-
-                if( CSLCount(papszTokens) != 3 ) {
-                    CSLDestroy(papszTokens);
-                    CPLDebug("PDF", "Ignore '%s', unparsable.", *papszIter);
-                    papszIter ++;
-                    continue;
-                }
-
-                const char* pszLayerName = papszTokens[0];
-                int nNum = atoi(papszTokens[1]);
-                int nGen = atoi(papszTokens[2]);
-
-                CPLString osSanitizedName(PDFSanitizeLayerName(pszLayerName));
+                CPLString osSanitizedName(PDFSanitizeLayerName(oLayerWithref.osName));
 
                 OGRPDFLayer* poLayer = (OGRPDFLayer*) GetLayerByName(osSanitizedName.c_str());
                 if (poLayer == nullptr)
@@ -1775,10 +1783,7 @@ void PDFDataset::ExploreContentsNonStructured(GDALPDFObject* poContents,
                     nLayers ++;
                 }
 
-                oMapNumGenToLayer[ std::pair<int,int>(nNum, nGen) ] = poLayer;
-
-                CSLDestroy(papszTokens);
-                papszIter ++;
+                oMapNumGenToLayer[ std::pair<int,int>(oLayerWithref.nOCGNum.toInt(), oLayerWithref.nOCGGen) ] = poLayer;
             }
 
             std::map<CPLString, GDALPDFObject*>& oMap =
