@@ -38,7 +38,10 @@
 
 #include <cstdlib>
 
-CPL_CVSID("$Id: byndataset.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
+CPL_CVSID("$Id: byndataset.cpp 026ccc0bd255d04c0ece5f86190db8f992010531 2020-11-05 22:52:15 +0100 Even Rouault $")
+
+// Specification at
+// https://www.nrcan.gc.ca/sites/www.nrcan.gc.ca/files/earthsciences/pdf/gpshgrid_e.pdf
 
 const static BYNEllipsoids EllipsoidTable[] = {
     { "GRS80",       6378137.0,  298.257222101 },
@@ -87,7 +90,9 @@ double BYNRasterBand::GetNoDataValue( int *pbSuccess )
     {
         return dfNoData;
     }
-    return eDataType == GDT_Int16 ? 32767.0 : 9999.0 * GetScale();
+    const double dfFactor =
+        reinterpret_cast<BYNDataset*>(poDS)->hHeader.dfFactor;
+    return eDataType == GDT_Int16 ? 32767.0 : 9999.0 * dfFactor;
 }
 
 /************************************************************************/
@@ -197,11 +202,21 @@ int BYNDataset::Identify( GDALOpenInfo *poOpenInfo )
         hHeader.nDatum     < 0 || hHeader.nDatum     > 1 ||
         hHeader.nEllipsoid < 0 || hHeader.nEllipsoid > 7 ||
         hHeader.nByteOrder < 0 || hHeader.nByteOrder > 1 ||
-        hHeader.nScale     < 0 || hHeader.nScale     > 1 ||
-        hHeader.nTideSys   < 0 || hHeader.nTideSys   > 2 ||
-        hHeader.nPtType    < 0 || hHeader.nPtType    > 1 )
+        hHeader.nScale     < 0 || hHeader.nScale     > 1 )
         return FALSE;
 
+    if((hHeader.nTideSys   < 0 || hHeader.nTideSys   > 2 ||
+        hHeader.nPtType    < 0 || hHeader.nPtType    > 1 ))
+    {
+        // Some datasets use 0xCC as a marker for invalidity for
+        // records starting from Geopotential Wo
+        for( int i = 52; i < 78; i++ )
+        {
+            if( poOpenInfo->pabyHeader[i] != 0xCC )
+                return FALSE;
+        }
+    }
+
     if( hHeader.nScale == 0 )
     {
         if( ( std::abs( static_cast<GIntBig>( hHeader.nSouth ) - 
@@ -889,7 +904,7 @@ void GDALRegister_BYN()
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "Natural Resources Canada's Geoid" );
     poDriver->SetMetadataItem( GDAL_DMD_EXTENSIONS, "byn err" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_byn.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/byn.html" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Int16 Int32" );
 
     poDriver->pfnOpen = BYNDataset::Open;
