@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2009, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,7 +34,7 @@
 
 #include <vector>
 
-CPL_CVSID("$Id: cpgdataset.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
+CPL_CVSID("$Id: cpgdataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 enum Interleave { BSQ, BIL, BIP };
 
@@ -171,7 +171,7 @@ char **CPGDataset::GetFileList()
 /* ==================================================================== */
 /************************************************************************/
 
-class SIRC_QSLCRasterBand : public GDALRasterBand
+class SIRC_QSLCRasterBand final: public GDALRasterBand
 {
     friend class CPGDataset;
 
@@ -205,7 +205,7 @@ constexpr int M44 = 15;
 /* ==================================================================== */
 /************************************************************************/
 
-class CPG_STOKESRasterBand : public GDALRasterBand
+class CPG_STOKESRasterBand final: public GDALRasterBand
 {
     friend class CPGDataset;
 
@@ -975,7 +975,7 @@ GDALDataset *CPGDataset::InitializeType3Dataset( const char *pszFilename )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "%s is missing a required parameter (number of pixels, "
-                  "number of lines,\nnumber of bands, bytes per pixel, or "
+                  "number of lines,\number of bands, bytes per pixel, or "
                   "data organization).",
                   pszWorkname );
         CPLFree(pszWorkname);
