@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2002, Andrey Kiselev <dron@ak4719.spb.edu>
- * Copyright (c) 2007-2011, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2007-2011, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,7 +32,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: fastdataset.cpp a542b2797f15f2ed694cfcee9ff17d86b339dfee 2018-04-02 00:24:03 +0200 Even Rouault $")
+CPL_CVSID("$Id: fastdataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 // constexpr int ADM_STD_HEADER_SIZE = 4608;  // Format specification says it
 constexpr int ADM_HEADER_SIZE = 5000;  // Should be 4608, but some vendors
@@ -100,7 +100,7 @@ enum FASTSatellite  // Satellites:
 /* ==================================================================== */
 /************************************************************************/
 
-class FASTDataset : public GDALPamDataset
+class FASTDataset final: public GDALPamDataset
 {
     friend class FASTRasterBand;
 
@@ -117,6 +117,8 @@ class FASTDataset : public GDALPamDataset
 
     int         OpenChannel( const char *pszFilename, int iBand );
 
+    CPL_DISALLOW_COPY_ASSIGN(FASTDataset)
+
   public:
     FASTDataset();
     ~FASTDataset() override;
@@ -124,7 +126,10 @@ class FASTDataset : public GDALPamDataset
     static GDALDataset *Open( GDALOpenInfo * );
 
     CPLErr      GetGeoTransform( double * ) override;
-    const char  *GetProjectionRef() override;
+    const char  *_GetProjectionRef() override;
+    const OGRSpatialReference* GetSpatialRef() const override {
+        return GetSpatialRefFromOldGetProjectionRef();
+    }
     VSILFILE    *FOpenChannel( const char *, int iBand, int iFASTBand );
     void        TryEuromap_IRS_1C_1D_ChannelNameConvention();
 
@@ -137,10 +142,12 @@ class FASTDataset : public GDALPamDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class FASTRasterBand : public RawRasterBand
+class FASTRasterBand final: public RawRasterBand
 {
     friend class FASTDataset;
 
+    CPL_DISALLOW_COPY_ASSIGN(FASTRasterBand)
+
   public:
                 FASTRasterBand( FASTDataset *, int, VSILFILE *, vsi_l_offset,
                                 int, int, GDALDataType, int );
@@ -155,7 +162,8 @@ FASTRasterBand::FASTRasterBand( FASTDataset *poDSIn, int nBandIn, VSILFILE * fpR
                                 int nLineOffsetIn,
                                 GDALDataType eDataTypeIn, int bNativeOrderIn ) :
     RawRasterBand( poDSIn, nBandIn, fpRawIn, nImgOffsetIn, nPixelOffsetIn,
-                   nLineOffsetIn, eDataTypeIn, bNativeOrderIn, TRUE )
+                   nLineOffsetIn, eDataTypeIn, bNativeOrderIn,
+                   RawRasterBand::OwnFP::NO )
 {}
 
 /************************************************************************/
@@ -222,7 +230,7 @@ CPLErr FASTDataset::GetGeoTransform( double * padfTransform )
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *FASTDataset::GetProjectionRef()
+const char *FASTDataset::_GetProjectionRef()
 
 {
     if( pszProjection )
@@ -961,7 +969,7 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
     CPLFree( pszTemp );
 
     // Read 15 USGS projection parameters
-    double adfProjParms[15] = { 0.0 };
+    double adfProjParams[15] = { 0.0 };
     pszTemp = strstr( pszHeader, USGS_PARAMETERS );
     if ( pszTemp && !EQUAL( pszTemp, "" ) )
     {
@@ -971,7 +979,7 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
             pszTemp = strpbrk( pszTemp, "-.0123456789" );
             if ( pszTemp )
             {
-                adfProjParms[i] = CPLScanDouble( pszTemp, VALUE_SIZE );
+                adfProjParams[i] = CPLScanDouble( pszTemp, VALUE_SIZE );
                 pszTemp = strpbrk( pszTemp, " \t" );
             }
             if (pszTemp == nullptr )
@@ -1057,7 +1065,7 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
         // Create projection definition
         OGRSpatialReference oSRS;
         OGRErr eErr =
-            oSRS.importFromUSGS( iProjSys, iZone, adfProjParms, iDatum, bAnglesInPackedDMSFormat );
+            oSRS.importFromUSGS( iProjSys, iZone, adfProjParams, iDatum, bAnglesInPackedDMSFormat );
         if ( eErr != OGRERR_NONE )
             CPLDebug( "FAST", "Import projection from USGS failed: %d", eErr );
         oSRS.SetLinearUnits( SRS_UL_METER, 1.0 );
@@ -1199,7 +1207,7 @@ void GDALRegister_FAST()
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                "EOSAT FAST Format" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                               "frmt_fast.html" );
+                               "drivers/raster/fast.html" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
     poDriver->pfnOpen = FASTDataset::Open;
