@@ -30,7 +30,7 @@
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: iscedataset.cpp 74385c8cc244684d8cff57e1b66ec50a255057c1 2018-06-21 20:46:53 +0200 Even Rouault $")
+CPL_CVSID("$Id: iscedataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 static const char * const apszISCE2GDALDatatypes[] = {
     "BYTE:Byte",
@@ -74,7 +74,7 @@ static const char * const apszSchemeNames[] = { "BIL", "BIP", "BSQ", nullptr };
 
 class ISCERasterBand;
 
-class ISCEDataset : public RawDataset
+class ISCEDataset final: public RawDataset
 {
     friend class ISCERasterBand;
 
@@ -84,6 +84,8 @@ class ISCEDataset : public RawDataset
 
     enum Scheme eScheme;
 
+    CPL_DISALLOW_COPY_ASSIGN(ISCEDataset)
+
   public:
     ISCEDataset();
     ~ISCEDataset() override;
@@ -105,14 +107,15 @@ class ISCEDataset : public RawDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class ISCERasterBand : public RawRasterBand
+class ISCERasterBand final: public RawRasterBand
 {
+        CPL_DISALLOW_COPY_ASSIGN(ISCERasterBand)
+
     public:
-                ISCERasterBand( GDALDataset *poDS, int nBand, void *fpRaw,
+                ISCERasterBand( GDALDataset *poDS, int nBand, VSILFILE *fpRaw,
                                   vsi_l_offset nImgOffset, int nPixelOffset,
                                   int nLineOffset,
-                                  GDALDataType eDataType, int bNativeOrder,
-                                  int bIsVSIL = FALSE, int bOwnsFP = FALSE );
+                                  GDALDataType eDataType, int bNativeOrder );
 };
 
 /************************************************************************/
@@ -123,6 +126,9 @@ static CPLString getXMLFilename( GDALOpenInfo *poOpenInfo )
 {
     CPLString osXMLFilename;
 
+    if( poOpenInfo->fpL == nullptr )
+        return CPLString();
+
     char **papszSiblingFiles = poOpenInfo->GetSiblingFiles();
     if ( papszSiblingFiles == nullptr )
     {
@@ -173,7 +179,7 @@ ISCEDataset::ISCEDataset() :
 
 ISCEDataset::~ISCEDataset( void )
 {
-    FlushCache();
+    ISCEDataset::FlushCache();
     if ( fpImage != nullptr )
     {
         if( VSIFCloseL( fpImage ) != 0 )
@@ -311,7 +317,7 @@ void ISCEDataset::FlushCache( void )
                                  "createCoordinate" );
     CPLCreateXMLElementAndValue( psCoordinate1Node,
                                  "doc",
-                                 "First coordinate of a 2D image (witdh)." );
+                                 "First coordinate of a 2D image (width)." );
     /* Property name */
     psTmpNode = CPLCreateXMLNode( psCoordinate1Node,
                                   CXT_Element,
@@ -641,6 +647,12 @@ GDALDataset *ISCEDataset::Open( GDALOpenInfo *poOpenInfo, bool bFileSizeCheck )
     }
     const GDALDataType eDataType = GDALGetDataTypeByName( pszDataType );
     const int nDTSize = GDALGetDataTypeSizeBytes(eDataType);
+    if( nDTSize == 0 )
+    {
+        delete poDS;
+        CSLDestroy( papszXmlProps );
+        return nullptr;
+    }
     const char *pszScheme = CSLFetchNameValue( papszXmlProps, "SCHEME" );
     int nPixelOffset = 0;
     int nLineOffset = 0;
@@ -737,8 +749,7 @@ GDALDataset *ISCEDataset::Open( GDALOpenInfo *poOpenInfo, bool bFileSizeCheck )
                        new ISCERasterBand( poDS, b + 1, poDS->fpImage,
                                            nBandOffset * b,
                                            nPixelOffset, nLineOffset,
-                                           eDataType, bNativeOrder,
-                                           TRUE, FALSE ) );
+                                           eDataType, bNativeOrder ) );
     }
 
 /* -------------------------------------------------------------------- */
@@ -764,7 +775,7 @@ GDALDataset *ISCEDataset::Open( GDALOpenInfo *poOpenInfo, bool bFileSizeCheck )
 
         /* ISCE format seems not to have a projection field, but uses   */
         /* WGS84.                                                       */
-        poDS->SetProjection( SRS_WKT_WGS84 );
+        poDS->SetProjection( SRS_WKT_WGS84_LAT_LONG );
     }
 
 /* -------------------------------------------------------------------- */
@@ -909,14 +920,13 @@ GDALDataset *ISCEDataset::Create( const char *pszFilename,
 /*                          ISCERasterBand()                            */
 /************************************************************************/
 
-ISCERasterBand::ISCERasterBand( GDALDataset *poDSIn, int nBandIn, void *fpRawIn,
+ISCERasterBand::ISCERasterBand( GDALDataset *poDSIn, int nBandIn, VSILFILE *fpRawIn,
                                 vsi_l_offset nImgOffsetIn, int nPixelOffsetIn,
                                 int nLineOffsetIn,
-                                GDALDataType eDataTypeIn, int bNativeOrderIn,
-                                int bIsVSILIn, int bOwnsFPIn ) :
+                                GDALDataType eDataTypeIn, int bNativeOrderIn ) :
     RawRasterBand( poDSIn, nBandIn, fpRawIn, nImgOffsetIn, nPixelOffsetIn,
-                   nLineOffsetIn, eDataTypeIn, bNativeOrderIn, bIsVSILIn,
-                   bOwnsFPIn )
+                   nLineOffsetIn, eDataTypeIn, bNativeOrderIn,
+                   RawRasterBand::OwnFP::NO )
 {}
 
 /************************************************************************/
@@ -932,7 +942,7 @@ void GDALRegister_ISCE()
 
     poDriver->SetDescription( "ISCE" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "ISCE raster" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_various.html#ISCE" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/isce.html" );
     poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES,
                                "Byte Int16 Int32 Int64 Float32"
                                " Float64 CInt16 CInt64 CFloat32 "
