@@ -78,7 +78,7 @@ static void GDALRDADriverUnload(GDALDriver*)
 /************************************************************************/
 enum class RDADatasetType : std::int8_t { UNDEFINED = -1, GRAPH = 1, TEMPLATE = 2 };
 
-class GDALRDADataset: public GDALDataset
+class GDALRDADataset final: public GDALDataset
 {
         friend class GDALRDARasterBand;
 
@@ -143,6 +143,7 @@ class GDALRDADataset: public GDALDataset
         bool      GetAuthorization();
         bool      ParseAuthorizationResponse(const CPLString& osAuth);
         bool      ParseConnectionString( GDALOpenInfo* poOpenInfo );
+        bool      ParseImageReferenceString( GDALOpenInfo* poOpenInfo );
         json_object* ReadJSonFile(const char* pszFilename, const char* pszKey,
                                   bool bErrorOn404);
         CPLString ConstructTileFetchURL(const CPLString& baseUrl,
@@ -172,7 +173,10 @@ class GDALRDADataset: public GDALDataset
         static GDALDataset* OpenStatic( GDALOpenInfo* poOpenInfo );
 
         CPLErr          GetGeoTransform(double *padfTransform) override;
-        const char*     GetProjectionRef() override;
+        const char*     _GetProjectionRef() override;
+        const OGRSpatialReference* GetSpatialRef() const override {
+            return GetSpatialRefFromOldGetProjectionRef();
+        }
         CPLErr          IRasterIO(GDALRWFlag eRWFlag,
                                       int nXOff, int nYOff,
                                       int nXSize, int nYSize,
@@ -201,7 +205,7 @@ class GDALRDADataset: public GDALDataset
 /*                         GDALRDARasterBand                            */
 /************************************************************************/
 
-class GDALRDARasterBand: public GDALRasterBand
+class GDALRDARasterBand final: public GDALRasterBand
 {
     public:
         GDALRDARasterBand( GDALRDADataset* poDS, int nBand);
@@ -343,9 +347,12 @@ int GDALRDADataset::Identify( GDALOpenInfo* poOpenInfo )
 {
     bool retval = false;
     // if connection string is JSON
-    if((strstr(poOpenInfo->pszFilename, "graph-id") != nullptr &&
+    if(((strstr(poOpenInfo->pszFilename, "graph-id") != nullptr &&
         strstr(poOpenInfo->pszFilename, "node-id") != nullptr) ||
-       strstr(poOpenInfo->pszFilename, "template-id") != nullptr)
+       strstr(poOpenInfo->pszFilename, "template-id") != nullptr) ||
+        (strstr(poOpenInfo->pszFilename, "graphId") != nullptr &&
+         strstr(poOpenInfo->pszFilename, "nodeId") != nullptr) ||
+        strstr(poOpenInfo->pszFilename, "templateId") != nullptr)
     {
         retval = true;
     }
@@ -357,7 +364,9 @@ int GDALRDADataset::Identify( GDALOpenInfo* poOpenInfo )
                 poOpenInfo->pabyHeader);
             if( pszHeader != nullptr && STARTS_WITH_CI(pszHeader, "{") &&
                 (strstr(pszHeader,"graph-id") != nullptr ||
-                 strstr(pszHeader,"template-id") != nullptr))
+                 strstr(pszHeader,"template-id") != nullptr ||
+                 strstr(pszHeader,"graphId") != nullptr ||
+                 strstr(pszHeader,"templateId") != nullptr))
             {
                 retval = true;
             }
@@ -619,6 +628,12 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
     {
         osConnectionString = poOpenInfo->pszFilename;
     }
+
+    //Bypass parsing JSON if not in the expected format
+    if (!(strstr(osConnectionString, "graph-id") != nullptr ||
+          strstr(osConnectionString, "template-id") != nullptr))
+        return false;
+
     json_object* poObj = nullptr;
     if( !OGRJSonParse(osConnectionString, &poObj, true) )
     {
@@ -629,10 +644,10 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
     JsonObjectUniquePtr oObj(poObj);
 
     json_object* poGraphId =
-        CPL_json_object_object_get(poObj, "graph-id");
+            CPL_json_object_object_get(poObj, "graph-id");
 
     if( poGraphId != nullptr &&
-         json_object_get_type(poGraphId) == json_type_string)
+        json_object_get_type(poGraphId) == json_type_string)
     {
         m_osType = RDADatasetType::GRAPH;
         m_osGraphId = json_object_get_string(poGraphId);
@@ -659,7 +674,7 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
 
 
     json_object* poNodeId =
-        CPL_json_object_object_get(poObj, "node-id");
+            CPL_json_object_object_get(poObj, "node-id");
     if( (poNodeId == nullptr ||
          json_object_get_type(poNodeId) != json_type_string) &&
         m_osType == RDADatasetType::GRAPH )
@@ -674,16 +689,16 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
                  ?  json_object_get_string(poNodeId) :"";
 
     json_object* poDeleteOnClose = json_ex_get_object_by_path(poObj,
-                                                "options.delete-on-close");
+                                                              "options.delete-on-close");
     if( poDeleteOnClose &&
         json_object_get_type(poDeleteOnClose) == json_type_boolean )
     {
         m_bDeleteOnClose = CPL_TO_BOOL(
-            json_object_get_boolean(poDeleteOnClose));
+                json_object_get_boolean(poDeleteOnClose));
     }
 
     json_object* poMaxConnections = json_ex_get_object_by_path(poObj,
-                                                    "options.max-connections");
+                                                               "options.max-connections");
     if( poMaxConnections &&
         json_object_get_type(poMaxConnections) == json_type_int )
     {
@@ -691,7 +706,7 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
     }
 
     json_object* poEnforceAdviseRead = json_ex_get_object_by_path(poObj,
-                                                        "options.advise-read");
+                                                                  "options.advise-read");
     if( poEnforceAdviseRead &&
         json_object_get_type(poEnforceAdviseRead) == json_type_boolean )
     {
@@ -704,9 +719,9 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
         json_object *poParams = CPL_json_object_object_get(poObj, "params");
 
         if(poParams != nullptr &&
-            json_object_get_type(poParams) == json_type_array ) {
-            const int nSize = json_object_array_length(poParams);
-            for (int i = 0; i < nSize; ++i) {
+           json_object_get_type(poParams) == json_type_array ) {
+            const auto nSize = json_object_array_length(poParams);
+            for (auto i = decltype(nSize){0}; i < nSize; ++i) {
                 json_object *ds = json_object_array_get_idx(poParams, i);
                 if (ds != nullptr) {
                     json_object_iter it;
@@ -719,11 +734,11 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
                         {
                             CPLString tkey = it.key;
                             const char* tval =
-                                json_object_get_string(it.val);
+                                    json_object_get_string(it.val);
                             if(tval != nullptr)
                             {
                                 m_osParams.push_back(std::make_tuple(tkey,
-                                                        CPLString(tval)));
+                                                                     CPLString(tval)));
                             }
                         }
 
@@ -736,6 +751,141 @@ bool GDALRDADataset::ParseConnectionString( GDALOpenInfo* poOpenInfo )
 
     return true;
 }
+/************************************************************************/
+/*                      ParseImageReferenceString()                         */
+/************************************************************************/
+
+bool GDALRDADataset::ParseImageReferenceString( GDALOpenInfo* poOpenInfo )
+{
+    CPLString osConnectionString;
+    if(EQUAL(CPLGetExtension(poOpenInfo->pszFilename), "DGRDA"))
+    {
+        CSLUniquePtr papszContent(CSLLoad2( poOpenInfo->pszFilename,
+                                            -1, -1, nullptr));
+        char** papszIter = papszContent.get();
+        if(papszIter != nullptr)
+            osConnectionString = *(papszIter);
+    }
+    else
+    {
+        osConnectionString = poOpenInfo->pszFilename;
+    }
+
+    //Bypass parsing JSON if not in the expected format
+    if (!(strstr(osConnectionString, "graphId") != nullptr ||
+          strstr(osConnectionString, "templateId") != nullptr))
+        return false;
+
+    json_object* poObj = nullptr;
+    if( !OGRJSonParse(osConnectionString, &poObj, true) )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Invalid JSon document as dataset name");
+        return false;
+    }
+    JsonObjectUniquePtr oObj(poObj);
+
+    json_object* poGraphId =
+            CPL_json_object_object_get(poObj, "graphId");
+
+    if( poGraphId != nullptr &&
+        json_object_get_type(poGraphId) == json_type_string)
+    {
+        m_osType = RDADatasetType::GRAPH;
+        m_osGraphId = json_object_get_string(poGraphId);
+    }
+
+
+
+    json_object* poTemplateId =
+            CPL_json_object_object_get(poObj, "templateId");
+
+    if( poTemplateId != nullptr &&
+        json_object_get_type(poTemplateId) == json_type_string)
+    {
+        m_osType = RDADatasetType::TEMPLATE;
+        m_osTemplateId = json_object_get_string(poTemplateId);
+    }
+
+    if(m_osType == RDADatasetType::UNDEFINED)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Missing graphId or templateId");
+        return false;
+    }
+
+
+    json_object* poNodeId =
+            CPL_json_object_object_get(poObj, "nodeId");
+    if( (poNodeId == nullptr ||
+         json_object_get_type(poNodeId) != json_type_string) &&
+        m_osType == RDADatasetType::GRAPH )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Missing nodeId");
+        return false;
+    }
+
+    m_osNodeId = (poNodeId != nullptr &&
+                  json_object_get_type(poNodeId) == json_type_string)
+                 ?  json_object_get_string(poNodeId) :"";
+
+    json_object* poDeleteOnClose = json_ex_get_object_by_path(poObj,
+                                                              "options.delete-on-close");
+    if( poDeleteOnClose &&
+        json_object_get_type(poDeleteOnClose) == json_type_boolean )
+    {
+        m_bDeleteOnClose = CPL_TO_BOOL(
+                json_object_get_boolean(poDeleteOnClose));
+    }
+
+    json_object* poMaxConnections = json_ex_get_object_by_path(poObj,
+                                                               "options.max-connections");
+    if( poMaxConnections &&
+        json_object_get_type(poMaxConnections) == json_type_int )
+    {
+        MaxCurlConnectionsSet(json_object_get_int(poMaxConnections));
+    }
+
+    json_object* poEnforceAdviseRead = json_ex_get_object_by_path(poObj,
+                                                                  "options.advise-read");
+    if( poEnforceAdviseRead &&
+        json_object_get_type(poEnforceAdviseRead) == json_type_boolean )
+    {
+        m_bAdviseRead = CPL_TO_BOOL(
+                json_object_get_boolean(poEnforceAdviseRead));
+    }
+
+    if(m_osType == RDADatasetType::TEMPLATE)
+    {
+        json_object *poParams = CPL_json_object_object_get(poObj, "parameters");
+
+        if(poParams != nullptr &&
+           json_object_get_type(poParams) == json_type_object ) {
+            json_object_iter it;
+            it.key = nullptr;
+            it.val = nullptr;
+            it.entry = nullptr;
+            json_object_object_foreachC( poParams, it )
+            {
+                if(it.key != nullptr && it.val != nullptr)
+                {
+                    CPLString tkey = it.key;
+                    const char* tval =
+                            json_object_get_string(it.val);
+                    if(tval != nullptr)
+                    {
+                        m_osParams.push_back(std::make_tuple(tkey,
+                                                             CPLString(tval)));
+                    }
+                }
+            }
+        }
+
+    }
+
+    return true;
+}
 
 /************************************************************************/
 /*                             GetHTTPOptions()                         */
@@ -1005,7 +1155,7 @@ bool GDALRDADataset::ReadImageMetadata()
     m_nTileYSize = static_cast<int>(std::max<GIntBig>(0,
         std::min<GIntBig>(
             GetJsonInt64(poObj, "tileYSize", true, bError), INT_MAX)));
-    nBands = static_cast<int>(std::max<GIntBig>(0, 
+    nBands = static_cast<int>(std::max<GIntBig>(0,
         std::min<GIntBig>(
             GetJsonInt64(poObj, "numBands", true, bError), INT_MAX)));
     if( !bError && !GDALCheckBandCount(nBands, false) )
@@ -1207,7 +1357,7 @@ static CPLString Get20Coeffs(json_object* poObj, const char* pszPath,
         json_object_get_type(poCoeffs) != json_type_array ||
         json_object_array_length(poCoeffs) != 20 )
     {
-        if( bVerboseError ) 
+        if( bVerboseError )
         {
             CPLError(CE_Failure, CPLE_AppDefined,
                      "Cannot find %s of type array of 20 double", pszPath);
@@ -1298,6 +1448,23 @@ bool GDALRDADataset::ReadRPCs()
                                   CPLSPrintf("%.18g", dfMaxY));
     }
 
+
+    double errBias = GetJsonDouble(poObj, "errBias", true, bError);
+    if (bError) {
+        errBias = 0.0;
+        bError = false;
+    }
+    papszMD = CSLSetNameValue(papszMD, RPC_ERR_BIAS,
+        CPLSPrintf("%.18g", errBias));
+
+    double errRand = GetJsonDouble(poObj, "errRand", true, bError);
+    if (bError) {
+        errRand = 0.0;
+        bError = false;
+    }
+    papszMD = CSLSetNameValue(papszMD, RPC_ERR_RAND,
+        CPLSPrintf("%.18g", errRand));
+
     papszMD = CSLSetNameValue(papszMD, RPC_LINE_OFF,
         CPLSPrintf("%.18g", GetJsonDouble(poObj, "lineOffset", true, bError)));
     papszMD = CSLSetNameValue(papszMD, RPC_SAMP_OFF,
@@ -1387,7 +1554,7 @@ CPLErr GDALRDADataset::GetGeoTransform(double *padfTransform)
 /*                        GetProjectionRef()                            */
 /************************************************************************/
 
-const char* GDALRDADataset::GetProjectionRef()
+const char* GDALRDADataset::_GetProjectionRef()
 {
     if( !m_bTriedReadGeoreferencing )
         ReadGeoreferencing();
@@ -1400,7 +1567,7 @@ const char* GDALRDADataset::GetProjectionRef()
 
 bool GDALRDADataset::Open( GDALOpenInfo* poOpenInfo )
 {
-    if( !ParseConnectionString(poOpenInfo) )
+    if( !(ParseImageReferenceString(poOpenInfo) || ParseConnectionString(poOpenInfo) ))
         return false;
 
     if( !ReadConfiguration() )
@@ -1479,7 +1646,7 @@ GDALColorInterp GDALRDARasterBand::GetColorInterpretation()
         const char* pszName;
         GDALColorInterp aeInter[5];
     }
-    asColorInterpretations[] = 
+    asColorInterpretations[] =
     {
         { "PAN", { GCI_GrayIndex, GCI_Undefined, GCI_Undefined,
                    GCI_Undefined, GCI_Undefined } },
@@ -1518,7 +1685,7 @@ GDALColorInterp GDALRDARasterBand::GetColorInterpretation()
                 }
             }
         }
-        
+
     }
 
     return GCI_Undefined;
@@ -1905,10 +2072,10 @@ CPLErr GDALRDADataset::IRasterIO(GDALRWFlag eRWFlag,
 /************************************************************************/
 
 CPLErr GDALRDADataset::AdviseRead (int nXOff, int nYOff,
-                                   int nXSize, int nYSize, 
+                                   int nXSize, int nYSize,
                                    int /* nBufXSize */,
                                    int /* nBufYSize */,
-                                   GDALDataType /* eBufType */, 
+                                   GDALDataType /* eBufType */,
                                    int /*nBands*/, int* /*panBands*/,
                                    char ** /* papszOptions */)
 {
@@ -1947,10 +2114,10 @@ CPLErr GDALRDARasterBand::IRasterIO(GDALRWFlag eRWFlag,
 /************************************************************************/
 
 CPLErr GDALRDARasterBand::AdviseRead (int nXOff, int nYOff,
-                                        int nXSize, int nYSize, 
+                                        int nXSize, int nYSize,
                                         int /* nBufXSize */,
                                         int /* nBufYSize */,
-                                        GDALDataType /* eBufType */, 
+                                        GDALDataType /* eBufType */,
                                         char ** /* papszOptions */)
 {
     GDALRDADataset* poGDS = reinterpret_cast<GDALRDADataset*>(poDS);
@@ -2134,7 +2301,7 @@ void GDALRegister_RDA()
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                "DigitalGlobe Raster Data Access driver" );
     poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                               "frmt_rda.html" );
+                               "drivers/raster/rda.html" );
     poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "dgrda" );
 
     poDriver->SetMetadataItem( GDAL_DMD_OPENOPTIONLIST,
