@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: rmfdataset.h 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $
+ * $Id: rmfdataset.h 86933038c3926cd4dc3ff37c431b317abb69e602 2021-03-27 23:20:49 +0100 Even Rouault $
  *
  * Project:  Raster Matrix Format
  * Purpose:  Private class declarations for the RMF classes used to read/write
@@ -130,6 +130,7 @@ typedef struct
 typedef struct
 {
     GInt32      nEllipsoid;
+    GInt32      nVertDatum;
     GInt32      nDatum;
     GInt32      nZone;
 } RMFExtHeader;
@@ -138,7 +139,7 @@ typedef struct
 /*                            RMFCompressionJob                         */
 /************************************************************************/
 
-typedef struct
+struct RMFCompressionJob
 {
     RMFDataset* poDS = nullptr;
     CPLErr eResult = CE_None;
@@ -150,7 +151,7 @@ typedef struct
     size_t nCompressedBytes = 0;
     GUInt32 nXSize = 0;
     GUInt32 nYSize = 0;
-} RMFCompressionJob;
+};
 
 /************************************************************************/
 /*                            RMFCompressData                           */
@@ -176,11 +177,11 @@ struct RMFCompressData
 /*                            RMFTileData                               */
 /************************************************************************/
 
-typedef struct
+struct RMFTileData
 {
     std::vector<GByte>  oData;
     int                 nBandsWritten = 0;
-} RMFTileData;
+};
 
 /************************************************************************/
 /*                              RMFDataset                              */
@@ -198,6 +199,7 @@ private:
     GUInt32         *paiTiles;
     GByte           *pabyDecompressBuffer;
     GByte           *pabyCurrentTile;
+    bool            bCurrentTileIsNull;
     int             nCurrentTileXOff;
     int             nCurrentTileYOff;
     GUInt32         nCurrentTileBytes;
@@ -271,7 +273,7 @@ private:
 
     static int          Identify( GDALOpenInfo * poOpenInfo );
     static GDALDataset  *Open( GDALOpenInfo * );
-    static GDALDataset  *Open(GDALOpenInfo *, RMFDataset* poParentDS, vsi_l_offset nNextHeaderOffset );
+    static RMFDataset   *Open(GDALOpenInfo *, RMFDataset* poParentDS, vsi_l_offset nNextHeaderOffset );
     static GDALDataset  *Create( const char *, int, int, int,
                                  GDALDataType, char ** );
     static GDALDataset  *Create( const char *, int, int, int,
@@ -312,7 +314,7 @@ private:
     int                 SetupCompression(GDALDataType eType,
                                          const char* pszFilename);
     static void         WriteTileJobFunc(void* pData);
-    CPLErr              InitCompressorData(char **papszParmList);
+    CPLErr              InitCompressorData(char **papszParamList);
     CPLErr              WriteTile(int nBlockXOff, int nBlockYOff,
                                   GByte* pabyData, size_t nBytes,
                                   GUInt32 nRawXSize, GUInt32 nRawYSize);
@@ -320,7 +322,8 @@ private:
                                      GByte* pabyData, size_t nBytes);
     CPLErr              ReadTile(int nBlockXOff, int nBlockYOff,
                                  GByte* pabyData, size_t nBytes,
-                                 GUInt32 nRawXSize, GUInt32 nRawYSize);
+                                 GUInt32 nRawXSize, GUInt32 nRawYSize,
+                                 bool& bNullTile);
     void                SetupNBits();
 };
 
