@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2006, Frank Warmerdam
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -230,7 +230,7 @@ WCSDataset::DirectRasterIO( CPL_UNUSED GDALRWFlag eRWFlag,
                             int *panBandMap,
                             GSpacing nPixelSpace, GSpacing nLineSpace,
                             GSpacing nBandSpace,
-                            CPL_UNUSED GDALRasterIOExtraArg* psExtraArg)
+                            GDALRasterIOExtraArg* psExtraArg)
 {
     CPLDebug( "WCS", "DirectRasterIO(%d,%d,%d,%d) -> (%d,%d) (%d bands)\n",
               nXOff, nYOff, nXSize, nYSize,
@@ -253,7 +253,7 @@ WCSDataset::DirectRasterIO( CPL_UNUSED GDALRWFlag eRWFlag,
     CPLHTTPResult *psResult = nullptr;
     CPLErr eErr =
         GetCoverage( nXOff, nYOff, nXSize, nYSize, nBufXSize, nBufYSize,
-                     band_count, panBandMap, &psResult );
+                     band_count, panBandMap, psExtraArg, &psResult );
 
     if( eErr != CE_None )
         return eErr;
@@ -339,6 +339,7 @@ static bool ProcessError( CPLHTTPResult *psResult );
 CPLErr WCSDataset::GetCoverage( int nXOff, int nYOff, int nXSize, int nYSize,
                                 int nBufXSize, int nBufYSize,
                                 int nBandCount, int *panBandList,
+                                GDALRasterIOExtraArg *psExtraArg,
                                 CPLHTTPResult **ppsResult )
 
 {
@@ -378,8 +379,18 @@ CPLErr WCSDataset::GetCoverage( int nXOff, int nYOff, int nXSize, int nYSize,
 /*      Fetch the result.                                               */
 /* -------------------------------------------------------------------- */
     CPLErrorReset();
-    *ppsResult = CPLHTTPFetch( osRequest, papszHttpOptions );
-
+    if( psExtraArg && psExtraArg->pfnProgress != nullptr)
+    {
+        *ppsResult = CPLHTTPFetchEx( osRequest, papszHttpOptions,
+                                     psExtraArg->pfnProgress,
+                                     psExtraArg->pProgressData,
+                                     nullptr, nullptr);
+    }
+    else
+    {
+        *ppsResult = CPLHTTPFetch( osRequest, papszHttpOptions );
+    }
+    
     if( ProcessError( *ppsResult ) )
         return CE_Failure;
     else
@@ -518,13 +529,17 @@ static bool ProcessError( CPLHTTPResult *psResult )
     {
         CPLXMLNode *psTree = CPLParseXMLString( (const char *)psResult->pabyData );
         CPLStripXMLNamespace( psTree, nullptr, TRUE );
-        const char *pszMsg = CPLGetXMLValue(psTree, "=ServiceExceptionReport.ServiceException", nullptr);
-        if (!pszMsg) {
-            pszMsg = CPLGetXMLValue(psTree, "=ExceptionReport.Exception.ExceptionText", nullptr);
+        CPLString msg = CPLGetXMLValue(psTree, "=ServiceExceptionReport.ServiceException", "");
+        if (msg == "") {
+            msg = CPLGetXMLValue(psTree, "=ExceptionReport.Exception.exceptionCode", "");
+            if (msg != "") {
+                msg += ": ";
+            }
+            msg += CPLGetXMLValue(psTree, "=ExceptionReport.Exception.ExceptionText", "");
         }
-        if( pszMsg )
+        if( msg != "" )
             CPLError( CE_Failure, CPLE_AppDefined,
-                      "%s", pszMsg );
+                      "%s", msg.c_str() );
         else
             CPLError( CE_Failure, CPLE_AppDefined,
                       "Corrupt Service Exception:\n%s",
@@ -539,7 +554,10 @@ static bool ProcessError( CPLHTTPResult *psResult )
 /*      sufficient.                                                     */
 /* -------------------------------------------------------------------- */
     if( CPLGetLastErrorNo() != 0 )
+    {
+        CPLHTTPDestroyResult( psResult );
         return TRUE;
+    }
 
     return false;
 }
@@ -579,7 +597,7 @@ int WCSDataset::EstablishRasterDetails()
     CPLHTTPResult *psResult = nullptr;
     CPLErr eErr;
 
-    eErr = GetCoverage( 0, 0, 2, 2, 2, 2, 0, nullptr, &psResult );
+    eErr = GetCoverage( 0, 0, 2, 2, 2, 2, 0, nullptr, nullptr, &psResult );
     if( eErr != CE_None )
         return false;
 
@@ -1032,57 +1050,6 @@ static WCSDataset *BootstrapGlobal(GDALOpenInfo * poOpenInfo, const CPLString &c
     return WCSDataset::CreateFromCapabilities(cache, filename, url);
 }
 
-/************************************************************************/
-/*                        CreateServiceMetadata()                       */
-/************************************************************************/
-
-// master filename is the name of the global metadata file
-// filename is the name of the metadata file of the service file
-static void CreateServiceMetadata(const CPLString &coverage,
-                                  const CPLString &master_filename,
-                                  const CPLString &meta_filename)
-{
-    CPLXMLTreeCloser doc(CPLParseXMLFile(master_filename));
-    CPLXMLNode *metadata = doc.getDocumentElement();
-    // remove other subdatasets than the current
-    int subdataset = 0;
-    CPLXMLNode *domain = SearchChildWithValue(metadata, "domain", "SUBDATASETS");
-    if (domain == nullptr) {
-        return;
-    }
-    for (CPLXMLNode *node = domain->psChild; node != nullptr; node = node->psNext) {
-        if (node->eType != CXT_Element) {
-            continue;
-        }
-        CPLString key = CPLGetXMLValue(node, "key", "");
-        if (!STARTS_WITH(key, "SUBDATASET_")) {
-            continue;
-        }
-        CPLString value = CPLGetXMLValue(node, nullptr, "");
-        if (value.find(coverage) != std::string::npos) {
-            key.erase(0, 11); // SUBDATASET_
-            key.erase(key.find("_"), std::string::npos);
-            subdataset = atoi(key);
-            break;
-        }
-    }
-    if (subdataset > 0) {
-        CPLXMLNode *next = nullptr;
-        for (CPLXMLNode *node = domain->psChild; node != nullptr; node = next) {
-            next = node->psNext;
-            if (node->eType != CXT_Element) {
-                continue;
-            }
-            CPLString key = CPLGetXMLValue(node, "key", "");
-            if (key.find(CPLString().Printf("SUBDATASET_%i_", subdataset)) == std::string::npos) {
-                CPLRemoveXMLChild(domain, node);
-                CPLDestroyXMLNode(node);
-            }
-        }
-    }
-    CPLSerializeXMLTreeToFile(metadata, meta_filename);
-}
-
 /************************************************************************/
 /*                          CreateService()                             */
 /************************************************************************/
@@ -1298,8 +1265,6 @@ GDALDataset *WCSDataset::Open( GDALOpenInfo * poOpenInfo )
             if (version == "") {
                 version = global->Version();
             }
-            CPLString global_meta = CPLString(global->GetDescription()) + ".aux.xml";
-            delete global;
             service = CreateService(url, version, coverage, parameters);
 /* -------------------------------------------------------------------- */
 /*          The filename for the new service file.                      */
@@ -1308,8 +1273,24 @@ GDALDataset *WCSDataset::Open( GDALOpenInfo * poOpenInfo )
             if (AddEntryToCache(cache, full_url, filename, ".xml") != CE_None) {
                 return nullptr; // error in cache
             }
-            CreateServiceMetadata(coverage, global_meta, filename + ".aux.xml");
-            CPLSetXMLValue(service, "Parameters", parameters);
+            // Create basic service metadata
+            // copy global metadata (not SUBDATASETS metadata)
+            CPLString global_base = CPLString(global->GetDescription());
+            CPLString global_meta = global_base + ".aux.xml";
+            CPLString capabilities = global_base + ".xml";
+            CPLXMLTreeCloser doc(CPLParseXMLFile(global_meta));
+            CPLXMLNode *metadata = doc.getDocumentElement();
+            CPLXMLNode *domain = SearchChildWithValue(metadata, "domain", "SUBDATASETS");
+            if (domain != nullptr) {
+                CPLRemoveXMLChild(metadata, domain);
+                CPLDestroyXMLNode(domain);
+            }
+            // get metadata for this coverage from the capabilities XML
+            CPLXMLTreeCloser doc2(CPLParseXMLFile(capabilities));
+            global->ParseCoverageCapabilities(doc2.getDocumentElement(), coverage, metadata->psChild);
+            delete global;
+            CPLString metadata_filename = filename + ".aux.xml";
+            CPLSerializeXMLTreeToFile(metadata, metadata_filename);
             updated = true;
         }
         CPLFree(poOpenInfo->pszFilename);
@@ -1426,24 +1407,24 @@ GDALDataset *WCSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Capture HTTP parameters.                                        */
 /* -------------------------------------------------------------------- */
-    const char  *pszParm;
+    const char  *pszParam;
 
     poDS->papszHttpOptions =
         CSLSetNameValue(poDS->papszHttpOptions,
                         "TIMEOUT",
                         CPLGetXMLValue( service, "Timeout", "30" ) );
 
-    pszParm = CPLGetXMLValue( service, "HTTPAUTH", nullptr );
-    if( pszParm )
+    pszParam = CPLGetXMLValue( service, "HTTPAUTH", nullptr );
+    if( pszParam )
         poDS->papszHttpOptions =
             CSLSetNameValue( poDS->papszHttpOptions,
-                             "HTTPAUTH", pszParm );
+                             "HTTPAUTH", pszParam );
 
-    pszParm = CPLGetXMLValue( service, "USERPWD", nullptr );
-    if( pszParm )
+    pszParam = CPLGetXMLValue( service, "USERPWD", nullptr );
+    if( pszParam )
         poDS->papszHttpOptions =
             CSLSetNameValue( poDS->papszHttpOptions,
-                             "USERPWD", pszParm );
+                             "USERPWD", pszParam );
 
 /* -------------------------------------------------------------------- */
 /*      If we don't have the DescribeCoverage result for this           */
@@ -1517,7 +1498,7 @@ GDALDataset *WCSDataset::Open( GDALOpenInfo * poOpenInfo )
     for( iBand = 0; iBand < nBandCount; iBand++ ) {
         WCSRasterBand *band = new WCSRasterBand(poDS, iBand+1, -1);
         // copy band specific metadata to the band
-        char **md_from = poDS->GetMetadata("SUBDATASETS");
+        char **md_from = poDS->GetMetadata("");
         char **md_to = nullptr;
         if (md_from) {
             CPLString our_key = CPLString().Printf("FIELD_%d_", iBand + 1);
@@ -1615,10 +1596,10 @@ CPLErr WCSDataset::GetGeoTransform( double * padfTransform )
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *WCSDataset::GetProjectionRef()
+const char *WCSDataset::_GetProjectionRef()
 
 {
-    const char* pszPrj = GDALPamDataset::GetProjectionRef();
+    const char* pszPrj = GDALPamDataset::_GetProjectionRef();
     if( pszPrj && strlen(pszPrj) > 0 )
         return pszPrj;
 
@@ -1661,7 +1642,7 @@ char **WCSDataset::GetMetadataDomainList()
 {
     return BuildMetadataDomainList(GDALPamDataset::GetMetadataDomainList(),
                                    TRUE,
-                                   "xml:CoverageOffering", NULL);
+                                   "xml:CoverageOffering", nullptr);
 }
 
 /************************************************************************/
@@ -1711,7 +1692,7 @@ void GDALRegister_WCS()
     poDriver->SetDescription( "WCS" );
     poDriver->SetMetadataItem( GDAL_DCAP_RASTER, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, "OGC Web Coverage Service" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_wcs.html" );
+    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/raster/wcs.html" );
     poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
     poDriver->SetMetadataItem( GDAL_DMD_SUBDATASETS, "YES" );
 
