@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2007, Adam Nowacki
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  * Copyright (c) 2017, Dmitry Baryshnikov, <polimax@mail.ru>
  * Copyright (c) 2017, NextGIS, <info@nextgis.com>
  *
@@ -46,7 +46,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: gdalwmsdataset.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
+CPL_CVSID("$Id: gdalwmsdataset.cpp 5e703afd56b19ad73c76ab865cb4f1988211b9ad 2021-01-20 14:45:04 -0800 Lucian Plesea $")
 
 /************************************************************************/
 /*                           GDALWMSDataset()                           */
@@ -156,6 +156,9 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
 
     if (ret == CE_None) {
         const char *max_conn = CPLGetXMLValue(config, "MaxConnections", "");
+        if (max_conn[0] == '\0') {
+            max_conn = CPLGetConfigOption("GDAL_MAX_CONNECTIONS", "");
+        }
         if (max_conn[0] != '\0') {
             m_http_max_conn = atoi(max_conn);
         }
@@ -174,6 +177,10 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
         }
     }
 
+    if (ret == CE_None) {
+        m_osAccept = CPLGetXMLValue(config, "Accept", "");
+    }
+
     if (ret == CE_None) {
         const char *offline_mode = CPLGetXMLValue(config, "OfflineMode", "");
         if (offline_mode[0] != '\0') {
@@ -493,6 +500,8 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
         if (ret == CE_None)
         {
             const char *data_type = CPLGetXMLValue(config, "DataType", "Byte");
+            if (!STARTS_WITH(data_type, "Byte"))
+                SetTileOO("@DATATYPE", data_type);
             m_data_type = GDALGetDataTypeByName(data_type);
             if (m_data_type == GDT_Unknown || m_data_type >= GDT_TypeCount)
             {
@@ -500,9 +509,6 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
                     "GDALWMS: Invalid value in DataType. Data type \"%s\" is not supported.", data_type);
                 ret = CE_Failure;
             }
-            else if (!STARTS_WITH(data_type, "Byte")) { // Valid, non-byte
-                m_tileOO = CSLSetNameValue(m_tileOO, "@DATATYPE", data_type);
-            }
         }
 
         // Initialize the bands and the overviews.  Assumes overviews are powers of two
@@ -558,12 +564,15 @@ CPLErr GDALWMSDataset::Initialize(CPLXMLNode *config, char **l_papszOpenOptions)
     if (ret == CE_None) {
         // Data values are attributes, they include NoData Min and Max
         if (nullptr!=CPLGetXMLNode(config,"DataValues")) {
-            const char *nodata=CPLGetXMLValue(config,"DataValues.NoData",nullptr);
-            if (nodata!=nullptr) WMSSetNoDataValue(nodata);
-            const char *min=CPLGetXMLValue(config,"DataValues.min",nullptr);
-            if (min!=nullptr) WMSSetMinValue(min);
-            const char *max=CPLGetXMLValue(config,"DataValues.max",nullptr);
-            if (max!=nullptr) WMSSetMaxValue(max);
+            const char *nodata = CPLGetXMLValue(config, "DataValues.NoData", "");
+            if (strlen(nodata) > 0) {
+                SetTileOO("@NDV", nodata);
+                WMSSetNoDataValue(nodata);
+            }
+            const char *min = CPLGetXMLValue(config, "DataValues.min", nullptr);
+            if (min != nullptr) WMSSetMinValue(min);
+            const char *max = CPLGetXMLValue(config, "DataValues.max", nullptr);
+            if (max != nullptr) WMSSetMaxValue(max);
         }
     }
 
@@ -728,6 +737,19 @@ const char * const * GDALWMSDataset::GetHTTPRequestOpts()
     if (m_http_max_conn > 0)
         opts = CSLAddString(opts, CPLOPrintf("MAXCONN=%d", m_http_max_conn));
 
+    if (!m_osAccept.empty() )
+        opts = CSLAddNameValue(opts, "ACCEPT", m_osAccept.c_str());
+
     m_http_options = opts;
     return m_http_options;
 }
+
+void GDALWMSDataset::SetTileOO(const char* pszName, const char* pszValue) {
+    if (pszName == nullptr || strlen(pszName) == 0)
+        return;
+    int oldidx = CSLFindName(m_tileOO, pszName);
+    if (oldidx >= 0)
+        m_tileOO = CSLRemoveStrings(m_tileOO, oldidx, 1, nullptr);
+    if (pszValue != nullptr && strlen(pszValue))
+        m_tileOO = CSLAddNameValue(m_tileOO, pszName, pszValue);
+}
