@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1998, 2000, Frank Warmerdam
- * Copyright (c) 2007-2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2007-2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -32,9 +32,9 @@
 #include "gdal_priv.h"
 #include "gdal_rat.h"
 
-#include <errno.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cerrno>
+#include <cstdlib>
+#include <cstring>
 #include <sys/stat.h>
 
 #include "cpl_conv.h"
@@ -46,12 +46,7 @@
 #include "ogr_core.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: gdaldriver.cpp 040a627fb5bb4da9e468acb59aa5b350cdbf193f 2018-04-16 11:17:40 +0200 Even Rouault $")
-
-CPL_C_START
-// TODO(schwehr): Why is this not in a header?
-const char* GDALClientDatasetGetFilename( const char* pszFilename );
-CPL_C_END
+CPL_CVSID("$Id: gdaldriver.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 /************************************************************************/
 /*                             GDALDriver()                             */
@@ -60,11 +55,14 @@ CPL_C_END
 GDALDriver::GDALDriver() :
     pfnOpen(nullptr),
     pfnCreate(nullptr),
+    pfnCreateEx(nullptr),
+    pfnCreateMultiDimensional(nullptr),
     pfnDelete(nullptr),
     pfnCreateCopy(nullptr),
     pDriverData(nullptr),
     pfnUnloadDriver(nullptr),
     pfnIdentify(nullptr),
+    pfnIdentifyEx(nullptr),
     pfnRename(nullptr),
     pfnCopyFiles(nullptr),
     pfnOpenWithDriverArg(nullptr),
@@ -139,9 +137,6 @@ void CPL_STDCALL GDALDestroyDriver( GDALDriverH hDriver )
  * file handle, but also ensures that all the data and metadata has been written
  * to the dataset (GDALFlushCache() is not sufficient for that purpose).
  *
- * In some situations, the new dataset can be created in another process through
- * the \ref gdal_api_proxy mechanism.
- *
  * In GDAL 2, the arguments nXSize, nYSize and nBands can be passed to 0 when
  * creating a vector-only dataset for a compatible driver.
  *
@@ -167,7 +162,7 @@ GDALDataset * GDALDriver::Create( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Does this format support creation.                              */
 /* -------------------------------------------------------------------- */
-    if( pfnCreate == nullptr && pfnCreateVectorOnly == nullptr )
+    if( pfnCreate == nullptr && pfnCreateEx == nullptr && pfnCreateVectorOnly == nullptr )
     {
         CPLError( CE_Failure, CPLE_NotSupported,
                   "GDALDriver::Create() ... no create method implemented"
@@ -198,40 +193,6 @@ GDALDataset * GDALDriver::Create( const char * pszFilename,
         return nullptr;
     }
 
-    const char* pszClientFilename = GDALClientDatasetGetFilename(pszFilename);
-    if( pszClientFilename != nullptr && !EQUAL(GetDescription(), "MEM") &&
-        !EQUAL(GetDescription(), "VRT")  )
-    {
-        GDALDriver* poAPIPROXYDriver = GDALGetAPIPROXYDriver();
-        if( poAPIPROXYDriver != this )
-        {
-            if( poAPIPROXYDriver == nullptr ||
-                poAPIPROXYDriver->pfnCreate == nullptr )
-                return nullptr;
-            char** papszOptionsDup = CSLDuplicate(papszOptions);
-            papszOptionsDup = CSLAddNameValue(papszOptionsDup, "SERVER_DRIVER",
-                                              GetDescription());
-            GDALDataset* poDstDS = poAPIPROXYDriver->pfnCreate(
-                pszClientFilename, nXSize, nYSize, nBands,
-                eType, papszOptionsDup );
-
-            CSLDestroy(papszOptionsDup);
-
-            if( poDstDS != nullptr )
-            {
-                if( poDstDS->GetDescription() == nullptr
-                    || strlen(poDstDS->GetDescription()) == 0 )
-                    poDstDS->SetDescription( pszFilename );
-
-                if( poDstDS->poDriver == nullptr )
-                    poDstDS->poDriver = poAPIPROXYDriver;
-            }
-
-            if( poDstDS != nullptr || CPLGetLastErrorNo() != CPLE_NotSupported )
-                return poDstDS;
-        }
-    }
-
 /* -------------------------------------------------------------------- */
 /*      Make sure we cleanup if there is an existing dataset of this    */
 /*      name.  But even if that seems to fail we will continue since    */
@@ -243,9 +204,12 @@ GDALDataset * GDALDriver::Create( const char * pszFilename,
         // memory driver doesn't expect files with those names to be deleted
         // on a file system...
         // This is somewhat messy. Ideally there should be a way for the
-        // driver to overload the default behaviour
+        // driver to overload the default behavior
         if( !EQUAL(GetDescription(), "MEM") &&
-            !EQUAL(GetDescription(), "Memory") )
+            !EQUAL(GetDescription(), "Memory") &&
+            // ogr2ogr -f PostgreSQL might reach the Delete method of the
+            // PostgisRaster driver which is undesirable
+            !EQUAL(GetDescription(), "PostgreSQL") )
         {
             QuietDelete( pszFilename );
         }
@@ -267,7 +231,12 @@ GDALDataset * GDALDriver::Create( const char * pszFilename,
               papszOptions );
 
     GDALDataset *poDS = nullptr;
-    if( pfnCreate != nullptr )
+    if( pfnCreateEx != nullptr )
+    {
+        poDS = pfnCreateEx( this, pszFilename, nXSize, nYSize, nBands, eType,
+                          papszOptions );
+    }
+    else if( pfnCreate != nullptr )
     {
         poDS = pfnCreate( pszFilename, nXSize, nYSize, nBands, eType,
                           papszOptions );
@@ -317,6 +286,129 @@ GDALCreate( GDALDriverH hDriver, const char * pszFilename,
                                                     const_cast<char**>(papszOptions) );
 }
 
+/************************************************************************/
+/*                       CreateMultiDimensional()                       */
+/************************************************************************/
+
+/**
+ * \brief Create a new multidimensional dataset with this driver.
+ * 
+ * Only drivers that advertise the GDAL_DCAP_MULTIDIM_RASTER capability and
+ * implement the pfnCreateMultiDimensional method might return a non nullptr
+ * GDALDataset.
+ *
+ * This is the same as the C function GDALCreateMultiDimensional().
+ * 
+ * @param pszFilename  the name of the dataset to create.  UTF-8 encoded.
+ * @param papszRootGroupOptions driver specific options regarding the creation
+ *                              of the root group. Might be nullptr.
+ * @param papszOptions driver specific options regarding the creation
+ *                     of the dataset. Might be nullptr.
+ * @return a new dataset, or nullptr in case of failure.
+ *
+ * @since GDAL 3.1
+ */
+
+GDALDataset * GDALDriver::CreateMultiDimensional( const char * pszFilename,
+                                                  CSLConstList papszRootGroupOptions,
+                                                  CSLConstList papszOptions )
+
+{
+/* -------------------------------------------------------------------- */
+/*      Does this format support creation.                              */
+/* -------------------------------------------------------------------- */
+    if( pfnCreateMultiDimensional== nullptr )
+    {
+        CPLError( CE_Failure, CPLE_NotSupported,
+                  "GDALDriver::CreateMultiDimensional() ... "
+                  "no CreateMultiDimensional method implemented "
+                  "for this format." );
+
+        return nullptr;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Validate creation options.                                      */
+/* -------------------------------------------------------------------- */
+    if( CPLTestBool(
+            CPLGetConfigOption("GDAL_VALIDATE_CREATION_OPTIONS", "YES") ) )
+    {
+        const char *pszOptionList = GetMetadataItem( GDAL_DMD_MULTIDIM_DATASET_CREATIONOPTIONLIST );
+        CPLString osDriver;
+        osDriver.Printf("driver %s", GetDescription());
+        GDALValidateOptions( pszOptionList, papszOptions,
+                            "creation option", osDriver );
+    }
+
+    auto poDstDS = pfnCreateMultiDimensional(pszFilename,
+                                     papszRootGroupOptions,
+                                     papszOptions);
+
+    if( poDstDS != nullptr )
+    {
+        if( poDstDS->GetDescription() == nullptr
+            || strlen(poDstDS->GetDescription()) == 0 )
+            poDstDS->SetDescription( pszFilename );
+
+        if( poDstDS->poDriver == nullptr )
+            poDstDS->poDriver = this;
+    }
+
+    return poDstDS;
+}
+
+/************************************************************************/
+/*                       GDALCreateMultiDimensional()                   */
+/************************************************************************/
+
+/** \brief Create a new multidimensional dataset with this driver.
+ * 
+ * This is the same as the C++ method GDALDriver::CreateMultiDimensional().
+ */
+GDALDatasetH GDALCreateMultiDimensional( GDALDriverH hDriver,
+                                                 const char * pszName,
+                                                 CSLConstList papszRootGroupOptions,
+                                                 CSLConstList papszOptions )
+{
+    VALIDATE_POINTER1( hDriver, __func__, nullptr );
+    VALIDATE_POINTER1( pszName, __func__, nullptr );
+    return GDALDataset::ToHandle(
+        GDALDriver::FromHandle(hDriver)->CreateMultiDimensional(
+            pszName, papszRootGroupOptions, papszOptions));
+}
+
+/************************************************************************/
+/*                  DefaultCreateCopyMultiDimensional()                 */
+/************************************************************************/
+
+//! @cond Doxygen_Suppress
+
+CPLErr GDALDriver::DefaultCreateCopyMultiDimensional(
+                                     GDALDataset *poSrcDS,
+                                     GDALDataset *poDstDS,
+                                     bool bStrict,
+                                     CSLConstList papszOptions,
+                                     GDALProgressFunc pfnProgress,
+                                     void * pProgressData )
+{
+    if( pfnProgress == nullptr )
+        pfnProgress = GDALDummyProgress;
+
+    auto poSrcRG = poSrcDS->GetRootGroup();
+    if( !poSrcRG )
+        return CE_Failure;
+    auto poDstRG = poDstDS->GetRootGroup();
+    if( !poDstRG )
+        return CE_Failure;
+    GUInt64 nCurCost = 0;
+    return poDstRG->CopyFrom(poDstRG, poSrcDS, poSrcRG, bStrict,
+                             nCurCost, poSrcRG->GetTotalCopyCost(),
+                             pfnProgress, pProgressData, papszOptions) ?
+                CE_None : CE_Failure;
+}
+//! @endcond
+
+
 /************************************************************************/
 /*                          DefaultCopyMasks()                          */
 /************************************************************************/
@@ -327,6 +419,21 @@ CPLErr GDALDriver::DefaultCopyMasks( GDALDataset *poSrcDS,
                                      int bStrict )
 
 {
+    return DefaultCopyMasks(poSrcDS, poDstDS, bStrict,
+                            nullptr, nullptr, nullptr);
+}
+
+CPLErr GDALDriver::DefaultCopyMasks( GDALDataset *poSrcDS,
+                                     GDALDataset *poDstDS,
+                                     int bStrict,
+                                     CSLConstList /*papszOptions*/,
+                                     GDALProgressFunc pfnProgress,
+                                     void * pProgressData )
+
+{
+    if( pfnProgress == nullptr )
+        pfnProgress = GDALDummyProgress;
+
     int nBands = poSrcDS->GetRasterCount();
     if (nBands == 0)
         return CE_None;
@@ -337,6 +444,20 @@ CPLErr GDALDriver::DefaultCopyMasks( GDALDataset *poSrcDS,
     const char* papszOptions[2] = { "COMPRESSED=YES", nullptr };
     CPLErr eErr = CE_None;
 
+    int nTotalBandsWithMask = 0;
+    for( int iBand = 0; iBand < nBands; ++iBand )
+    {
+        GDALRasterBand *poSrcBand = poSrcDS->GetRasterBand( iBand+1 );
+
+        int nMaskFlags = poSrcBand->GetMaskFlags();
+        if( !(nMaskFlags &
+                 (GMF_ALL_VALID|GMF_PER_DATASET|GMF_ALPHA|GMF_NODATA) ) )
+        {
+            nTotalBandsWithMask ++;
+        }
+    }
+
+    int iBandWithMask = 0;
     for( int iBand = 0;
          eErr == CE_None && iBand < nBands;
          ++iBand )
@@ -354,11 +475,17 @@ CPLErr GDALDriver::DefaultCopyMasks( GDALDataset *poSrcDS,
                 eErr = poDstBand->CreateMaskBand( nMaskFlags );
                 if( eErr == CE_None )
                 {
+                    // coverity[divide_by_zero]
+                    void* pScaledData = GDALCreateScaledProgress(
+                        double(iBandWithMask) / nTotalBandsWithMask,
+                        double(iBandWithMask + 1) / nTotalBandsWithMask,
+                        pfnProgress, pProgressData );
                     eErr = GDALRasterBandCopyWholeRaster(
                         poSrcBand->GetMaskBand(),
                         poDstBand->GetMaskBand(),
                         papszOptions,
-                        GDALDummyProgress, nullptr);
+                        GDALScaledProgress, pScaledData);
+                    GDALDestroyScaledProgress(pScaledData);
                 }
                 else if( !bStrict )
                 {
@@ -383,7 +510,7 @@ CPLErr GDALDriver::DefaultCopyMasks( GDALDataset *poSrcDS,
                 poSrcDS->GetRasterBand(1)->GetMaskBand(),
                 poDstDS->GetRasterBand(1)->GetMaskBand(),
                 papszOptions,
-                GDALDummyProgress, nullptr);
+                pfnProgress, pProgressData);
         }
         else if( !bStrict )
         {
@@ -410,6 +537,37 @@ GDALDataset *GDALDriver::DefaultCreateCopy( const char * pszFilename,
 
     CPLErrorReset();
 
+/* -------------------------------------------------------------------- */
+/*      Use multidimensional raster API if available.                   */
+/* -------------------------------------------------------------------- */
+    auto poSrcGroup = poSrcDS->GetRootGroup();
+    if( poSrcGroup != nullptr && GetMetadataItem(GDAL_DCAP_MULTIDIM_RASTER) )
+    {
+        CPLStringList aosDatasetCO;
+        for( CSLConstList papszIter = papszOptions; papszIter && *papszIter; ++papszIter )
+        {
+            if( !STARTS_WITH_CI(*papszIter, "ARRAY:") )
+                aosDatasetCO.AddString(*papszIter);
+        }
+        auto poDstDS = std::unique_ptr<GDALDataset>(
+            CreateMultiDimensional(pszFilename,
+                                   nullptr,
+                                   aosDatasetCO.List()));
+        if( !poDstDS )
+            return nullptr;
+        auto poDstGroup = poDstDS->GetRootGroup();
+        if( !poDstGroup )
+            return nullptr;
+        if( DefaultCreateCopyMultiDimensional(poSrcDS,
+                                              poDstDS.get(),
+                                              CPL_TO_BOOL(bStrict),
+                                              papszOptions,
+                                              pfnProgress,
+                                              pProgressData) != CE_None )
+            return nullptr;
+        return poDstDS.release();
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Validate that we can create the output as requested.            */
 /* -------------------------------------------------------------------- */
@@ -589,6 +747,15 @@ GDALDataset *GDALDriver::DefaultCreateCopy( const char * pszFilename,
     if( papszMD )
         poDstDS->SetMetadata( papszMD, "RPC" );
 
+/* -------------------------------------------------------------------- */
+/*      Copy XMPmetadata.                                               */
+/* -------------------------------------------------------------------- */
+    char** papszXMP = poSrcDS->GetMetadata("xml:XMP");
+    if (papszXMP != nullptr && *papszXMP != nullptr)
+    {
+        poDstDS->SetMetadata(papszXMP, "xml:XMP");
+    }
+    
 /* -------------------------------------------------------------------- */
 /*      Loop copying bands.                                             */
 /* -------------------------------------------------------------------- */
@@ -752,8 +919,9 @@ GDALDataset *GDALDriver::DefaultCreateCopy( const char * pszFilename,
  * file handle, but also ensures that all the data and metadata has been written
  * to the dataset (GDALFlushCache() is not sufficient for that purpose).
  *
- * In some situations, the new dataset can be created in another process through
- * the \ref gdal_api_proxy mechanism.
+ * For multidimensional datasets, papszOptions can contain array creation options,
+ * if they are prefixed with "ARRAY:". \see GDALGroup::CopyFrom() documentation
+ * for further details regarding such options.
  *
  * @param pszFilename the name for the new dataset.  UTF-8 encoded.
  * @param poSrcDS the dataset being duplicated.
@@ -779,37 +947,6 @@ GDALDataset *GDALDriver::CreateCopy( const char * pszFilename,
     if( pfnProgress == nullptr )
         pfnProgress = GDALDummyProgress;
 
-    const char* pszClientFilename = GDALClientDatasetGetFilename(pszFilename);
-    if( pszClientFilename != nullptr && !EQUAL(GetDescription(), "MEM") &&
-        !EQUAL(GetDescription(), "VRT") )
-    {
-        GDALDriver* poAPIPROXYDriver = GDALGetAPIPROXYDriver();
-        if( poAPIPROXYDriver != this )
-        {
-            if( poAPIPROXYDriver->pfnCreateCopy == nullptr )
-                return nullptr;
-            char** papszOptionsDup = CSLDuplicate(papszOptions);
-            papszOptionsDup = CSLAddNameValue(papszOptionsDup, "SERVER_DRIVER",
-                                              GetDescription());
-            GDALDataset* const poDstDS = poAPIPROXYDriver->pfnCreateCopy(
-                pszClientFilename, poSrcDS, bStrict, papszOptionsDup,
-                pfnProgress, pProgressData);
-            if( poDstDS != nullptr )
-            {
-                if( poDstDS->GetDescription() == nullptr
-                    || strlen(poDstDS->GetDescription()) == 0 )
-                    poDstDS->SetDescription( pszFilename );
-
-                if( poDstDS->poDriver == nullptr )
-                    poDstDS->poDriver = poAPIPROXYDriver;
-            }
-
-            CSLDestroy(papszOptionsDup);
-            if( poDstDS != nullptr || CPLGetLastErrorNo() != CPLE_NotSupported )
-                return poDstDS;
-        }
-    }
-
 /* -------------------------------------------------------------------- */
 /*      Make sure we cleanup if there is an existing dataset of this    */
 /*      name.  But even if that seems to fail we will continue since    */
@@ -826,7 +963,7 @@ GDALDataset *GDALDriver::CreateCopy( const char * pszFilename,
         // memory driver doesn't expect files with those names to be deleted
         // on a file system...
         // This is somewhat messy. Ideally there should be a way for the
-        // driver to overload the default behaviour
+        // driver to overload the default behavior
         if( !EQUAL(GetDescription(), "MEM") &&
             !EQUAL(GetDescription(), "Memory") )
         {
@@ -839,7 +976,7 @@ GDALDataset *GDALDriver::CreateCopy( const char * pszFilename,
         CSLPartialFindString(papszOptions, "QUIET_DELETE_ON_CREATE_COPY=");
     if( iIdxQuietDeleteOnCreateCopy >= 0 )
     {
-        if( papszOptionsToDelete == nullptr )
+        //if( papszOptionsToDelete == nullptr )
             papszOptionsToDelete = CSLDuplicate(papszOptions);
         papszOptions =
             CSLRemoveStrings(papszOptionsToDelete, iIdxQuietDeleteOnCreateCopy,
@@ -872,7 +1009,23 @@ GDALDataset *GDALDriver::CreateCopy( const char * pszFilename,
 /* -------------------------------------------------------------------- */
     if( CPLTestBool(
             CPLGetConfigOption("GDAL_VALIDATE_CREATION_OPTIONS", "YES") ) )
-        GDALValidateCreationOptions( this, papszOptions );
+    {
+        auto poSrcGroup = poSrcDS->GetRootGroup();
+        if( poSrcGroup != nullptr && GetMetadataItem(GDAL_DCAP_MULTIDIM_RASTER) )
+        {
+            CPLStringList aosDatasetCO;
+            for( CSLConstList papszIter = papszOptions; papszIter && *papszIter; ++papszIter )
+            {
+                if( !STARTS_WITH_CI(*papszIter, "ARRAY:") )
+                    aosDatasetCO.AddString(*papszIter);
+            }
+            GDALValidateCreationOptions( this, aosDatasetCO.List() );
+        }
+        else
+        {
+            GDALValidateCreationOptions( this, papszOptions );
+        }
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Advise the source raster that we are going to read it completely */
@@ -970,12 +1123,19 @@ GDALDatasetH CPL_STDCALL GDALCreateCopy( GDALDriverH hDriver,
  * using Identify().
  *
  * @param pszName the dataset name to try and delete.
+ * @param papszAllowedDrivers NULL to consider all candidate drivers, or a NULL
+ * terminated list of strings with the driver short names that must be
+ * considered. (Note: functionality currently broken. Argument considered as NULL)
  * @return CE_None if the dataset does not exist, or is deleted without issues.
  */
 
-CPLErr GDALDriver::QuietDelete( const char *pszName )
+CPLErr GDALDriver::QuietDelete( const char *pszName,
+                                const char *const *papszAllowedDrivers )
 
 {
+    // FIXME! GDALIdentifyDriver() accepts a file list, not a driver list
+    CPL_IGNORE_RET_VAL(papszAllowedDrivers);
+
     VSIStatBufL sStat;
     const bool bExists =
         VSIStatExL(pszName, &sStat,
@@ -1028,8 +1188,8 @@ CPLErr GDALDriver::QuietDelete( const char *pszName )
  *
  * The driver will attempt to delete the named dataset in a driver specific
  * fashion.  Full featured drivers will delete all associated files,
- * database objects, or whatever is appropriate.  The default behaviour when
- * no driver specific behaviour is provided is to attempt to delete all the
+ * database objects, or whatever is appropriate.  The default behavior when
+ * no driver specific behavior is provided is to attempt to delete all the
  * files that are returned by GDALGetFileList() on the dataset handle.
  *
  * It is unwise to have open dataset handles on this dataset when it is
@@ -2001,24 +2161,19 @@ GDALIdentifyDriverEx( const char* pszFilename,
     const int nDriverCount = poDM->GetDriverCount();
 
     // First pass: only use drivers that have a pfnIdentify implementation.
-    for( int iDriver = -1; iDriver < nDriverCount; ++iDriver )
+    for( int iDriver = 0; iDriver < nDriverCount; ++iDriver )
     {
-        GDALDriver* poDriver;
-
-        if( iDriver < 0 )
-            poDriver = GDALGetAPIPROXYDriver();
-        else
+        GDALDriver* poDriver = poDM->GetDriver( iDriver );
+        if (papszAllowedDrivers != nullptr &&
+            CSLFindString(papszAllowedDrivers,
+                            GDALGetDriverShortName(poDriver)) == -1)
         {
-            poDriver = poDM->GetDriver( iDriver );
-            if (papszAllowedDrivers != nullptr &&
-                CSLFindString(papszAllowedDrivers,
-                              GDALGetDriverShortName(poDriver)) == -1)
-                continue;
+            continue;
         }
 
         VALIDATE_POINTER1( poDriver, "GDALIdentifyDriver", nullptr );
 
-        if( poDriver->pfnIdentify == nullptr )
+        if( poDriver->pfnIdentify == nullptr && poDriver->pfnIdentifyEx == nullptr)
         {
             continue;
         }
@@ -2036,23 +2191,26 @@ GDALIdentifyDriverEx( const char* pszFilename,
             poDriver->GetMetadataItem(GDAL_DCAP_VECTOR) == nullptr )
             continue;
 
-        if( poDriver->pfnIdentify( &oOpenInfo ) > 0 )
-            return poDriver;
+        if( poDriver->pfnIdentifyEx )
+        {
+            if( poDriver->pfnIdentifyEx( poDriver, &oOpenInfo ) > 0 )
+                return poDriver;
+        }
+        else
+        {
+            if( poDriver->pfnIdentify( &oOpenInfo ) > 0 )
+                return poDriver;
+        }
     }
 
     // Second pass: slow method.
-    for( int iDriver = -1; iDriver < nDriverCount; ++iDriver )
+    for( int iDriver = 0; iDriver < nDriverCount; ++iDriver )
     {
-        GDALDriver* poDriver;
-
-        if( iDriver < 0 )
-            poDriver = GDALGetAPIPROXYDriver();
-        else
+        GDALDriver* poDriver = poDM->GetDriver( iDriver );
+        if (papszAllowedDrivers != nullptr &&
+            CSLFindString(papszAllowedDrivers,
+                            GDALGetDriverShortName(poDriver)) == -1)
         {
-            poDriver = poDM->GetDriver( iDriver );
-            if (papszAllowedDrivers != nullptr &&
-                CSLFindString(papszAllowedDrivers,
-                              GDALGetDriverShortName(poDriver)) == -1)
                 continue;
         }
 
@@ -2067,7 +2225,12 @@ GDALIdentifyDriverEx( const char* pszFilename,
             poDriver->GetMetadataItem(GDAL_DCAP_VECTOR) == nullptr )
             continue;
 
-        if( poDriver->pfnIdentify != nullptr )
+        if( poDriver->pfnIdentifyEx != nullptr )
+        {
+            if( poDriver->pfnIdentifyEx( poDriver, &oOpenInfo ) == 0 )
+                continue;
+        }
+        else if( poDriver->pfnIdentify != nullptr )
         {
             if( poDriver->pfnIdentify( &oOpenInfo ) == 0 )
                 continue;
