@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $Id: mkgdaldist.sh ed89c6d75ce36c0528a80e59d46766736ec93818 2018-06-22 10:35:28 +0200 Even Rouault $
+# $Id: mkgdaldist.sh e19989c4b038be832b2f2ddeffd220a4d3cd8f6f 2021-04-01 21:30:04 +0300 Idan Miara $
 #
 # mkgdaldist.sh - prepares GDAL source distribution package
 
@@ -143,15 +143,22 @@ if test -d "man"; then
     rm -rf man
 fi
 
-(cat Doxyfile ; echo "ENABLED_SECTIONS=man"; echo "INPUT=apps swig/python/scripts"; echo "FILE_PATTERNS=*.cpp *.dox"; echo "GENERATE_HTML=NO"; echo "GENERATE_MAN=YES"; echo "QUIET=YES") | doxygen -
+if test -f "doc/Makefile"; then
+    (cd doc; make man)
+    mkdir -p man/man1
+    cp doc/build/man/*.1 man/man1
+    rm -rf doc/build
+    rm -f doc/.doxygen_up_to_date
+else
+    (cat Doxyfile ; echo "ENABLED_SECTIONS=man"; echo "INPUT=apps swig/python/gdal-utils/scripts"; echo "FILE_PATTERNS=*.cpp *.dox"; echo "GENERATE_HTML=NO"; echo "GENERATE_MAN=YES"; echo "QUIET=YES") | doxygen -
+    rm -f doxygen_sqlite3.db
+    rm -f man/man1/*_dist_wrk_gdal_gdal_apps_.1
+fi
 
 if test ! -d "man"; then
     echo " make man failed"
 fi
 
-rm -f doxygen_sqlite3.db
-rm -f man/man1/*_dist_wrk_gdal_gdal_apps_.1
-
 cd "$CWD"
 
 # They currently require SWIG 1.3.X, which is not convenient as we need
