@@ -38,7 +38,7 @@
 #include "ogr_p.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: ogrcompoundcurve.cpp 1b132b40204d500e99c91f6a7986b44b6a065de8 2018-04-10 15:04:42 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogrcompoundcurve.cpp 3798cbe48457b7127606931896549f26507469db 2021-04-09 15:04:16 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                         OGRCompoundCurve()                           */
@@ -48,7 +48,7 @@ CPL_CVSID("$Id: ogrcompoundcurve.cpp 1b132b40204d500e99c91f6a7986b44b6a065de8 20
  * \brief Create an empty compound curve.
  */
 
-OGRCompoundCurve::OGRCompoundCurve() {}
+OGRCompoundCurve::OGRCompoundCurve() = default;
 
 /************************************************************************/
 /*             OGRCompoundCurve( const OGRCompoundCurve& )              */
@@ -63,16 +63,13 @@ OGRCompoundCurve::OGRCompoundCurve() {}
  * @since GDAL 2.1
  */
 
-OGRCompoundCurve::OGRCompoundCurve( const OGRCompoundCurve& other ) :
-    OGRCurve(other),
-    oCC(other.oCC)
-{}
+OGRCompoundCurve::OGRCompoundCurve( const OGRCompoundCurve& ) = default;
 
 /************************************************************************/
 /*                         ~OGRCompoundCurve()                          */
 /************************************************************************/
 
-OGRCompoundCurve::~OGRCompoundCurve() {}
+OGRCompoundCurve::~OGRCompoundCurve() = default;
 
 /************************************************************************/
 /*                 operator=( const OGRCompoundCurve&)                  */
@@ -98,6 +95,16 @@ OGRCompoundCurve& OGRCompoundCurve::operator=( const OGRCompoundCurve& other )
     return *this;
 }
 
+/************************************************************************/
+/*                               clone()                                */
+/************************************************************************/
+
+OGRCompoundCurve *OGRCompoundCurve::clone() const
+
+{
+    return new (std::nothrow) OGRCompoundCurve(*this);
+}
+
 /************************************************************************/
 /*                          getGeometryType()                           */
 /************************************************************************/
@@ -128,7 +135,7 @@ const char * OGRCompoundCurve::getGeometryName() const
 /************************************************************************/
 /*                              WkbSize()                               */
 /************************************************************************/
-int OGRCompoundCurve::WkbSize() const
+size_t OGRCompoundCurve::WkbSize() const
 {
     return oCC.WkbSize();
 }
@@ -149,12 +156,12 @@ OGRErr OGRCompoundCurve::addCurveDirectlyFromWkb( OGRGeometry* poSelf,
 /************************************************************************/
 
 OGRErr OGRCompoundCurve::importFromWkb( const unsigned char * pabyData,
-                                        int nSize,
+                                        size_t nSize,
                                         OGRwkbVariant eWkbVariant,
-                                       int& nBytesConsumedOut )
+                                        size_t& nBytesConsumedOut )
 {
     OGRwkbByteOrder eByteOrder = wkbNDR;
-    int nDataOffset = 0;
+    size_t nDataOffset = 0;
     // coverity[tainted_data]
     OGRErr eErr = oCC.importPreambleFromWkb(this, pabyData, nSize, nDataOffset,
                                              eByteOrder, 9, eWkbVariant);
@@ -162,7 +169,7 @@ OGRErr OGRCompoundCurve::importFromWkb( const unsigned char * pabyData,
         return eErr;
 
     eErr =  oCC.importBodyFromWkb(this, pabyData + nDataOffset, nSize,
-                                 FALSE,  // bAcceptCompoundCurve
+                                 false,  // bAcceptCompoundCurve
                                  addCurveDirectlyFromWkb,
                                  eWkbVariant,
                                  nBytesConsumedOut);
@@ -211,29 +218,10 @@ OGRErr OGRCompoundCurve::importFromWkt( const char ** ppszInput )
 /************************************************************************/
 /*                            exportToWkt()                             */
 /************************************************************************/
-OGRErr OGRCompoundCurve::exportToWkt( char ** ppszDstText,
-                                      OGRwkbVariant /* eWkbVariant */ ) const
-
+std::string OGRCompoundCurve::exportToWkt(const OGRWktOptions& opts,
+                                          OGRErr *err) const
 {
-    return oCC.exportToWkt(this, ppszDstText);
-}
-
-/************************************************************************/
-/*                               clone()                                */
-/************************************************************************/
-
-OGRGeometry *OGRCompoundCurve::clone() const
-{
-    OGRCompoundCurve *poNewCC = new OGRCompoundCurve;
-    poNewCC->assignSpatialReference( getSpatialReference() );
-    poNewCC->flags = flags;
-
-    for( int i = 0; i < oCC.nCurveCount; i++ )
-    {
-        poNewCC->addCurve( oCC.papoCurves[i] );
-    }
-
-    return poNewCC;
+    return oCC.exportToWkt(this, opts, err);
 }
 
 /************************************************************************/
@@ -526,7 +514,7 @@ OGRCurve* OGRCompoundCurve::stealCurve( int iCurve )
 
 OGRErr OGRCompoundCurve::addCurve( OGRCurve* poCurve, double dfToleranceEps )
 {
-    OGRCurve* poClonedCurve = poCurve->clone()->toCurve();
+    OGRCurve* poClonedCurve = poCurve->clone();
     const OGRErr eErr = addCurveDirectly( poClonedCurve, dfToleranceEps );
     if( eErr != OGRERR_NONE )
         delete poClonedCurve;
@@ -705,16 +693,18 @@ int OGRCompoundCurve::getNumPoints() const
 
 class OGRCompoundCurvePointIterator final: public OGRPointIterator
 {
-        const OGRCompoundCurve *poCC;
-        int                     iCurCurve;
-        OGRPointIterator       *poCurveIter;
+        CPL_DISALLOW_COPY_ASSIGN(OGRCompoundCurvePointIterator)
+
+        const OGRCompoundCurve *poCC = nullptr;
+        int                     iCurCurve = 0;
+        OGRPointIterator       *poCurveIter = nullptr;
 
     public:
         explicit OGRCompoundCurvePointIterator( const OGRCompoundCurve* poCCIn ) :
-            poCC(poCCIn), iCurCurve(0), poCurveIter(nullptr) {}
-        virtual ~OGRCompoundCurvePointIterator() { delete poCurveIter; }
+            poCC(poCCIn) {}
+        ~OGRCompoundCurvePointIterator() override { delete poCurveIter; }
 
-        virtual OGRBoolean getNextPoint( OGRPoint* p ) override;
+        OGRBoolean getNextPoint( OGRPoint* p ) override;
 };
 
 /************************************************************************/
