@@ -31,6 +31,7 @@
 #include "io.h"
 
 #include <iostream>
+#include <cstdint>
 #include <cstring>
 
 const size_t DWGConstants::SentinelLength = 16;
@@ -256,10 +257,10 @@ double CADBuffer::ReadBITDOUBLE()
 
             m_nBitOffsetFromStart += 64;
 
-            void   * ptr    = aDoubleBytes;
-            double * result = static_cast< double *> ( ptr );
-
-            return * result;
+            double result;
+            memcpy(&result, aDoubleBytes, sizeof(result));
+            FromLSB(result);
+            return result;
         }
 
         case BITDOUBLE_ONE_VALUE:
@@ -337,12 +338,13 @@ short CADBuffer::ReadRAWSHORT()
             break;
     }
 
-    void  * ptr    = aShortBytes;
-    short * result = static_cast<short *>(ptr);
+    int16_t result;
+    memcpy(&result, aShortBytes, sizeof(result));
+    FromLSB(result);
 
     m_nBitOffsetFromStart += 16;
 
-    return * result;
+    return result;
 }
 
 double CADBuffer::ReadRAWDOUBLE()
@@ -384,12 +386,13 @@ double CADBuffer::ReadRAWDOUBLE()
             break;
     }
 
-    void   * ptr    = aDoubleBytes;
-    double * result = static_cast<double *>(ptr);
+    double result;
+    memcpy(&result, aDoubleBytes, sizeof(result));
+    FromLSB(result);
 
     m_nBitOffsetFromStart += 64;
 
-    return * result;
+    return result;
 }
 
 int CADBuffer::ReadRAWLONG()
@@ -423,12 +426,13 @@ int CADBuffer::ReadRAWLONG()
             break;
     }
 
-    void * ptr    = aLongBytes;
-    int  * result = static_cast<int *>(ptr);
+    int32_t result;
+    memcpy(&result, aLongBytes, sizeof(result));
+    FromLSB(result);
 
     m_nBitOffsetFromStart += 32;
 
-    return * result;
+    return result;
 }
 
 bool CADBuffer::ReadBIT()
@@ -475,10 +479,11 @@ short CADBuffer::ReadBITSHORT()
 
             m_nBitOffsetFromStart += 16;
 
-            void  * ptr    = aShortBytes;
-            short * result = static_cast < short * > ( ptr );
+            int16_t result;
+            memcpy(&result, aShortBytes, sizeof(result));
+            FromLSB(result);
 
-            return * result;
+            return result;
         }
 
         case BITSHORT_UNSIGNED_CHAR:
@@ -633,7 +638,6 @@ long CADBuffer::ReadMCHAR()
 
 unsigned int CADBuffer::ReadMSHORT()
 {
-    unsigned int  result = 0;
     unsigned char aMShortBytes[8]; // 8 bytes is maximum.
 
     // TODO: this function does not support MSHORTS longer than 4 bytes. ODA says
@@ -654,7 +658,7 @@ unsigned int CADBuffer::ReadMSHORT()
     {
         aMShortBytes[0] &= binary( 01111111 ); // drop high order flag bit.
     }
-    else if ( MShortBytesCount == 4 )
+    else
     {
         aMShortBytes[0] &= binary( 01111111 );
         aMShortBytes[2] &= binary( 01111111 );
@@ -664,8 +668,18 @@ unsigned int CADBuffer::ReadMSHORT()
         aMShortBytes[1] |= ( aMShortBytes[0] << 7 );
         aMShortBytes[0] = ( aMShortBytes[0] >> 1 );
     }
-    SwapEndianness( aMShortBytes, MShortBytesCount ); // MSB to LSB
-    memcpy( & result, aMShortBytes, MShortBytesCount );
+
+    unsigned int result;
+    if( MShortBytesCount == 2 )
+    {
+        result = (aMShortBytes[0] << 8) | aMShortBytes[1];
+    }
+    else
+    {
+        result = (static_cast<unsigned>(aMShortBytes[0]) << 24) |
+                 (aMShortBytes[1] << 16) | (aMShortBytes[2] << 8) | aMShortBytes[3];
+    }
+
     return result;
 }
 
@@ -692,10 +706,10 @@ double CADBuffer::ReadBITDOUBLEWD(double defaultvalue )
             aDefaultValueBytes[2] = ReadCHAR();
             aDefaultValueBytes[3] = ReadCHAR();
 
-            void   * ptr    = aDefaultValueBytes;
-            double * result = static_cast< double *> ( ptr );
-
-            return * result;
+            double result;
+            memcpy(&result, aDefaultValueBytes, sizeof(result));
+            FromLSB(result);
+            return result;
         }
 
         case BITDOUBLEWD_6BYTES_PATCHED:
@@ -707,10 +721,10 @@ double CADBuffer::ReadBITDOUBLEWD(double defaultvalue )
             aDefaultValueBytes[2] = ReadCHAR();
             aDefaultValueBytes[3] = ReadCHAR();
 
-            void   * ptr    = aDefaultValueBytes;
-            double * result = static_cast< double *> ( ptr );
-
-            return * result;
+            double result;
+            memcpy(&result, aDefaultValueBytes, sizeof(result));
+            FromLSB(result);
+            return result;
         }
 
         case BITDOUBLEWD_FULL_RD:
@@ -724,10 +738,10 @@ double CADBuffer::ReadBITDOUBLEWD(double defaultvalue )
             aDefaultValueBytes[6] = ReadCHAR();
             aDefaultValueBytes[7] = ReadCHAR();
 
-            void   * ptr    = aDefaultValueBytes;
-            double * result = static_cast< double *> ( ptr );
-
-            return * result;
+            double result;
+            memcpy(&result, aDefaultValueBytes, sizeof(result));
+            FromLSB(result);
+            return result;
         }
     }
 
@@ -797,10 +811,10 @@ int CADBuffer::ReadBITLONG()
 
             m_nBitOffsetFromStart += 32;
 
-            void * ptr    = aLongBytes;
-            int  * result = static_cast < int * > ( ptr );
-
-            return * result;
+            int32_t result;
+            memcpy(&result, aLongBytes, sizeof(result));
+            FromLSB(result);
+            return result;
         }
 
         case BITLONG_UNSIGNED_CHAR:
@@ -821,8 +835,10 @@ int CADBuffer::ReadBITLONG()
 
         case BITLONG_NOT_USED:
         {
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
             std::cerr <<
             "THAT SHOULD NEVER HAPPENED! BUG. (in file, or reader, or both.) ReadBITLONG(), case BITLONG_NOT_USED\n";
+#endif
             m_nBitOffsetFromStart += 0;
             return 0;
         }
@@ -834,6 +850,13 @@ int CADBuffer::ReadBITLONG()
 void CADBuffer::SkipTV()
 {
     short stringLength = ReadBITSHORT();
+    if( stringLength < 0 )
+    {
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+        std::cerr << "Negative string length" << std::endl;
+#endif
+        return;
+    }
     m_nBitOffsetFromStart += size_t( stringLength * 8 );
 }
 
