@@ -8,7 +8,7 @@
  ******************************************************************************
  * Copyright (c) 2010, Ragi Yaser Burhum
  * Copyright (c) 2011, Paul Ramsey <pramsey at cleverelephant.ca>
- * Copyright (c) 2011-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2011-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -36,7 +36,9 @@
 #include "FGdbUtils.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: FGdbDatasource.cpp 7e07230bbff24eb333608de4dbd460b7312839d0 2017-12-11 19:08:47Z Even Rouault $")
+#include "filegdb_fielddomain.h"
+
+CPL_CVSID("$Id: FGdbDatasource.cpp 96222122f96148e93f6a92c00ddc5b7d76de4edd 2021-04-08 13:09:47 +0200 Even Rouault $")
 
 using std::vector;
 using std::wstring;
@@ -162,16 +164,7 @@ int FGdbDataSource::Open(const char * pszNewName, int bUpdate,
     m_bUpdate = CPL_TO_BOOL(bUpdate);
     m_poOpenFileGDBDrv = (GDALDriver*) GDALGetDriverByName("OpenFileGDB");
 
-    std::vector<std::wstring> typesRequested;
-
-    // We're only interested in Tables, Feature Datasets and Feature Classes
-    typesRequested.push_back(L"Feature Class");
-    typesRequested.push_back(L"Table");
-    typesRequested.push_back(L"Feature Dataset");
-
-    bool rv = LoadLayers(L"\\");
-
-    return rv;
+    return LoadLayers(L"\\");
 }
 
 /************************************************************************/
@@ -260,6 +253,13 @@ int FGdbDataSource::ReOpen()
 bool FGdbDataSource::OpenFGDBTables(const std::wstring &type,
                                     const std::vector<std::wstring> &layers)
 {
+#ifdef DISPLAY_RELATED_DATASETS
+    std::vector<std::wstring> relationshipTypes;
+    m_pGeodatabase->GetDatasetRelationshipTypes(relationshipTypes);
+    std::vector<std::wstring> datasetTypes;
+    m_pGeodatabase->GetDatasetTypes(datasetTypes);
+#endif
+
     fgdbError hr;
     for ( unsigned int i = 0; i < layers.size(); i++ )
     {
@@ -287,6 +287,43 @@ bool FGdbDataSource::OpenFGDBTables(const std::wstring &type,
                    (". Skipping it. " + CPLString(pszLikelyReason)).c_str());
             continue;
         }
+
+#ifdef DISPLAY_RELATED_DATASETS
+        CPLDebug("FGDB", "Finding datasets related to %s",
+                 WStringToString(layers[i]).c_str());
+        // Slow !
+        for ( const auto& relType: relationshipTypes )
+        {
+            for ( const auto& itemType: datasetTypes )
+            {
+                std::vector<std::wstring> relatedDatasets;
+                m_pGeodatabase->GetRelatedDatasets(
+                    layers[i], relType, itemType, relatedDatasets);
+
+                std::vector<std::string> relatedDatasetDefs;
+                m_pGeodatabase->GetRelatedDatasetDefinitions(
+                    layers[i], relType, itemType, relatedDatasetDefs);
+
+                if( !relatedDatasets.empty() || !relatedDatasetDefs.empty())
+                {
+                    CPLDebug("FGDB", "relationshipType: %s",
+                             WStringToString(relType).c_str());
+                    CPLDebug("FGDB", "itemType: %s",
+                             WStringToString(itemType).c_str());
+                }
+                for( const auto& name: relatedDatasets )
+                {
+                    CPLDebug("FGDB", "Related dataset: %s",
+                             WStringToString(name).c_str());
+                }
+                for( const auto& xml: relatedDatasetDefs )
+                {
+                    CPLDebug("FGDB", "Related dataset def: %s", xml.c_str());
+                }
+            }
+        }
+#endif
+
         FGdbLayer* pLayer = new FGdbLayer();
         if (!pLayer->Initialize(this, pTable, layers[i], type))
         {
@@ -343,86 +380,10 @@ bool FGdbDataSource::LoadLayers(const std::wstring &root)
         if ( !featureclasses.empty() && ! OpenFGDBTables(L"Feature Class", featureclasses) )
             return false;
     }
+
     return true;
 }
 
-#if 0
-/************************************************************************/
-/*                            LoadLayersOld()                              */
-/************************************************************************/
-
-/* Old recursive LoadLayers. Removed in favor of simple one that only
-   looks at FeatureClasses and Tables. */
-
-// Flattens out hierarchical GDB structure.
-bool FGdbDataSource::LoadLayersOld(const std::vector<wstring> & datasetTypes,
-                                const wstring & parent)
-{
-    long hr = S_OK;
-
-    // I didn't find an API to give me the type of the dataset based on name - I am *not*
-    // parsing XML for something like this - in the meantime I can use this hack to see
-    // if the dataset had any children whatsoever - if so, then I won't attempt to open it
-    // otherwise, do attempt to do that
-
-    bool childrenFound = false;
-    bool errorsEncountered = false;
-
-    for (size_t dsTypeIndex = 0; dsTypeIndex < datasetTypes.size(); dsTypeIndex++)
-    {
-        std::vector<wstring> childDatasets;
-        m_pGeodatabase->GetChildDatasets( parent, datasetTypes[dsTypeIndex], childDatasets);
-
-        if (!childDatasets.empty())
-        {
-            //it is a container of other datasets
-
-            for (size_t childDatasetIndex = 0;
-                 childDatasetIndex < childDatasets.size();
-                 childDatasetIndex++)
-            {
-                childrenFound = true;
-
-                // do something with it
-                // For now, we just ignore dataset containers and only open the children
-                // std::wcout << datasetTypes[dsTypeIndex] << L" " << childDatasets[childDatasetIndex] << std::endl;
-
-                if (!LoadLayersOld(datasetTypes, childDatasets[childDatasetIndex]))
-                    errorsEncountered = true;
-            }
-        }
-    }
-
-    //it is a full fledged dataset itself without children - open it (except the root)
-
-    if ((!childrenFound) && parent != L"\\")
-    {
-        // wcout << "Opening " << parent << "...";
-        Table* pTable = new Table;
-        if (FAILED(hr = m_pGeodatabase->OpenTable(parent,*pTable)))
-        {
-            delete pTable;
-            return GDBErr(hr, "Error opening " + WStringToString(parent));
-        }
-
-        FGdbLayer* pLayer = new FGdbLayer;
-
-        //pLayer has ownership of the table pointer as soon Initialize is called
-        if (!pLayer->Initialize(this, pTable, parent))
-        {
-            delete pLayer;
-
-            return GDBErr(hr, "Error initializing OGRLayer for " +
-                          WStringToString(parent));
-        }
-
-        m_layers.push_back(pLayer);
-    }
-
-    return !errorsEncountered;
-}
-#endif
-
 /************************************************************************/
 /*                            DeleteLayer()                             */
 /************************************************************************/
@@ -527,7 +488,7 @@ FGdbDataSource::ICreateLayer( const char * pszLayerName,
 /*                   OGRFGdbSingleFeatureLayer                          */
 /************************************************************************/
 
-class OGRFGdbSingleFeatureLayer : public OGRLayer
+class OGRFGdbSingleFeatureLayer final: public OGRLayer
 {
   private:
     char               *pszVal;
@@ -753,3 +714,29 @@ void FGdbDataSource::SetSymlinkFlagOnAllLayers()
         m_layers[i]->SetSymlinkFlag();
     }
 }
+
+/************************************************************************/
+/*                        GetFieldDomain()                              */
+/************************************************************************/
+
+const OGRFieldDomain* FGdbDataSource::GetFieldDomain(const std::string& name) const
+{
+    const auto baseRet = GDALDataset::GetFieldDomain(name);
+    if( baseRet )
+        return baseRet;
+
+    std::string domainDef;
+    const auto hr = m_pGeodatabase->GetDomainDefinition(StringToWString(name), domainDef);
+    if (FAILED(hr))
+    {
+        GDBErr(hr, "Failed in GetDomainDefinition()");
+        return nullptr;
+    }
+
+    auto poDomain = ParseXMLFieldDomainDef(domainDef);
+    if( !poDomain )
+        return nullptr;
+    const auto domainName = poDomain->GetName();
+    m_oMapFieldDomains[domainName] = std::move(poDomain);
+    return GDALDataset::GetFieldDomain(name);
+}
