@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2007, Mateusz Loskot
- * Copyright (c) 2008-2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -43,8 +43,14 @@
 #include <ogr_api.h>
 #include <ogr_p.h>
 
-CPL_CVSID("$Id: ogrgeojsonwriter.cpp 98dfb4b4012c5ae4621e246e8eb393b3c05a3f48 2018-04-02 22:09:55 +0200 Even Rouault $")
+#include <algorithm>
+#include <cstdint>
 
+CPL_CVSID("$Id: ogrgeojsonwriter.cpp df398e80769422a4bbd5d4a295f4ede443c9fec6 2021-04-04 00:17:15 +0200 Even Rouault $")
+
+static json_object *
+json_object_new_float_with_significant_figures( float fVal,
+                                                int nSignificantFigures );
 
 /************************************************************************/
 /*                         SetRFC7946Settings()                         */
@@ -60,6 +66,28 @@ void OGRGeoJSONWriteOptions::SetRFC7946Settings()
     bCanPatchCoordinatesWithNativeData = false;
     bHonourReservedRFC7946Members = true;
 }
+
+void OGRGeoJSONWriteOptions::SetIDOptions(CSLConstList papszOptions)
+{
+
+    osIDField = CSLFetchNameValueDef(papszOptions, "ID_FIELD", "");
+    const char* pszIDFieldType = CSLFetchNameValue(papszOptions, "ID_TYPE");
+    if( pszIDFieldType )
+    {
+        if( EQUAL(pszIDFieldType, "String") )
+        {
+            bForceIDFieldType = true;
+            eForcedIDFieldType = OFTString;
+        }
+        else if( EQUAL(pszIDFieldType, "Integer") )
+        {
+            bForceIDFieldType = true;
+            eForcedIDFieldType = OFTInteger64;
+        }
+    }
+    bGenerateID = CPL_TO_BOOL(CSLFetchBoolean(papszOptions, "ID_GENERATE", false));
+}
+
 /*! @endcond */
 
 /************************************************************************/
@@ -122,8 +150,8 @@ static bool OGRGeoJSONIsCompatiblePosition( json_object* poJSonCoordinates,
 static void OGRGeoJSONPatchPosition( json_object* poJSonCoordinates,
                                      json_object* poNativeCoordinates )
 {
-    const int nLength = json_object_array_length(poNativeCoordinates);
-    for( int i = 3; i < nLength; i++ )
+    const auto nLength = json_object_array_length(poNativeCoordinates);
+    for( auto i = decltype(nLength){3}; i < nLength; i++ )
     {
         json_object_array_add(poJSonCoordinates,
             json_object_get(
@@ -145,7 +173,7 @@ static bool OGRGeoJSONIsPatchableArray( json_object* poJSonArray,
     if( json_object_get_type(poJSonArray) == json_type_array &&
         json_object_get_type(poNativeArray) == json_type_array )
     {
-        auto nLength = json_object_array_length(poJSonArray);
+        const auto nLength = json_object_array_length(poJSonArray);
         if( nLength == json_object_array_length(poNativeArray) )
         {
             if( nLength > 0 )
@@ -196,10 +224,10 @@ static bool OGRGeoJSONComputePatchableOrCompatibleArrayInternal(
     if( json_object_get_type(poJSonArray) == json_type_array &&
         json_object_get_type(poNativeArray) == json_type_array )
     {
-        auto nLength = json_object_array_length(poJSonArray);
+        const auto nLength = json_object_array_length(poJSonArray);
         if (nLength == json_object_array_length(poNativeArray) )
         {
-            for( decltype(nLength) i=0; i < nLength; i++ )
+            for( auto i = decltype(nLength){0}; i < nLength; i++ )
             {
                 json_object* poJSonChild =
                     json_object_array_get_idx(poJSonArray, i);
@@ -255,8 +283,8 @@ static void OGRGeoJSONPatchArray( json_object* poJSonArray,
         OGRGeoJSONPatchPosition(poJSonArray, poNativeArray);
         return;
     }
-    const int nLength = json_object_array_length(poJSonArray);
-    for( int i = 0; i<nLength; i++ )
+    const auto nLength = json_object_array_length(poJSonArray);
+    for( auto i = decltype(nLength){0}; i<nLength; i++ )
     {
         json_object* poJSonChild = json_object_array_get_idx(poJSonArray, i);
         json_object* poNativeChild =
@@ -327,10 +355,10 @@ static bool OGRGeoJSONIsPatchableGeometry( json_object* poJSonGeometry,
             if( json_object_get_type(poJSonGeometries) == json_type_array &&
                 json_object_get_type(poNativeGeometries) == json_type_array )
             {
-                auto nLength = json_object_array_length(poJSonGeometries);
+                const auto nLength = json_object_array_length(poJSonGeometries);
                 if( nLength == json_object_array_length(poNativeGeometries) )
                 {
-                    for( decltype(nLength) i=0; i < nLength; i++ )
+                    for( auto i = decltype(nLength){0}; i < nLength; i++ )
                     {
                         json_object* poJSonChild =
                             json_object_array_get_idx(poJSonGeometries, i);
@@ -402,8 +430,8 @@ static void OGRGeoJSONPatchGeometry( json_object* poJSonGeometry,
             json_object* poJSonGeometries =
                 CPL_json_object_object_get(poJSonGeometry, "geometries");
             json_object* poNativeGeometries = it.val;
-            int nLength = json_object_array_length(poJSonGeometries);
-            for( int i=0; i < nLength; i++ )
+            const auto nLength = json_object_array_length(poJSonGeometries);
+            for( auto i=decltype(nLength){0}; i < nLength; i++ )
             {
                 json_object* poJSonChild =
                     json_object_array_get_idx(poJSonGeometries, i);
@@ -615,7 +643,7 @@ json_object* OGRGeoJSONWriteFeature( OGRFeature* poFeature,
 
                     if( it.val != nullptr )
                     {
-                        int nIdx = poFeature->GetFieldIndex("id");
+                        int nIdx = poFeature->GetDefnRef()->GetFieldIndexCaseSensitive("id");
                         if( eType == json_type_string &&
                             nIdx >= 0 &&
                             poFeature->GetFieldDefnRef(nIdx)->GetType() == OFTString &&
@@ -657,7 +685,7 @@ json_object* OGRGeoJSONWriteFeature( OGRFeature* poFeature,
 /* -------------------------------------------------------------------- */
     if( !oOptions.osIDField.empty() )
     {
-        int nIdx = poFeature->GetFieldIndex(oOptions.osIDField);
+        int nIdx = poFeature->GetDefnRef()->GetFieldIndexCaseSensitive(oOptions.osIDField);
         if( nIdx >= 0 )
         {
             if( (oOptions.bForceIDFieldType &&
@@ -776,7 +804,14 @@ json_object* OGRGeoJSONWriteAttributes( OGRFeature* poFeature,
     OGRFeatureDefn* poDefn = poFeature->GetDefnRef();
 
     const int nIDField = !oOptions.osIDField.empty() ?
-        poFeature->GetFieldIndex(oOptions.osIDField) : -1;
+        poDefn->GetFieldIndexCaseSensitive(oOptions.osIDField) : -1;
+
+    constexpr int MAX_SIGNIFICANT_DIGITS_FLOAT32 = 8;
+    const int nFloat32SignificantDigits =
+        oOptions.nSignificantFigures >= 0 ?
+                        std::min(oOptions.nSignificantFigures,
+                                 MAX_SIGNIFICANT_DIGITS_FLOAT32):
+                        MAX_SIGNIFICANT_DIGITS_FLOAT32;
 
     const int nFieldCount = poDefn->GetFieldCount();
     for( int nField = 0; nField < nFieldCount; ++nField )
@@ -823,9 +858,32 @@ json_object* OGRGeoJSONWriteAttributes( OGRFeature* poFeature,
         }
         else if( OFTReal == eType )
         {
-            poObjProp = json_object_new_double_with_significant_figures(
-                poFeature->GetFieldAsDouble(nField),
-                oOptions.nSignificantFigures );
+            const double val = poFeature->GetFieldAsDouble(nField);
+            if( !CPLIsFinite(val) )
+            {
+                if( !oOptions.bAllowNonFiniteValues )
+                {
+                    static bool bHasWarned = false;
+                    if( !bHasWarned )
+                    {
+                        bHasWarned = true;
+                        CPLError(CE_Warning, CPLE_AppDefined,
+                                 "NaN of Infinity value found. Skipped");
+                    }
+                    continue;
+                }
+            }
+            if( eSubType == OFSTFloat32 )
+            {
+                poObjProp = json_object_new_float_with_significant_figures(
+                    static_cast<float>(val), nFloat32SignificantDigits );
+            }
+            else
+            {
+                poObjProp = json_object_new_double_with_significant_figures(
+                    val,
+                    oOptions.nSignificantFigures );
+            }
         }
         else if( OFTString == eType )
         {
@@ -885,11 +943,22 @@ json_object* OGRGeoJSONWriteAttributes( OGRFeature* poFeature,
             poObjProp = json_object_new_array();
             for( int i = 0; i < nSize; i++ )
             {
-                json_object_array_add(
-                    poObjProp,
-                    json_object_new_double_with_significant_figures(
-                        padfList[i],
-                        oOptions.nSignificantFigures));
+                if( eSubType == OFSTFloat32 )
+                {
+                    json_object_array_add(
+                        poObjProp,
+                        json_object_new_float_with_significant_figures(
+                            static_cast<float>(padfList[i]),
+                            nFloat32SignificantDigits));
+                }
+                else
+                {
+                    json_object_array_add(
+                        poObjProp,
+                        json_object_new_double_with_significant_figures(
+                            padfList[i],
+                            oOptions.nSignificantFigures));
+                }
             }
         }
         else if( OFTStringList == eType )
@@ -903,6 +972,18 @@ json_object* OGRGeoJSONWriteAttributes( OGRFeature* poFeature,
                     json_object_new_string(papszStringList[i]));
             }
         }
+        else if( OFTDateTime == eType || OFTDate == eType )
+        {
+            char* pszDT = OGRGetXMLDateTime(poFeature->GetRawFieldRef(nField));
+            if( eType == OFTDate )
+            {
+                char* pszT = strchr(pszDT, 'T');
+                if( pszT )
+                    *pszT = 0;
+            }
+            poObjProp = json_object_new_string(pszDT);
+            CPLFree(pszDT);
+        }
         else
         {
             poObjProp = json_object_new_string(
@@ -942,7 +1023,7 @@ json_object* OGRGeoJSONWriteGeometry( const OGRGeometry* poGeometry,
 
     OGRwkbGeometryType eFType = wkbFlatten(poGeometry->getGeometryType());
     // For point empty, return a null geometry. For other empty geometry types,
-    // we will generate an empty coordinate array, which is propably also
+    // we will generate an empty coordinate array, which is probably also
     // borderline.
     if( eFType == wkbPoint && poGeometry->IsEmpty() )
     {
@@ -1422,9 +1503,26 @@ char* OGR_G_ExportToJsonEx( OGRGeometryH hGeometry, char** papszOptions )
     oOptions.nCoordPrecision = nCoordPrecision;
     oOptions.nSignificantFigures = nSignificantFigures;
 
+    // If the CRS has latitude, longitude (or northing, easting) axis order,
+    // and the data axis to SRS axis mapping doesn't change that order,
+    // then swap X and Y values.
+    bool bHasSwappedXY = false;
+    const auto poSRS = poGeometry->getSpatialReference();
+    if( poSRS &&
+        (poSRS->EPSGTreatsAsLatLong() || poSRS->EPSGTreatsAsNorthingEasting()) &&
+        poSRS->GetDataAxisToSRSAxisMapping() == std::vector<int>{1, 2} )
+    {
+        poGeometry->swapXY();
+        bHasSwappedXY = true;
+    }
+
     json_object* poObj =
        OGRGeoJSONWriteGeometry( poGeometry, oOptions );
 
+    // Unswap back
+    if( bHasSwappedXY )
+        poGeometry->swapXY();
+
     if( nullptr != poObj )
     {
         char* pszJson = CPLStrdup( json_object_to_json_string( poObj ) );
@@ -1448,19 +1546,26 @@ static int OGR_json_double_with_precision_to_string( struct json_object *jso,
                                                      int /* level */,
                                                      int /* flags */)
 {
-    // TODO(schwehr): Explain this casting.
-    const int nPrecision =
+    const void* userData =
 #if (!defined(JSON_C_VERSION_NUM)) || (JSON_C_VERSION_NUM < JSON_C_VER_013)
-        static_cast<int>(reinterpret_cast<GUIntptr_t>(jso->_userdata));
+        jso->_userdata;
 #else
-        static_cast<int>(reinterpret_cast<GUIntptr_t>(json_object_get_userdata(jso)));
+        json_object_get_userdata(jso);
 #endif
+    // Precision is stored as a uintptr_t content casted to void*
+    const uintptr_t nPrecision = reinterpret_cast<uintptr_t>(userData);
     char szBuffer[75] = {};
-    OGRFormatDouble( szBuffer, sizeof(szBuffer), json_object_get_double(jso), '.',
-                     (nPrecision < 0) ? 15 : nPrecision );
-    if( szBuffer[0] == 't' /*oobig */ )
+    const double dfVal =  json_object_get_double(jso);
+    if( fabs(dfVal) > 1e50 && !CPLIsInf(dfVal) )
     {
-        CPLsnprintf(szBuffer, sizeof(szBuffer), "%.18g", json_object_get_double(jso));
+        CPLsnprintf(szBuffer, sizeof(szBuffer), "%.18g", dfVal);
+    }
+    else
+    {
+        const bool bPrecisionIsNegative =
+            (nPrecision >> (8 * sizeof(nPrecision)-1)) != 0;
+        OGRFormatDouble( szBuffer, sizeof(szBuffer), dfVal, '.',
+             bPrecisionIsNegative ? 15 : static_cast<int>(nPrecision) );
     }
     return printbuf_memappend(pb, szBuffer, static_cast<int>(strlen(szBuffer)));
 }
@@ -1490,11 +1595,12 @@ OGR_json_double_with_significant_figures_to_string( struct json_object *jso,
 {
     char szBuffer[75] = {};
     int nSize = 0;
-    if( CPLIsNan(json_object_get_double(jso)))
+    const double dfVal = json_object_get_double(jso);
+    if( CPLIsNan(dfVal))
         nSize = CPLsnprintf(szBuffer, sizeof(szBuffer), "NaN");
-    else if( CPLIsInf(json_object_get_double(jso)) )
+    else if( CPLIsInf(dfVal) )
     {
-        if( json_object_get_double(jso) > 0 )
+        if( dfVal > 0 )
             nSize = CPLsnprintf(szBuffer, sizeof(szBuffer), "Infinity");
         else
             nSize = CPLsnprintf(szBuffer, sizeof(szBuffer), "-Infinity");
@@ -1502,24 +1608,22 @@ OGR_json_double_with_significant_figures_to_string( struct json_object *jso,
     else
     {
         char szFormatting[32] = {};
+        const void* userData =
 #if (!defined(JSON_C_VERSION_NUM)) || (JSON_C_VERSION_NUM < JSON_C_VER_013)
-        const int nSignificantFigures = (int) (GUIntptr_t) jso->_userdata;
+            jso->_userdata;
 #else
-        const int nSignificantFigures = (int) (GUIntptr_t) json_object_get_userdata(jso);
+            json_object_get_userdata(jso);
 #endif
+        const uintptr_t nSignificantFigures = reinterpret_cast<uintptr_t>(userData);
+        const bool bSignificantFiguresIsNegative =
+            (nSignificantFigures >> (8 * sizeof(nSignificantFigures)-1)) != 0;
         const int nInitialSignificantFigures =
-            nSignificantFigures >= 0 ? nSignificantFigures : 17;
+            bSignificantFiguresIsNegative ? 17 : static_cast<int>(nSignificantFigures);
         CPLsnprintf(szFormatting, sizeof(szFormatting),
                     "%%.%dg", nInitialSignificantFigures);
         nSize = CPLsnprintf(szBuffer, sizeof(szBuffer),
-                            szFormatting, json_object_get_double(jso));
-        const char* pszDot = nullptr;
-        if( nSize+2 < static_cast<int>(sizeof(szBuffer)) &&
-            (pszDot = strchr(szBuffer, '.')) == nullptr )
-        {
-            nSize += CPLsnprintf(szBuffer + nSize, sizeof(szBuffer) - nSize,
-                                 ".0");
-        }
+                            szFormatting, dfVal);
+        const char* pszDot = strchr(szBuffer, '.');
 
         // Try to avoid .xxxx999999y or .xxxx000000y rounding issues by
         // decreasing a bit precision.
@@ -1534,7 +1638,7 @@ OGR_json_double_with_significant_figures_to_string( struct json_object *jso,
                 CPLsnprintf(szFormatting, sizeof(szFormatting),
                             "%%.%dg", nInitialSignificantFigures- i);
                 nSize = CPLsnprintf(szBuffer, sizeof(szBuffer),
-                                    szFormatting, json_object_get_double(jso));
+                                    szFormatting, dfVal);
                 pszDot = strchr(szBuffer, '.');
                 if( pszDot != nullptr &&
                     strstr(pszDot, "999999") == nullptr &&
@@ -1549,16 +1653,18 @@ OGR_json_double_with_significant_figures_to_string( struct json_object *jso,
                 CPLsnprintf(szFormatting, sizeof(szFormatting),
                             "%%.%dg", nInitialSignificantFigures);
                 nSize = CPLsnprintf(szBuffer, sizeof(szBuffer),
-                                    szFormatting, json_object_get_double(jso));
-                if( nSize+2 < static_cast<int>(sizeof(szBuffer)) &&
-                    strchr(szBuffer, '.') == nullptr )
-                {
-                    nSize +=
-                        CPLsnprintf(szBuffer + nSize, sizeof(szBuffer) - nSize,
-                                    ".0");
-                }
+                                    szFormatting, dfVal);
             }
         }
+
+        if( nSize+2 < static_cast<int>(sizeof(szBuffer)) &&
+            strchr(szBuffer, '.') == nullptr &&
+            strchr(szBuffer, 'e') == nullptr )
+        {
+            nSize += CPLsnprintf(szBuffer + nSize, sizeof(szBuffer) - nSize,
+                                 ".0");
+        }
+
     }
 
     return printbuf_memappend(pb, szBuffer, nSize);
@@ -1573,9 +1679,67 @@ json_object_new_double_with_significant_figures( double dfVal,
                                                  int nSignificantFigures )
 {
     json_object* jso = json_object_new_double(dfVal);
-    // TODO(schwehr): Convert C cast.
     json_object_set_serializer(
         jso, OGR_json_double_with_significant_figures_to_string,
-        (void*)(size_t)nSignificantFigures, nullptr );
+        reinterpret_cast<void*>(static_cast<size_t>(nSignificantFigures)),
+        nullptr );
+    return jso;
+}
+
+/************************************************************************/
+/*             OGR_json_float_with_significant_figures_to_string()      */
+/************************************************************************/
+
+static int
+OGR_json_float_with_significant_figures_to_string( struct json_object *jso,
+                                                   struct printbuf *pb,
+                                                   int /* level */,
+                                                   int /* flags */)
+{
+    char szBuffer[75] = {};
+    int nSize = 0;
+    const float fVal = static_cast<float>(json_object_get_double(jso));
+    if( CPLIsNan(fVal))
+        nSize = CPLsnprintf(szBuffer, sizeof(szBuffer), "NaN");
+    else if( CPLIsInf(fVal) )
+    {
+        if( fVal > 0 )
+            nSize = CPLsnprintf(szBuffer, sizeof(szBuffer), "Infinity");
+        else
+            nSize = CPLsnprintf(szBuffer, sizeof(szBuffer), "-Infinity");
+    }
+    else
+    {
+        const void* userData =
+#if (!defined(JSON_C_VERSION_NUM)) || (JSON_C_VERSION_NUM < JSON_C_VER_013)
+            jso->_userdata;
+#else
+            json_object_get_userdata(jso);
+#endif
+        const uintptr_t nSignificantFigures = reinterpret_cast<uintptr_t>(userData);
+        const bool bSignificantFiguresIsNegative =
+            (nSignificantFigures >> (8 * sizeof(nSignificantFigures)-1)) != 0;
+        const int nInitialSignificantFigures =
+            bSignificantFiguresIsNegative ? 8 : static_cast<int>(nSignificantFigures);
+        nSize = OGRFormatFloat(szBuffer, sizeof(szBuffer), fVal,
+                               nInitialSignificantFigures, 'g');
+    }
+
+    return printbuf_memappend(pb, szBuffer, nSize);
+}
+
+/************************************************************************/
+/*              json_object_new_float_with_significant_figures()        */
+/************************************************************************/
+
+json_object *
+json_object_new_float_with_significant_figures( float fVal,
+                                                int nSignificantFigures )
+{
+    json_object* jso = json_object_new_double(fVal);
+    json_object_set_serializer(
+        jso, OGR_json_float_with_significant_figures_to_string,
+        reinterpret_cast<void*>(static_cast<size_t>(nSignificantFigures)),
+        nullptr );
     return jso;
 }
