@@ -32,7 +32,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: ogrgmtlayer.cpp 8c3e4ef55212f20eec95aa7e12ba5d48dacfdc47 2020-10-01 21:20:51 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogrgmtlayer.cpp 6a73451ff0b40272a30aa9470d5493f6970ab096 2021-03-28 15:28:29 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                            OGRGmtLayer()                             */
@@ -465,7 +465,7 @@ OGRFeature *OGRGmtLayer::GetNextRawFeature()
                 {
                     // Add a hole to the current polygon.
                     poMP->getGeometryRef(
-                        poMP->getNumGeometries()-1 )->toPolygon()->
+                        poMP->getNumGeometries()-1 )->
                         addRingDirectly( new OGRLinearRing() );
                 }
                 else if( !NextIsFeature() )
@@ -606,7 +606,7 @@ OGRFeature *OGRGmtLayer::GetNextRawFeature()
                       {
                           OGRMultiPolygon *poMP = poGeom->toMultiPolygon();
                           poPoly = poMP->getGeometryRef(
-                              poMP->getNumGeometries() - 1 )->toPolygon();
+                              poMP->getNumGeometries() - 1 );
                       }
                       else
                           poPoly = poGeom->toPolygon();
@@ -629,7 +629,7 @@ OGRFeature *OGRGmtLayer::GetNextRawFeature()
                   {
                       OGRMultiLineString *poML = poGeom->toMultiLineString();
                       OGRLineString *poLine = poML->getGeometryRef(
-                          poML->getNumGeometries() -1 )->toLineString();
+                          poML->getNumGeometries() -1 );
 
                       if( nDim == 3 )
                           poLine->addPoint( dfX, dfY, dfZ );
