@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_geopackage.h 1f5037da497fb5ee7497768ccf4172b92a91578d 2019-02-19 21:19:51 +0100 Even Rouault $
+ * $Id: ogr_geopackage.h eef59c953468d4ebdce3d88b5194e46054d89366 2021-04-08 19:51:47 +0200 Even Rouault $
  *
  * Project:  GeoPackage Translator
  * Purpose:  Definition of classes for OGR GeoPackage driver.
@@ -52,7 +52,6 @@
 typedef enum
 {
     GPKG_ATTRIBUTES,
-    OGR_ASPATIAL,
     NOT_REGISTERED,
 } GPKGASpatialVariant;
 
@@ -60,7 +59,8 @@ typedef enum
 static const GUInt32 GP10_APPLICATION_ID = 0x47503130U;
 static const GUInt32 GP11_APPLICATION_ID = 0x47503131U;
 static const GUInt32 GPKG_APPLICATION_ID = 0x47504B47U;
-static const GUInt32 GPKG_1_2_VERSION = 0x000027D8U; // 10200
+static const GUInt32 GPKG_1_2_VERSION = 10200U;
+static const GUInt32 GPKG_1_3_VERSION = 10300U;
 
 static const size_t knApplicationIdPos = 68;
 static const size_t knUserVersionPos = 60;
@@ -92,6 +92,7 @@ class OGRGeoPackageTableLayer;
 class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALGPKGMBTilesLikePseudoDataset
 {
     friend class GDALGeoPackageRasterBand;
+    friend class OGRGeoPackageLayer;
     friend class OGRGeoPackageTableLayer;
 
     GUInt32             m_nApplicationId;
@@ -112,7 +113,7 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
     bool                m_bGridCellEncodingAsCO = false;
     bool                m_bHasReadMetadataFromStorage;
     bool                m_bMetadataDirty;
-    char              **m_papszSubDatasets;
+    CPLStringList       m_aosSubDatasets{};
     char               *m_pszProjection;
     bool                m_bRecordInsertedInGPKGContent;
     bool                m_bGeoTransformValid;
@@ -127,7 +128,7 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
 
     bool                m_bInFlushCache;
 
-    bool                m_bTableCreated;
+    bool                m_bDateTimeWithTZ = true;
 
     CPLString           m_osTilingScheme;
 
@@ -183,7 +184,7 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
         bool                    RegisterZoomOtherExtension();
         void                    ParseCompressionOptions(char** papszOptions);
 
-        bool                    HasMetadataTables();
+        bool                    HasMetadataTables() const;
         bool                    CreateMetadataTables();
         const char*             CheckMetadataDomain( const char* pszDomain );
         void                    WriteMetadata(CPLXMLNode* psXMLNode, /* will be destroyed by the method */
@@ -195,7 +196,6 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
         bool                    HasGriddedCoverageAncillaryTable();
         bool                    CreateTileGriddedTable(char** papszOptions);
 
-        void                    CreateOGREmptyTableIfNeeded();
         void                    RemoveOGREmptyTable();
 
         std::map<CPLString, CPLString> m_oMapNameToType;
@@ -221,6 +221,10 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
 
         bool                ConvertGpkgSpatialRefSysToExtensionWkt2();
 
+        std::map<int, bool> m_oSetGPKGLayerWarnings{};
+
+        void                FixupWrongRTreeTrigger();
+
     public:
                             GDALGeoPackageDataset();
                             virtual ~GDALGeoPackageDataset();
@@ -267,6 +271,10 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
                                          char **papszOptions ) override;
         int                 TestCapability( const char * ) override;
 
+        const OGRFieldDomain* GetFieldDomain(const std::string& name) const override;
+        bool                AddFieldDomain(std::unique_ptr<OGRFieldDomain>&& domain,
+                                           std::string& failureReason) override;
+
         virtual std::pair<OGRLayer*, IOGRSQLiteGetSpatialWhere*> GetLayerWithGetSpatialWhereByName( const char* pszName ) override;
 
         virtual OGRLayer *  ExecuteSQL( const char *pszSQLCommand,
@@ -277,18 +285,18 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
         virtual OGRErr      CommitTransaction() override;
         virtual OGRErr      RollbackTransaction() override;
 
-        bool                IsInTransaction() const;
+        inline bool         IsInTransaction() const { return nSoftTransactionLevel > 0; }
 
         int                 GetSrsId( const OGRSpatialReference& oSRS );
         const char*         GetSrsName( const OGRSpatialReference& oSRS );
-        OGRSpatialReference* GetSpatialRef( int iSrsId );
+        OGRSpatialReference* GetSpatialRef( int iSrsId, bool bFallbackToEPSG = false );
         OGRErr              CreateExtensionsTableIfNecessary();
         bool                HasExtensionsTable();
-        OGRErr              CreateGDALAspatialExtension();
         void                SetMetadataDirty() { m_bMetadataDirty = true; }
 
-        bool                    HasDataColumnsTable();
-        bool                    HasDataColumnConstraintsTable();
+        bool                    HasDataColumnsTable() const;
+        bool                    HasDataColumnConstraintsTable() const;
+        bool                CreateColumnsTableAndColumnConstraintsTablesIfNecessary();
 
         const char*         GetGeometryTypeString(OGRwkbGeometryType eType);
 
@@ -303,6 +311,8 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
                                                    GDALProgressFunc pfnProgress,
                                                    void * pProgressData );
 
+        static std::string GetCurrentDateEscapedSQL();
+
     protected:
         // Coming from GDALGPKGMBTilesLikePseudoDataset
 
@@ -310,7 +320,7 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
         virtual int                     IGetRasterCount() override { return nBands; }
         virtual GDALRasterBand*         IGetRasterBand(int nBand) override { return GetRasterBand(nBand); }
         virtual sqlite3                *IGetDB() override { return GetDB(); }
-        virtual bool                    IGetUpdate() override { return bUpdate != FALSE; }
+        virtual bool                    IGetUpdate() override { return GetUpdate(); }
         virtual bool                    ICanIWriteBlock() override;
         virtual OGRErr                  IStartTransaction() override { return SoftStartTransaction(); }
         virtual OGRErr                  ICommitTransaction() override { return SoftCommitTransaction(); }
@@ -353,7 +363,7 @@ class GDALGeoPackageRasterBand final: public GDALGPKGMBTilesLikeRasterBand
 /*                           OGRGeoPackageLayer                         */
 /************************************************************************/
 
-class OGRGeoPackageLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
+class OGRGeoPackageLayer CPL_NON_FINAL: public OGRLayer, public IOGRSQLiteGetSpatialWhere
 {
   protected:
     GDALGeoPackageDataset *m_poDS;
@@ -363,6 +373,7 @@ class OGRGeoPackageLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
 
     sqlite3_stmt        *m_poQueryStatement;
     bool                 bDoStep;
+    bool                 m_bEOF = false;
 
     char                *m_pszFidColumn;
 
@@ -408,6 +419,8 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
     bool                        m_bIsInGpkgContents;
     bool                        m_bFeatureDefnCompleted;
     int                         m_iSrs;
+    int                         m_nZFlag = 0;
+    int                         m_nMFlag = 0;
     OGREnvelope*                m_poExtent;
 #ifdef ENABLE_GPKG_OGR_CONTENTS
     GIntBig                     m_nTotalFeatureCount;
@@ -439,9 +452,23 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
     CPLString                   m_osDescriptionLCO;
     bool                        m_bHasReadMetadataFromStorage;
     bool                        m_bHasTriedDetectingFID64;
-    GPKGASpatialVariant         m_eASPatialVariant;
+    GPKGASpatialVariant         m_eASpatialVariant;
     std::set<OGRwkbGeometryType> m_eSetBadGeomTypeWarned;
 
+    int                         m_nCountInsertInTransactionThreshold = -1;
+    GIntBig                     m_nCountInsertInTransaction = 0;
+    std::vector<CPLString >     m_aoRTreeTriggersSQL{};
+    typedef struct
+    {
+        GIntBig nId;
+        float   fMinX;
+        float   fMinY;
+        float   fMaxX;
+        float   fMaxY;
+    } GPKGRTreeEntry;
+    std::vector<GPKGRTreeEntry>  m_aoRTreeEntries{};
+
+
     virtual OGRErr      ResetStatement() override;
 
     void                BuildWhere();
@@ -463,6 +490,9 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
 
     bool                DoSpecialProcessingForColumnCreation(OGRFieldDefn* poField);
 
+    bool                StartDeferredSpatialIndexUpdate();
+    bool                FlushPendingSpatialIndexUpdate();
+
     public:
                         OGRGeoPackageTableLayer( GDALGeoPackageDataset *poDS,
                                                  const char * pszTableName );
@@ -520,8 +550,8 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
                                                const char* pszDescription );
     void                SetDeferredSpatialIndexCreation( bool bFlag )
                                 { m_bDeferredSpatialIndexCreation = bFlag; }
-    void                SetASpatialVariant( GPKGASpatialVariant eASPatialVariant )
-                                { m_eASPatialVariant = eASPatialVariant; }
+    void                SetASpatialVariant( GPKGASpatialVariant eASpatialVariant )
+                                { m_eASpatialVariant = eASpatialVariant; }
 
     void                CreateSpatialIndexIfNecessary();
     bool                CreateSpatialIndex(const char* pszTableName = nullptr);
@@ -552,6 +582,10 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
     void                SetTruncateFieldsFlag( int bFlag )
                                 { m_bTruncateFields = CPL_TO_BOOL( bFlag ); }
     OGRErr              RunDeferredCreationIfNecessary();
+    bool                RunDeferredDropRTreeTableIfNecessary();
+    bool                DoJobAtTransactionCommit();
+    bool                DoJobAtTransactionRollback();
+    bool                RunDeferredSpatialIndexUpdate();
 
 #ifdef ENABLE_GPKG_OGR_CONTENTS
     bool                GetAddOGRFeatureCountTriggers() const
@@ -596,7 +630,7 @@ class OGRGeoPackageSelectLayer final : public OGRGeoPackageLayer, public IOGRSQL
 {
     CPL_DISALLOW_COPY_ASSIGN(OGRGeoPackageSelectLayer)
 
-    OGRSQLiteSelectLayerCommonBehaviour* poBehaviour;
+    OGRSQLiteSelectLayerCommonBehaviour* poBehavior;
 
     virtual OGRErr      ResetStatement() override;
 
