@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_mssqlspatial.h 74bf9367d33c608de344cc32a57a9ca924e36191 2019-03-31 20:45:06 +0200 Tamas Szekeres $
+ * $Id: ogr_mssqlspatial.h d4f8ce3534e7f16fb3eb0eed97b7cb301828ccd3 2021-03-28 15:45:02 +0200 Even Rouault $
  *
  * Project:  MSSQL Spatial driver
  * Purpose:  Definition of classes for OGR MSSQL Spatial driver.
@@ -125,28 +125,23 @@ public:
 
     bool            IsValidLatLon(double longitude, double latitude);
     bool            IsValidCircularZ(double z1, double z2);
-    bool            IsValidPolygonRingCount(OGRCurve* poGeom);
-    bool            IsValidPolygonRingClosed(OGRCurve* poGeom);
-    bool            IsValid(OGRPoint* poGeom);
-    bool            IsValid(OGRMultiPoint* poGeom);
-    bool            IsValid(OGRLineString* poGeom);
-    bool            IsValid(OGRCircularString* poGeom);
-    bool            IsValid(OGRSimpleCurve* poGeom);
-    bool            IsValid(OGRCompoundCurve* poGeom);
-    bool            IsValid(OGRLinearRing* poGeom);
-    bool            IsValid(OGRMultiLineString* poGeom);
-    bool            IsValid(OGRPolygon* poGeom);
-    bool            IsValid(OGRCurvePolygon* poGeom);
-    bool            IsValid(OGRMultiPolygon* poGeom);
-    bool            IsValid(OGRGeometryCollection* poGeom);
-    bool            IsValid(OGRGeometry* poGeom);
+    bool            IsValidPolygonRingCount(const OGRCurve* poGeom);
+    bool            IsValidPolygonRingClosed(const OGRCurve* poGeom);
+    bool            IsValid(const OGRPoint* poGeom);
+    bool            IsValid(const OGRMultiPoint* poGeom);
+    bool            IsValid(const OGRCircularString* poGeom);
+    bool            IsValid(const OGRSimpleCurve* poGeom);
+    bool            IsValid(const OGRCompoundCurve* poGeom);
+    bool            IsValid(const OGRMultiLineString* poGeom);
+    bool            IsValid(const OGRCurvePolygon* poGeom);
+    bool            IsValid(const OGRMultiPolygon* poGeom);
+    bool            IsValid(const OGRGeometryCollection* poGeom);
+    bool            IsValid(const OGRGeometry* poGeom);
     void            MakeValid(OGRPoint* poGeom);
     void            MakeValid(OGRMultiPoint* poGeom);
-    void            MakeValid(OGRLineString* poGeom);
     void            MakeValid(OGRCircularString* poGeom);
     void            MakeValid(OGRSimpleCurve* poGeom);
     void            MakeValid(OGRCompoundCurve* poGeom);
-    void            MakeValid(OGRLinearRing* poGeom);
     void            MakeValid(OGRMultiLineString* poGeom);
     void            MakeValid(OGRPolygon* poGeom);
     void            MakeValid(OGRCurvePolygon* poGeom);
@@ -273,13 +268,15 @@ public:
 /*                             OGRMSSQLSpatialLayer                     */
 /************************************************************************/
 
-class OGRMSSQLSpatialLayer : public OGRLayer
+class OGRMSSQLSpatialLayer CPL_NON_FINAL: public OGRLayer
 {
     protected:
     OGRFeatureDefn     *poFeatureDefn = nullptr;
     int                 nRawColumns = 0;
 
     CPLODBCStatement   *poStmt = nullptr;
+    bool                m_bEOF = false;
+    bool                m_bResetNeeded = false;
 
     // Layer spatial reference system, and srid.
     OGRSpatialReference *poSRS = nullptr;
@@ -305,13 +302,14 @@ class OGRMSSQLSpatialLayer : public OGRLayer
                                           CPLODBCStatement *poStmt );
 
     virtual CPLODBCStatement *  GetStatement() { return poStmt; }
+    void                ClearStatement();
+    OGRFeature         *GetNextRawFeature();
 
   public:
                         OGRMSSQLSpatialLayer();
     virtual             ~OGRMSSQLSpatialLayer();
 
     virtual void        ResetReading() override;
-    virtual OGRFeature *GetNextRawFeature();
     virtual OGRFeature *GetNextFeature() override;
 
     virtual OGRFeature *GetFeature( GIntBig nFeatureId ) override;
@@ -393,7 +391,6 @@ class OGRMSSQLSpatialTableLayer final: public OGRMSSQLSpatialLayer
     int                 bIdentityInsert = FALSE;
 #endif
 
-    void                ClearStatement();
     CPLODBCStatement* BuildStatement(const char* pszColumns);
 
     CPLString BuildFields();
@@ -424,7 +421,6 @@ class OGRMSSQLSpatialTableLayer final: public OGRMSSQLSpatialLayer
     virtual OGRErr      GetExtent(OGREnvelope *psExtent, int bForce) override { return GetExtent(0, psExtent, bForce); }
     virtual OGRErr      GetExtent(int iGeomField, OGREnvelope *psExtent, int bForce) override;
 
-    virtual void        ResetReading() override;
     virtual GIntBig     GetFeatureCount( int ) override;
 
     virtual OGRFeatureDefn *GetLayerDefn() override;
@@ -486,9 +482,6 @@ class OGRMSSQLSpatialSelectLayer final: public OGRMSSQLSpatialLayer
 {
     char                *pszBaseStatement;
 
-    void                ClearStatement();
-    OGRErr              ResetStatement();
-
     virtual CPLODBCStatement *  GetStatement() override;
 
   public:
@@ -496,7 +489,6 @@ class OGRMSSQLSpatialSelectLayer final: public OGRMSSQLSpatialLayer
                                            CPLODBCStatement * );
                         virtual ~OGRMSSQLSpatialSelectLayer();
 
-    virtual void        ResetReading() override;
     virtual GIntBig     GetFeatureCount( int ) override;
 
     virtual OGRFeature *GetFeature( GIntBig nFeatureId ) override;
@@ -612,10 +604,10 @@ public:
 };
 
 /************************************************************************/
-/*                             OGRODBCDriver                            */
+/*                        OGRMSSQLSpatialDriver                         */
 /************************************************************************/
 
-class OGRMSSQLSpatialDriver : public OGRSFDriver
+class OGRMSSQLSpatialDriver final: public OGRSFDriver
 {
   public:
     virtual ~OGRMSSQLSpatialDriver();
