@@ -2,10 +2,10 @@
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements Open FileGDB OGR driver.
- * Author:   Even Rouault, <even dot rouault at mines-dash paris dot org>
+ * Author:   Even Rouault, <even dot rouault at spatialys.com>
  *
  ******************************************************************************
- * Copyright (c) 2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -49,9 +49,11 @@
 #include "ogr_geometry.h"
 #include "ogr_mem.h"
 #include "ogrsf_frmts.h"
-#include "swq.h"
+#include "ogr_swq.h"
 
-CPL_CVSID("$Id: ogropenfilegdbdatasource.cpp 22f8ae3bf7bc3cccd970992655c63fc5254d3206 2018-04-08 20:13:05 +0200 Even Rouault $")
+#include "filegdb_fielddomain.h"
+
+CPL_CVSID("$Id: ogropenfilegdbdatasource.cpp 7819e495986c82c51d7023358b14ecb2b0fd4d98 2021-04-01 20:02:10 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                      OGROpenFileGDBDataSource()                      */
@@ -183,6 +185,7 @@ int OGROpenFileGDBDataSource::Open( const char* pszFilename )
     }
 
     if( !(oTable.GetFieldCount() >= 2 &&
+          oTable.GetTotalRecordCount() < 100000 &&
           oTable.GetField(0)->GetName() == "Name" &&
           oTable.GetField(0)->GetType() == FGFT_STRING &&
           oTable.GetField(1)->GetName() == "FileFormat" &&
@@ -197,40 +200,47 @@ int OGROpenFileGDBDataSource::Open( const char* pszFilename )
     int iGDBObjectClasses = -1; /* V9.X */
 
     std::vector<std::string> aosTableNames;
-    for( int i=0;i<oTable.GetTotalRecordCount();i++)
+    try
     {
-        if( !oTable.SelectRow(i) )
+        for( int i=0;i<oTable.GetTotalRecordCount();i++)
         {
-            if( oTable.HasGotError() )
-                break;
-            aosTableNames.push_back("");
-            continue;
-        }
+            if( !oTable.SelectRow(i) )
+            {
+                if( oTable.HasGotError() )
+                    break;
+                aosTableNames.push_back("");
+                continue;
+            }
 
-        const OGRField* psField = oTable.GetFieldValue(0);
-        if( psField != nullptr )
-        {
-            aosTableNames.push_back(psField->String);
+            const OGRField* psField = oTable.GetFieldValue(0);
+            if( psField != nullptr )
+            {
+                aosTableNames.push_back(psField->String);
 
-            if( strcmp(psField->String, "GDB_Items") == 0 )
-            {
-                iGDBItems = i;
+                if( strcmp(psField->String, "GDB_Items") == 0 )
+                {
+                    iGDBItems = i;
+                }
+                else if( strcmp(psField->String, "GDB_FeatureClasses") == 0 )
+                {
+                    iGDBFeatureClasses = i;
+                }
+                else if( strcmp(psField->String, "GDB_ObjectClasses") == 0 )
+                {
+                    iGDBObjectClasses = i;
+                }
+                m_osMapNameToIdx[psField->String] = 1 + i;
             }
-            else if( strcmp(psField->String, "GDB_FeatureClasses") == 0 )
+            else
             {
-                iGDBFeatureClasses = i;
+                aosTableNames.push_back("");
             }
-            else if( strcmp(psField->String, "GDB_ObjectClasses") == 0 )
-            {
-                iGDBObjectClasses = i;
-            }
-            m_osMapNameToIdx[psField->String] = 1 + i;
-        }
-        else
-        {
-            aosTableNames.push_back("");
         }
     }
+    catch( const std::exception& )
+    {
+        return FALSE;
+    }
 
     oTable.Close();
 
@@ -398,6 +408,17 @@ int OGROpenFileGDBDataSource::OpenFileGDBv10(int iGDBItems,
                           nullptr, wkbUnknown );
             }
         }
+        else if( psField != nullptr &&
+            (strstr(psField->String, "GPCodedValueDomain2") != nullptr ||
+                strstr(psField->String, "GPRangeDomain2") != nullptr) )
+        {
+            auto poDomain = ParseXMLFieldDomainDef(psField->String);
+            if( poDomain )
+            {
+                const auto domainName = poDomain->GetName();
+                m_oMapFieldDomains[domainName] = std::move(poDomain);
+            }
+        }
     }
 
     if( m_apoLayers.empty() && nCandidateLayers > 0 &&
