@@ -2,10 +2,10 @@
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGDumpDataSource class.
- * Author:   Even Rouault, <even dot rouault at mines dash paris dot org>
+ * Author:   Even Rouault, <even dot rouault at spatialys.com>
  *
  ******************************************************************************
- * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,7 +31,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrpgdumpdatasource.cpp 7e07230bbff24eb333608de4dbd460b7312839d0 2017-12-11 19:08:47Z Even Rouault $")
+CPL_CVSID("$Id: ogrpgdumpdatasource.cpp 379fc8667418dc54e864d828ea35be513e69abc9 2021-04-03 21:58:21 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                      OGRPGDumpDataSource()                           */
@@ -39,14 +39,7 @@ CPL_CVSID("$Id: ogrpgdumpdatasource.cpp 7e07230bbff24eb333608de4dbd460b7312839d0
 
 OGRPGDumpDataSource::OGRPGDumpDataSource( const char* pszNameIn,
                                           char** papszOptions ) :
-    nLayers(0),
-    papoLayers(nullptr),
-    pszName(CPLStrdup(pszNameIn)),
-    bTriedOpen(false),
-    fp(nullptr),
-    bInTransaction(false),
-    poLayerInCopyMode(nullptr),
-    pszEOL("\n")
+    pszName(CPLStrdup(pszNameIn))
 {
     const char *pszCRLFFormat = CSLFetchNameValue( papszOptions, "LINEFORMAT");
 
@@ -366,20 +359,13 @@ OGRPGDumpDataSource::ICreateLayer( const char * pszLayerName,
 /*      Try to get the SRS Id of this spatial reference system,         */
 /*      adding tot the srs table if needed.                             */
 /* -------------------------------------------------------------------- */
-    int nUnknownSRSId = -1;
     const char* pszPostgisVersion =
-        CSLFetchNameValue( papszOptions, "POSTGIS_VERSION" );
-    int nPostGISMajor = 1;
-    int nPostGISMinor = 5;
-    if( pszPostgisVersion != nullptr && atoi(pszPostgisVersion) >= 2 )
-    {
-        nPostGISMajor = atoi(pszPostgisVersion);
-        if( strchr(pszPostgisVersion, '.') )
-            nPostGISMinor = atoi(strchr(pszPostgisVersion, '.')+1);
-        else
-            nPostGISMinor = 0;
-        nUnknownSRSId = 0;
-    }
+        CSLFetchNameValueDef( papszOptions, "POSTGIS_VERSION", "2.2" );
+    const int nPostGISMajor = atoi(pszPostgisVersion);
+    const char* pszPostgisVersionDot = strchr(pszPostgisVersion, '.');
+    const int nPostGISMinor =
+          pszPostgisVersionDot ? atoi(pszPostgisVersionDot+1) : 0;
+    const int nUnknownSRSId = nPostGISMajor >= 2 ? 0 : -1;
 
     int nSRSId = nUnknownSRSId;
     int nForcedSRSId = -2;
@@ -460,7 +446,7 @@ OGRPGDumpDataSource::ICreateLayer( const char * pszLayerName,
     }
     else
     {
-        osCreateTable.Printf("CREATE TABLE%s \"%s\".\"%s\"",
+        osCreateTable.Printf("CREATE%s TABLE \"%s\".\"%s\"",
                              CPLFetchBool( papszOptions, "UNLOGGED", false ) ?
                              " UNLOGGED": "",
                              pszSchemaName, pszTableName);
@@ -554,8 +540,19 @@ OGRPGDumpDataSource::ICreateLayer( const char * pszLayerName,
         Log(osCommand);
     }
 
-    const char *pszSI = CSLFetchNameValue( papszOptions, "SPATIAL_INDEX" );
-    const bool bCreateSpatialIndex = pszSI == nullptr || CPLTestBool(pszSI);
+    const char *pszSI = CSLFetchNameValueDef( papszOptions, "SPATIAL_INDEX", "GIST" );
+    const bool bCreateSpatialIndex = ( EQUAL(pszSI, "GIST") ||
+        EQUAL(pszSI, "SPGIST") || EQUAL(pszSI, "BRIN") ||
+        EQUAL(pszSI, "YES") || EQUAL(pszSI, "ON") || EQUAL(pszSI, "TRUE") );
+    if( !bCreateSpatialIndex && !EQUAL(pszSI, "NO") && !EQUAL(pszSI, "OFF") &&
+        !EQUAL(pszSI, "FALSE") && !EQUAL(pszSI, "NONE") )
+    {
+        CPLError(CE_Warning, CPLE_NotSupported,
+                 "SPATIAL_INDEX=%s not supported", pszSI);
+    }
+    const char* pszSpatialIndexType = EQUAL(pszSI, "SPGIST") ? "SPGIST" :
+                                      EQUAL(pszSI, "BRIN") ? "BRIN" : "GIST";
+
     if( bCreateTable && bHavePostGIS && bCreateSpatialIndex )
     {
 /* -------------------------------------------------------------------- */
@@ -567,8 +564,9 @@ OGRPGDumpDataSource::ICreateLayer( const char * pszLayerName,
         osCommand.Printf(
             "CREATE INDEX \"%s_%s_geom_idx\" "
             "ON \"%s\".\"%s\" "
-            "USING GIST (\"%s\")",
+            "USING %s (\"%s\")",
             pszTableName, pszGFldName, pszSchemaName, pszTableName,
+            pszSpatialIndexType,
             pszGFldName);
 
         Log(osCommand);
@@ -591,7 +589,7 @@ OGRPGDumpDataSource::ICreateLayer( const char * pszLayerName,
     poLayer->SetOverrideColumnTypes(pszOverrideColumnTypes);
     poLayer->SetUnknownSRSId(nUnknownSRSId);
     poLayer->SetForcedSRSId(nForcedSRSId);
-    poLayer->SetCreateSpatialIndexFlag(bCreateSpatialIndex);
+    poLayer->SetCreateSpatialIndex(bCreateSpatialIndex, pszSpatialIndexType);
     poLayer->SetPostGISVersion(nPostGISMajor, nPostGISMinor);
     poLayer->SetForcedGeometryTypeFlags(ForcedGeometryTypeFlags);
 
