@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_sqlite.h 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $
+ * $Id: ogr_sqlite.h be7fac6445d8606b05cb151ded7460fe18d60830 2021-04-04 21:42:47 +0200 Even Rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for OGR/SQLite driver.
@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2004, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2009-2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009-2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -42,29 +42,23 @@
 #include "ogrsf_frmts.h"
 #include "ogrsf_frmts.h"
 #include "rasterlite2_header.h"
-#include "sqlite3.h"
 
-#ifdef HAVE_SPATIALITE
-  #ifdef SPATIALITE_AMALGAMATION
-    /*
-    / using an AMALGAMATED version of SpatiaLite
-    / a private internal copy of SQLite is included:
-    / so we are required including the SpatiaLite's
-    / own header
-    /
-    / IMPORTANT NOTICE: using AMALAGATION is only
-    / useful on Windows (to skip DLL hell related oddities)
-    */
-    #include <spatialite/sqlite3.h>
-  #else
-    /*
-    / You MUST NOT use AMALGAMATION on Linux or any
-    / other "sane" operating system !!!!
-    */
-    #include "sqlite3.h"
-  #endif
+#ifdef SPATIALITE_AMALGAMATION
+/*
+/ using an AMALGAMATED version of SpatiaLite
+/ a private internal copy of SQLite is included:
+/ so we are required including the SpatiaLite's
+/ own header
+/
+/ IMPORTANT NOTICE: using AMALAGATION is only
+/ useful on Windows (to skip DLL hell related oddities)
+/
+/ You MUST NOT use AMALGAMATION on Linux or any
+/ other "sane" operating system !!!!
+*/
+#include <spatialite/sqlite3.h>
 #else
-#include "sqlite3.h"
+#include <sqlite3.h>
 #endif
 
 #ifndef DO_NOT_INCLUDE_SQLITE_CLASSES
@@ -220,7 +214,7 @@ class IOGRSQLiteGetSpatialWhere
                                           OGRGeometry* poFilterGeom) = 0;
 };
 
-class OGRSQLiteLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
+class OGRSQLiteLayer CPL_NON_FINAL: public OGRLayer, public IOGRSQLiteGetSpatialWhere
 {
   private:
     static OGRErr       createFromSpatialiteInternal(const GByte *pabyData,
@@ -254,6 +248,7 @@ class OGRSQLiteLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
 
     sqlite3_stmt        *hStmt;
     int                  bDoStep;
+    bool                 m_bEOF = false;
 
     OGRSQLiteDataSource *poDS;
 
@@ -267,6 +262,7 @@ class OGRSQLiteLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
     int                 bIsVirtualShape;
 
     void                BuildFeatureDefn( const char *pszLayerName,
+                                          bool bIsSelect,
                                           sqlite3_stmt *hStmt,
                                           const std::set<CPLString>* paosGeomCols,
                                           const std::set<CPLString>& aosIgnoredCols);
@@ -345,7 +341,7 @@ class OGRSQLiteLayer : public OGRLayer, public IOGRSQLiteGetSpatialWhere
 /*                         OGRSQLiteTableLayer                          */
 /************************************************************************/
 
-class OGRSQLiteTableLayer : public OGRSQLiteLayer
+class OGRSQLiteTableLayer final: public OGRSQLiteLayer
 {
     int                 bLaunderColumnNames;
     int                 bSpatialite2D;
@@ -487,7 +483,7 @@ class OGRSQLiteTableLayer : public OGRSQLiteLayer
 /*                         OGRSQLiteViewLayer                           */
 /************************************************************************/
 
-class OGRSQLiteViewLayer : public OGRSQLiteLayer
+class OGRSQLiteViewLayer final: public OGRSQLiteLayer
 {
     CPLString           osWHERE;
     CPLString           osQuery;
@@ -615,9 +611,9 @@ class OGRSQLiteSelectLayerCommonBehaviour
 /*                         OGRSQLiteSelectLayer                         */
 /************************************************************************/
 
-class OGRSQLiteSelectLayer : public OGRSQLiteLayer, public IOGRSQLiteSelectLayer
+class OGRSQLiteSelectLayer CPL_NON_FINAL: public OGRSQLiteLayer, public IOGRSQLiteSelectLayer
 {
-    OGRSQLiteSelectLayerCommonBehaviour* poBehaviour;
+    OGRSQLiteSelectLayerCommonBehaviour* poBehavior;
 
     virtual OGRErr      ResetStatement() override;
 
@@ -693,14 +689,13 @@ class OGRSQLiteSingleFeatureLayer final : public OGRLayer
 /************************************************************************/
 
 /* Used by both OGRSQLiteDataSource and OGRGeoPackageDataSource */
-class OGRSQLiteBaseDataSource : public GDALPamDataset
+class OGRSQLiteBaseDataSource CPL_NON_FINAL: public GDALPamDataset
 {
   protected:
     char               *m_pszFilename;
     bool                m_bCallUndeclareFileNotToOpen = false;
 
     sqlite3             *hDB;
-    int                 bUpdate;
 
     sqlite3_vfs*        pMyVFS;
 
@@ -736,7 +731,7 @@ class OGRSQLiteBaseDataSource : public GDALPamDataset
                         virtual ~OGRSQLiteBaseDataSource();
 
     sqlite3            *GetDB() { return hDB; }
-    int                 GetUpdate() const { return bUpdate; }
+    inline bool         GetUpdate() const { return eAccess == GA_Update; }
 
     void                NotifyFileOpened (const char* pszFilename,
                                           VSILFILE* fp);
@@ -746,13 +741,15 @@ class OGRSQLiteBaseDataSource : public GDALPamDataset
 
     virtual std::pair<OGRLayer*, IOGRSQLiteGetSpatialWhere*> GetLayerWithGetSpatialWhereByName( const char* pszName ) = 0;
 
+    virtual OGRErr     AbortSQL() override;
+
     virtual OGRErr      StartTransaction(int bForce = FALSE) override;
     virtual OGRErr      CommitTransaction() override;
     virtual OGRErr      RollbackTransaction() override;
 
     virtual int         TestCapability( const char * ) override;
 
-    virtual void *GetInternalHandle( const char * ) override;
+    virtual void        *GetInternalHandle( const char * ) override;
 
     OGRErr              SoftStartTransaction();
     OGRErr              SoftCommitTransaction();
@@ -774,8 +771,6 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
     int                *panSRID;
     OGRSpatialReference **papoSRS;
 
-    char              **papszOpenOptions;
-
     void                AddSRIDToCache(int nId, OGRSpatialReference * poSRS );
 
     int                 bHaveGeometryColumns;
@@ -874,7 +869,7 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
     int                 FetchSRSId( const OGRSpatialReference * poSRS );
     OGRSpatialReference*FetchSRS( int nSRID );
 
-    void                SetUpdate(int bUpdateIn) { bUpdate = bUpdateIn; }
+    void                DisableUpdate() { eAccess = GA_ReadOnly; }
 
     void                SetName(const char* pszNameIn);
 
@@ -895,6 +890,7 @@ class OGRSQLiteDataSource final : public OGRSQLiteBaseDataSource
     void                ReloadLayers();
 
 #ifdef HAVE_RASTERLITE2
+    void*               GetRL2Context() const { return m_hRL2Ctxt; }
     rl2CoveragePtr      GetRL2CoveragePtr() const { return m_pRL2Coverage; }
     GIntBig             GetSectionId() const { return m_nSectionId; }
     const double*       GetGeoTransform() const { return m_adfGeoTransform; }
