@@ -9,7 +9,7 @@
  ******************************************************************************
  * Copyright (c) 2011, Ben Ahmed Daho Ali
  * Copyright (c) 2013, NextGIS
- * Copyright (c) 2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2014, Even Rouault <even dot rouault at spatialys.com>
  * Copyright (c) 2019, NextGIS, <info@nextgis.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -40,41 +40,7 @@
 #include <map>
 #include <string>
 
-CPL_CVSID("$Id: ogrsxfdatasource.cpp 2e4886b9eb765d286e8a4bedd959920caae7f379 2019-03-23 14:08:34 +0100 Even Rouault $")
-
-constexpr long aoVCS[] =
-{
-    0,
-    5705,   //1
-    5711,   //2
-    0,      //3
-    5710,   //4
-    5710,   //5
-    0,      //6
-    0,      //7
-    0,      //8
-    0,      //9
-    5716,   //10
-    5733,   //11
-    0,      //12
-    0,      //13
-    0,      //14
-    0,      //15
-    5709,   //16
-    5776,   //17
-    0,      //18
-    0,      //19
-    5717,   //20
-    5613,   //21
-    0,      //22
-    5775,   //23
-    5702,   //24
-    0,      //25
-    0,      //26
-    5714    //27
-};
-
-#define NUMBER_OF_VERTICALCS    (sizeof(aoVCS)/sizeof(aoVCS[0]))
+CPL_CVSID("$Id: ogrsxfdatasource.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 // EPSG code range http://gis.stackexchange.com/a/18676/9904
 constexpr int MIN_EPSG = 1000;
@@ -156,7 +122,8 @@ OGRLayer *OGRSXFDataSource::GetLayer( int iLayer )
 /*                                Open()                                */
 /************************************************************************/
 
-int OGRSXFDataSource::Open( const char * pszFilename, int bUpdateIn)
+int OGRSXFDataSource::Open(const char * pszFilename, bool bUpdateIn,
+                           const char* const* papszOpenOpts)
 {
     if( bUpdateIn )
     {
@@ -184,6 +151,8 @@ int OGRSXFDataSource::Open( const char * pszFilename, int bUpdateIn)
         CloseFile();
         return FALSE;
     }
+    CPL_LSBPTR32(&(stSXFFileHeader.nHeaderLength));
+    CPL_LSBPTR32(&(stSXFFileHeader.nCheckSum));
 
     //check version
     oSXFPassport.version = 0;
@@ -229,7 +198,7 @@ int OGRSXFDataSource::Open( const char * pszFilename, int bUpdateIn)
     }
 
     //read spatial data
-    if (ReadSXFMapDescription(fpSXF, oSXFPassport) != OGRERR_NONE)
+    if (ReadSXFMapDescription(fpSXF, oSXFPassport, papszOpenOpts) != OGRERR_NONE)
     {
         CPLError(CE_Failure, CPLE_NotSupported, "SXF. Wrong state of the data.");
         CloseFile();
@@ -245,8 +214,10 @@ int OGRSXFDataSource::Open( const char * pszFilename, int bUpdateIn)
 /*---------------- TRY READ THE RSC FILE HEADER  -----------------------*/
 
     CPLString soRSCRileName;
-    const char* pszRSCRileName = CPLGetConfigOption("SXF_RSC_FILENAME", "");
-    if (CPLCheckForFile((char *)pszRSCRileName, nullptr) == TRUE)
+    const char* pszRSCRileName =
+        CSLFetchNameValueDef(papszOpenOpts, "SXF_RSC_FILENAME",
+                             CPLGetConfigOption("SXF_RSC_FILENAME", ""));
+    if (pszRSCRileName != nullptr && CPLCheckForFile((char *)pszRSCRileName, nullptr) == TRUE)
     {
         soRSCRileName = pszRSCRileName;
     }
@@ -301,7 +272,7 @@ int OGRSXFDataSource::Open( const char * pszFilename, int bUpdateIn)
         {
             CPLDebug( "OGRSXFDataSource", "RSC Filename: %s",
                       soRSCRileName.c_str() );
-            CreateLayers(fpRSC);
+            CreateLayers(fpRSC, papszOpenOpts);
             VSIFCloseL(fpRSC);
         }
     }
@@ -489,43 +460,21 @@ OGRErr OGRSXFDataSource::ReadSXFInformationFlags(VSILFILE* fpSXFIn, SXFPassport&
     return OGRERR_NONE;
 }
 
-void OGRSXFDataSource::SetVertCS(const long iVCS, SXFPassport& passport)
+void OGRSXFDataSource::SetVertCS(const long iVCS, SXFPassport& passport,
+                                 const char* const* papszOpenOpts)
 {
-    if (!CPLTestBool(CPLGetConfigOption("SXF_SET_VERTCS", "NO")))
+    const char* pszSetVertCS =
+        CSLFetchNameValueDef(papszOpenOpts,
+                             "SXF_SET_VERTCS",
+                              CPLGetConfigOption("SXF_SET_VERTCS", "NO"));
+    if (!CPLTestBool(pszSetVertCS))
         return;
 
-    const int nEPSG = static_cast<int>(aoVCS[iVCS]);
-
-    if (nEPSG < MIN_EPSG || nEPSG > MAX_EPSG)
-    {
-        CPLError(CE_Warning, CPLE_NotSupported, "SXF. Vertical coordinate system (SXF index %ld) not supported", iVCS);
-        return;
-    }
-
-    OGRSpatialReference sr;
-    sr.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
-    OGRErr eImportFromEPSGErr = sr.importFromEPSG(nEPSG);
-    if (eImportFromEPSGErr != OGRERR_NONE)
-    {
-        CPLError( CE_Warning, CPLE_None, "SXF. Vertical coordinate system (SXF index %ld, EPSG %d) import from EPSG error", iVCS, nEPSG);
-        return;
-    }
-
-    if (sr.IsVertical() != 1)
-    {
-        CPLError( CE_Warning, CPLE_None, "SXF. Coordinate system (SXF index %ld, EPSG %d) is not Vertical", iVCS, nEPSG);
-        return;
-    }
-
-    //passport.stMapDescription.pSpatRef->SetVertCS("Baltic", "Baltic Sea");
-    OGRErr eSetVertCSErr = passport.stMapDescription.pSpatRef->SetVertCS(sr.GetAttrValue("VERT_CS"), sr.GetAttrValue("VERT_DATUM"));
-    if (eSetVertCSErr != OGRERR_NONE)
-    {
-        CPLError(CE_Warning, CPLE_None, "SXF. Vertical coordinate system (SXF index %ld, EPSG %d) set error", iVCS, nEPSG);
-        return;
-    }
+    passport.stMapDescription.pSpatRef->importVertCSFromPanorama(static_cast<int>(iVCS));
 }
-OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& passport)
+
+OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& passport,
+                                               const char* const* papszOpenOpts)
 {
     // int nObjectsRead = 0;
     passport.stMapDescription.Env.MaxX = -100000000;
@@ -541,8 +490,8 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
         short nNoObjClass, nNoSemClass;
         /* nObjectsRead = */ VSIFReadL(&nNoObjClass, 2, 1, fpSXFIn);
         /* nObjectsRead = */ VSIFReadL(&nNoSemClass, 2, 1, fpSXFIn);
-        GByte baMask[8];
-        /* nObjectsRead = */ VSIFReadL(&baMask, 8, 1, fpSXFIn);
+        GByte byMask[8];
+        /* nObjectsRead = */ VSIFReadL(&byMask, 8, 1, fpSXFIn);
 
         int nCorners[8];
 
@@ -551,6 +500,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
 
         for( int i = 0; i < 8; i++ )
         {
+            CPL_LSBPTR32(&nCorners[i]);
             passport.stMapDescription.stProjCoords[i] = double(nCorners[i]) / 10.0;
             if (bIsX) //X
             {
@@ -573,6 +523,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
 
         for( int i = 0; i < 8; i++ )
         {
+            CPL_LSBPTR32(&nCorners[i]);
             passport.stMapDescription.stGeoCoords[i] = double(nCorners[i]) * 0.00000057295779513082; //from radians to degree * 100 000 000
         }
     }
@@ -580,6 +531,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
     {
         int nEPSG = 0;
         /* nObjectsRead = */ VSIFReadL(&nEPSG, 4, 1, fpSXFIn);
+        CPL_LSBPTR32(&nEPSG);
 
         if (nEPSG >= MIN_EPSG && nEPSG <= MAX_EPSG) //TODO: check epsg valid range
         {
@@ -592,6 +544,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
 
         for( int i = 0; i < 8; i++ )
         {
+            CPL_LSBPTR64(&dfCorners[i]);
             passport.stMapDescription.stProjCoords[i] = dfCorners[i];
             if (bIsX) //X
             {
@@ -614,6 +567,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
 
         for( int i = 0; i < 8; i++ )
         {
+            CPL_LSBPTR64(&dfCorners[i]);
             passport.stMapDescription.stGeoCoords[i] = dfCorners[i] * TO_DEGREES; // to degree
         }
     }
@@ -665,13 +619,22 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
             GUInt32 nFrameCode;
         } buff;
         /* nObjectsRead = */ VSIFReadL(&buff, 20, 1, fpSXFIn);
+        CPL_LSBPTR32(&buff.nRes);
+        CPL_LSBPTR32(&buff.nFrameCode);
         passport.stMapDescription.nResolution = buff.nRes; //resolution
 
         for( int i = 0; i < 8; i++ )
+        {
+            CPL_LSBPTR16(&(buff.anFrame[i]));
             passport.stMapDescription.stFrameCoords[i] = buff.anFrame[i];
+        }
 
         int anParams[5];
         /* nObjectsRead = */ VSIFReadL(&anParams, 20, 1, fpSXFIn);
+        for(int i = 0; i < 5; i++)
+        {
+            CPL_LSBPTR32(&anParams[i]);
+        }
 
         if (anParams[0] != -1)
             dfProjScale = double(anParams[0]) / 100000000.0;
@@ -716,6 +679,10 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
         VSIFSeekL(fpSXFIn, 312, SEEK_SET);
         GUInt32 buff[10];
         /* nObjectsRead = */ VSIFReadL(&buff, 40, 1, fpSXFIn);
+        for(int i = 0; i < 10; i++)
+        {
+            CPL_LSBPTR32(&buff[i]);
+        }
 
         passport.stMapDescription.nResolution = buff[0]; //resolution
         for( int i = 0; i < 8; i++ )
@@ -723,6 +690,10 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
 
         double adfParams[6] = {};
         /* nObjectsRead = */ VSIFReadL(&adfParams, 48, 1, fpSXFIn);
+        for(int i = 0; i < 6; i++)
+        {
+            CPL_LSBPTR64(&adfParams[i]);
+        }
 
         if (adfParams[1] != -1)
             dfProjScale = adfParams[1];
@@ -774,7 +745,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
             passport.stMapDescription.pSpatRef = new OGRSpatialReference();
             passport.stMapDescription.pSpatRef->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
             OGRErr eErr = passport.stMapDescription.pSpatRef->importFromEPSG(nEPSG);
-            SetVertCS(iVCS, passport);
+            SetVertCS(iVCS, passport, papszOpenOpts);
             return eErr;
         }
         else
@@ -807,7 +778,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
         passport.stMapDescription.pSpatRef = new OGRSpatialReference();
         passport.stMapDescription.pSpatRef->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         OGRErr eErr = passport.stMapDescription.pSpatRef->importFromEPSG(nEPSG);
-        SetVertCS(iVCS, passport);
+        SetVertCS(iVCS, passport, papszOpenOpts);
         return eErr;
     }
     else if (iEllips == 45 && iProjSys == 35) //Mercator 3857 on sphere wgs84
@@ -815,7 +786,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
         passport.stMapDescription.pSpatRef = new OGRSpatialReference("PROJCS[\"WGS 84 / Pseudo-Mercator\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Mercator_1SP\"],PARAMETER[\"central_meridian\",0],PARAMETER[\"scale_factor\",1],PARAMETER[\"false_easting\",0],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH],EXTENSION[\"PROJ4\",\"+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +wktext  +no_defs\"],AUTHORITY[\"EPSG\",\"3857\"]]");
         passport.stMapDescription.pSpatRef->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         OGRErr eErr = OGRERR_NONE; //passport.stMapDescription.pSpatRef->importFromEPSG(3857);
-        SetVertCS(iVCS, passport);
+        SetVertCS(iVCS, passport, papszOpenOpts);
         return eErr;
     }
     else if (iEllips == 9 && iProjSys == 35) //Mercator 3395 on ellips wgs84
@@ -823,7 +794,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
         passport.stMapDescription.pSpatRef = new OGRSpatialReference();
         passport.stMapDescription.pSpatRef->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         OGRErr eErr = passport.stMapDescription.pSpatRef->importFromEPSG(3395);
-        SetVertCS(iVCS, passport);
+        SetVertCS(iVCS, passport, papszOpenOpts);
         return eErr;
     }
     else if (iEllips == 9 && iProjSys == 34) //Miller 54003 on sphere wgs84
@@ -833,7 +804,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
         OGRErr eErr = OGRERR_NONE; //passport.stMapDescription.pSpatRef->importFromEPSG(3395);
         //OGRErr eErr = passport.stMapDescription.pSpatRef->importFromEPSG(54003);
 
-        SetVertCS(iVCS, passport);
+        SetVertCS(iVCS, passport, papszOpenOpts);
         return eErr;
     }
     else if (iEllips == 9 && iProjSys == 33 &&
@@ -842,7 +813,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
         passport.stMapDescription.pSpatRef = new OGRSpatialReference(SRS_WKT_WGS84_LAT_LONG);
         passport.stMapDescription.pSpatRef->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
         OGRErr eErr = OGRERR_NONE;
-        SetVertCS(iVCS, passport);
+        SetVertCS(iVCS, passport, papszOpenOpts);
         return eErr;
     }
 
@@ -866,7 +837,7 @@ OGRErr OGRSXFDataSource::ReadSXFMapDescription(VSILFILE* fpSXFIn, SXFPassport& p
     passport.stMapDescription.pSpatRef = new OGRSpatialReference();
     passport.stMapDescription.pSpatRef->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     OGRErr eErr = passport.stMapDescription.pSpatRef->importFromPanorama(anData[2], anData[3], anData[0], adfPrjParams);
-    SetVertCS(iVCS, passport);
+    SetVertCS(iVCS, passport, papszOpenOpts);
     return eErr;
 }
 
@@ -885,12 +856,14 @@ void OGRSXFDataSource::FillLayers()
         VSIFSeekL(fpSXF, 288, SEEK_SET);
         nObjectsRead = static_cast<int>(VSIFReadL(&nRecordCountMax, 4, 1, fpSXF));
         nOffset = 300;
+        CPL_LSBPTR32(&nRecordCountMax);
     }
     else if (oSXFPassport.version == 4)
     {
         VSIFSeekL(fpSXF, 440, SEEK_SET);
         nObjectsRead = static_cast<int>(VSIFReadL(&nRecordCountMax, 4, 1, fpSXF));
         nOffset = 452;
+        CPL_LSBPTR32(&nRecordCountMax);
     }
     /* else nOffset and nObjectsRead will be 0 */
 
@@ -906,6 +879,10 @@ void OGRSXFDataSource::FillLayers()
     {
         GInt32 buff[6];
         nObjectsRead = static_cast<int>(VSIFReadL(&buff, 24, 1, fpSXF));
+        for( int i = 0; i < 6; i++ )
+        {
+            CPL_LSBPTR32(&buff[i]);
+        }
 
         if (nObjectsRead != 1 || buff[0] != IDSXFOBJ)
         {
@@ -993,12 +970,11 @@ void OGRSXFDataSource::CreateLayers()
     nLayers++;
 }
 
-void OGRSXFDataSource::CreateLayers(VSILFILE* fpRSC)
+void OGRSXFDataSource::CreateLayers(VSILFILE* fpRSC, const char* const* papszOpenOpts)
 {
 
     RSCHeader stRSCFileHeader;
-    int nFileHeaderSize = sizeof(stRSCFileHeader);
-    int nObjectsRead = static_cast<int>(VSIFReadL(&stRSCFileHeader, nFileHeaderSize, 1, fpRSC));
+    int nObjectsRead = static_cast<int>(VSIFReadL(&stRSCFileHeader, sizeof(stRSCFileHeader), 1, fpRSC));
 
     if (nObjectsRead != 1)
     {
@@ -1006,6 +982,37 @@ void OGRSXFDataSource::CreateLayers(VSILFILE* fpRSC)
         return;
     }
 
+    CPL_LSBPTR32(&(stRSCFileHeader.nFileLength));
+    CPL_LSBPTR32(&(stRSCFileHeader.nVersion));
+    CPL_LSBPTR32(&(stRSCFileHeader.nEncoding));
+    CPL_LSBPTR32(&(stRSCFileHeader.nFileState));
+    CPL_LSBPTR32(&(stRSCFileHeader.nFileModState));
+    CPL_LSBPTR32(&(stRSCFileHeader.nLang));
+    CPL_LSBPTR32(&(stRSCFileHeader.nNextID));
+    CPL_LSBPTR32(&(stRSCFileHeader.nScale));
+
+#define SWAP_SECTION(x) \
+    CPL_LSBPTR32(&(x.nOffset)); \
+    CPL_LSBPTR32(&(x.nLength)); \
+    CPL_LSBPTR32(&(x.nRecordCount));
+
+    SWAP_SECTION(stRSCFileHeader.Objects);
+    SWAP_SECTION(stRSCFileHeader.Semantic);
+    SWAP_SECTION(stRSCFileHeader.ClassifySemantic);
+    SWAP_SECTION(stRSCFileHeader.Defaults);
+    SWAP_SECTION(stRSCFileHeader.Semantics);
+    SWAP_SECTION(stRSCFileHeader.Layers);
+    SWAP_SECTION(stRSCFileHeader.Limits);
+    SWAP_SECTION(stRSCFileHeader.Parameters);
+    SWAP_SECTION(stRSCFileHeader.Print);
+    SWAP_SECTION(stRSCFileHeader.Palettes);
+    SWAP_SECTION(stRSCFileHeader.Fonts);
+    SWAP_SECTION(stRSCFileHeader.Libs);
+    SWAP_SECTION(stRSCFileHeader.ImageParams);
+    SWAP_SECTION(stRSCFileHeader.Tables);
+    CPL_LSBPTR32(&(stRSCFileHeader.nFontEnc));
+    CPL_LSBPTR32(&(stRSCFileHeader.nColorsInPalette));
+
     GByte szLayersID[4];
     struct _layer{
         GUInt32 nLength;
@@ -1018,8 +1025,6 @@ void OGRSXFDataSource::CreateLayers(VSILFILE* fpRSC)
         GUInt16 nSemanticCount;
     };
 
-    size_t nLayerStructSize = sizeof(_layer);
-
     VSIFSeekL(fpRSC, stRSCFileHeader.Layers.nOffset - sizeof(szLayersID), SEEK_SET);
     VSIFReadL(&szLayersID, sizeof(szLayersID), 1, fpRSC);
     vsi_l_offset nOffset = stRSCFileHeader.Layers.nOffset;
@@ -1027,11 +1032,14 @@ void OGRSXFDataSource::CreateLayers(VSILFILE* fpRSC)
 
     for( GUInt32 i = 0; i < stRSCFileHeader.Layers.nRecordCount; ++i )
     {
-        VSIFReadL(&LAYER, nLayerStructSize, 1, fpRSC);
-
+        VSIFReadL(&LAYER, sizeof(LAYER), 1, fpRSC);
+        CPL_LSBPTR32(&(LAYER.nLength));
+        CPL_LSBPTR16(&(LAYER.nSemanticCount));
         papoLayers = (OGRLayer**)CPLRealloc(papoLayers, sizeof(OGRLayer*)* (nLayers + 1));
-        bool bLayerFullName = CPLTestBool(CPLGetConfigOption("SXF_LAYER_FULLNAME", "NO"));
-
+        bool bLayerFullName = CPLTestBool(
+                 CSLFetchNameValueDef(papszOpenOpts,
+                                      "SXF_LAYER_FULLNAME",
+                                       CPLGetConfigOption("SXF_LAYER_FULLNAME", "NO")));
         char* pszRecoded = nullptr;
         if (bLayerFullName)
         {
@@ -1095,6 +1103,10 @@ void OGRSXFDataSource::CreateLayers(VSILFILE* fpRSC)
     for( GUInt32 i = 0; i < stRSCFileHeader.Objects.nRecordCount; ++i )
     {
         VSIFReadL(&OBJECT, sizeof(_object), 1, fpRSC);
+        CPL_LSBPTR32(&(OBJECT.nLength));
+        CPL_LSBPTR32(&(OBJECT.nClassifyCode));
+        CPL_LSBPTR32(&(OBJECT.nObjectNumber));
+        CPL_LSBPTR32(&(OBJECT.nObjectCode));
 
         OGRSXFLayer* pLayer = GetLayerById(OBJECT.szLayernNo);
         if (nullptr != pLayer)
