@@ -52,7 +52,7 @@
 #include "ogr_geometry.h"
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrutils.cpp 1316179cb4c140cd2d3cc757a327d6ab201659a1 2020-11-11 21:27:28 +0100 Even Rouault $")
+CPL_CVSID("$Id: ogrutils.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
 
 // Returns whether a double fits within an int.
 // Unable to put this in cpl_port.h as include limit breaks grib.
@@ -124,7 +124,7 @@ std::string roundup(std::string s)
 }
 
 
-// This attempts to elimiate what is likely binary -> decimal representation
+// This attempts to eliminate what is likely binary -> decimal representation
 // error or the result of low-order rounding with calculations.  The result
 // may be more visually pleasing and takes up fewer places.
 std::string intelliround(std::string& s)
@@ -1326,6 +1326,11 @@ char* OGRGetRFC822DateTime( const OGRField* psField )
 /************************************************************************/
 
 char* OGRGetXMLDateTime(const OGRField* psField)
+{
+    return OGRGetXMLDateTime(psField, false);
+}
+
+char* OGRGetXMLDateTime(const OGRField* psField, bool bAlwaysMillisecond)
 {
     const GInt16 year = psField->Date.Year;
     const GByte month = psField->Date.Month;
@@ -1359,7 +1364,7 @@ char* OGRGetXMLDateTime(const OGRField* psField)
                      (TZFlag > 100) ? '+' : '-', TZHour, TZMinute);
     }
 
-    if( OGR_GET_MS(second) )
+    if( OGR_GET_MS(second) || bAlwaysMillisecond )
         pszRet = CPLStrdup(CPLSPrintf(
                                "%04d-%02u-%02uT%02u:%02u:%06.3f%s",
                                year, month, day, hour, minute, second,
