@@ -32,23 +32,24 @@
 #ifdef HAVE_CURL
 
 #include "cpl_aws.h"
+#include "cpl_azure.h"
 #include "cpl_port.h"
+#include "cpl_json.h"
 #include "cpl_string.h"
 #include "cpl_vsil_curl_priv.h"
 #include "cpl_mem_cache.h"
 
-#include <curl/curl.h>
+#include "cpl_curl_priv.h"
 
 #include <set>
 #include <map>
 #include <memory>
+#include <mutex>
 
 //! @cond Doxygen_Suppress
 
-// 7.18.1
-#if LIBCURL_VERSION_NUM >= 0x071201
+// Leave it for backward compatibility, but deprecate.
 #define HAVE_CURLINFO_REDIRECT_URL
-#endif
 
 void VSICurlStreamingClearCache( void ); // from cpl_vsil_curl_streaming.cpp
 
@@ -57,6 +58,13 @@ struct curl_slist* VSICurlSetOptions(CURL* hCurlHandle, const char* pszURL,
 struct curl_slist* VSICurlMergeHeaders( struct curl_slist* poDest,
                                         struct curl_slist* poSrcToDestroy );
 
+struct curl_slist* VSICurlSetContentTypeFromExt(struct curl_slist* polist,
+                                                const char *pszPath);
+
+struct curl_slist* VSICurlSetCreationHeadersFromOptions(struct curl_slist* headers,
+                                                        CSLConstList papszOptions,
+                                                        const char *pszPath);
+
 namespace cpl {
 
 typedef enum
@@ -69,6 +77,7 @@ typedef enum
 class FileProp
 {
   public:
+    unsigned int    nGenerationAuthParameters = 0;
     ExistStatus     eExists = EXIST_UNKNOWN;
     vsi_l_offset    fileSize = 0;
     time_t          mTime = 0;
@@ -76,17 +85,19 @@ class FileProp
     CPLString       osRedirectURL{};
     bool            bHasComputedFileSize = false;
     bool            bIsDirectory = false;
+    int             nMode = 0; // st_mode member of struct stat
     bool            bS3LikeRedirect = false;
     CPLString       ETag{};
 };
 
-typedef struct
+struct CachedDirList
 {
     bool            bGotFileList = false;
+    unsigned int    nGenerationAuthParameters = 0;
     CPLStringList   oFileList{}; /* only file name without path */
-} CachedDirList;
+};
 
-typedef struct
+struct WriteFuncStruct
 {
     char*           pBuffer = nullptr;
     size_t          nSize = 0;
@@ -108,13 +119,32 @@ typedef struct
     void               *pReadCbkUserData = nullptr;
     bool                bInterrupted = false;
 
-#if LIBCURL_VERSION_NUM < 0x073600
+#if !CURL_AT_LEAST_VERSION(7,54,0)
     // Workaround to ignore extra HTTP response headers from
     // proxies in older versions of curl.
     // CURLOPT_SUPPRESS_CONNECT_HEADERS fixes this
     bool            bIsProxyConnectHeader = false;
-#endif
-} WriteFuncStruct;
+#endif //!CURL_AT_LEAST_VERSION(7,54,0)
+};
+
+struct PutData
+{
+    const GByte* pabyData = nullptr;
+    size_t       nOff = 0;
+    size_t       nTotalSize = 0;
+
+    static size_t ReadCallBackBuffer( char *buffer, size_t size,
+                                        size_t nitems, void *instream )
+    {
+        PutData* poThis = static_cast<PutData *>(instream);
+        const size_t nSizeMax = size * nitems;
+        const size_t nSizeToWrite =
+            std::min(nSizeMax, poThis->nTotalSize - poThis->nOff);
+        memcpy(buffer, poThis->pabyData + poThis->nOff, nSizeToWrite);
+        poThis->nOff += nSizeToWrite;
+        return nSizeToWrite;
+    }
+};
 
 /************************************************************************/
 /*                     VSICurlFilesystemHandler                         */
@@ -159,7 +189,8 @@ class VSICurlFilesystemHandler : public VSIFilesystemHandler
                     std::shared_ptr<std::string>>>::iterator,
                     FilenameOffsetPairHasher>>;
 
-    RegionCacheType oRegionCache;
+    std::unique_ptr<RegionCacheType> m_poRegionCacheDoNotUseDirectly{}; // do not access directly. Use GetRegionCache();
+    RegionCacheType* GetRegionCache();
 
     lru11::Cache<std::string, FileProp>  oCacheFileProp;
 
@@ -207,7 +238,8 @@ public:
 
     VSIVirtualHandle *Open( const char *pszFilename,
                             const char *pszAccess,
-                            bool bSetError ) override;
+                            bool bSetError,
+                            CSLConstList /* papszOptions */ ) override;
 
     int Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
               int nFlags ) override;
@@ -218,6 +250,7 @@ public:
     char **ReadDir( const char *pszDirname ) override
         { return ReadDirEx(pszDirname, 0); }
     char **ReadDirEx( const char *pszDirname, int nMaxFiles ) override;
+    char **SiblingFiles( const char *pszFilename ) override;
 
     int HasOptimizedReadMultiRange( const char* /* pszPath */ )
         override { return true; }
@@ -226,11 +259,16 @@ public:
 
     const char* GetOptions() override;
 
+    char** GetFileMetadata( const char * pszFilename, const char* pszDomain,
+                            CSLConstList papszOptions ) override;
+
     char **ReadDirInternal( const char *pszDirname, int nMaxFiles,
                             bool* pbGotFileList );
     void InvalidateDirContent( const char *pszDirname );
 
-    virtual CPLString GetFSPrefix() { return "/vsicurl/"; }
+    virtual const char* GetDebugKey() const { return "VSICURL"; }
+
+    virtual CPLString GetFSPrefix() const { return "/vsicurl/"; }
     virtual bool      AllowCachedDataFor(const char* pszFilename);
 
     std::shared_ptr<std::string> GetRegion( const char* pszURL,
@@ -244,7 +282,7 @@ public:
     bool                GetCachedFileProp( const char* pszURL,
                                            FileProp& oFileProp );
     void                SetCachedFileProp( const char* pszURL,
-                                           const FileProp& oFileProp );
+                                           FileProp& oFileProp );
     void                InvalidateCachedData( const char* pszURL );
 
     CURLM              *GetCurlMultiHandleFor( const CPLString& osURL );
@@ -256,7 +294,7 @@ public:
     bool                GetCachedDirList( const char* pszURL,
                                           CachedDirList& oCachedDirList );
     void                SetCachedDirList( const char* pszURL,
-                                          const CachedDirList& oCachedDirList );
+                                          CachedDirList& oCachedDirList );
     bool ExistsInCacheDirList( const CPLString& osDirname, bool *pbIsDir );
 
     virtual CPLString GetURLFromFilename( const CPLString& osFilename );
@@ -279,6 +317,7 @@ class VSICurlHandle : public VSIVirtualHandle
 
     CPLString       m_osFilename{}; // e.g "/vsicurl/http://example.com/foo"
     char*           m_pszURL = nullptr;     // e.g "http://example.com/foo"
+    std::string     m_osQueryString{};      // e.g. an Azure SAS
 
     char          **m_papszHTTPOptions = nullptr;
 
@@ -293,6 +332,8 @@ class VSICurlHandle : public VSIVirtualHandle
     int                 m_nMaxRetry = 0;
     double              m_dfRetryDelay = 0.0;
 
+    CPLStringList       m_aosHeaders{};
+
     void                DownloadRegionPostProcess( const vsi_l_offset startOffset,
                                                    const int nBlocks,
                                                    const char* pBuffer,
@@ -323,6 +364,7 @@ class VSICurlHandle : public VSIVirtualHandle
     virtual bool IsDirectoryFromExists( const char* /*pszVerb*/, int /*response_code*/ ) { return false; }
     virtual void ProcessGetFileSizeResult(const char* /* pszContent */ ) {}
     void SetURL(const char* pszURL);
+    virtual bool Authenticate() { return false; }
 
   public:
 
@@ -343,11 +385,13 @@ class VSICurlHandle : public VSIVirtualHandle
     int Close() override;
 
     bool IsKnownFileSize() const { return oFileProp.bHasComputedFileSize; }
-    vsi_l_offset         GetFileSize() { return GetFileSize(false); }
-    virtual vsi_l_offset GetFileSize( bool bSetError );
+    vsi_l_offset         GetFileSizeOrHeaders(bool bSetError, bool bGetHeaders);
+    virtual vsi_l_offset GetFileSize( bool bSetError ) { return GetFileSizeOrHeaders(bSetError, false); }
     bool                 Exists( bool bSetError );
     bool                 IsDirectory() const { return oFileProp.bIsDirectory; }
+    int                  GetMode() const { return oFileProp.nMode; }
     time_t               GetMTime() const { return oFileProp.mTime; }
+    const CPLStringList& GetHeaders() { return m_aosHeaders; }
 
     int                  InstallReadCbk( VSICurlReadCbkFunc pfnReadCbk,
                                          void* pfnUserData,
@@ -365,6 +409,14 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
 {
     CPL_DISALLOW_COPY_ASSIGN(IVSIS3LikeFSHandler)
 
+    bool CopyFile(VSILFILE* fpIn,
+                     vsi_l_offset nSourceSize,
+                     const char* pszSource,
+                     const char* pszTarget,
+                     GDALProgressFunc pProgressFunc,
+                     void *pProgressData);
+    virtual int MkdirInternal( const char *pszDirname, long nMode, bool bDoStatCheck );
+
   protected:
     char** GetFileList( const char *pszFilename,
                         int nMaxFiles,
@@ -373,6 +425,9 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
     virtual IVSIS3LikeHandleHelper* CreateHandleHelper(
             const char* pszURI, bool bAllowNoObject) = 0;
 
+    virtual int      CopyObject( const char *oldpath, const char *newpath,
+                                 CSLConstList papszMetadata );
+
     IVSIS3LikeFSHandler() = default;
 
   public:
@@ -381,14 +436,15 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
     int Rmdir( const char *pszDirname ) override;
     int Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
               int nFlags ) override;
+    int Rename( const char *oldpath, const char *newpath ) override;
 
     virtual int      DeleteObject( const char *pszFilename );
 
-    virtual const char* GetDebugKey() const = 0;
-
     virtual void UpdateMapFromHandle(IVSIS3LikeHandleHelper*) {}
     virtual void UpdateHandleFromMap( IVSIS3LikeHandleHelper * ) {}
 
+    virtual CPLString GetStreamingPath( const char* pszFilename ) const;
+
     bool Sync( const char* pszSource, const char* pszTarget,
                 const char* const * papszOptions,
                 GDALProgressFunc pProgressFunc,
@@ -397,6 +453,37 @@ class IVSIS3LikeFSHandler: public VSICurlFilesystemHandler
 
     VSIDIR* OpenDir( const char *pszPath, int nRecurseDepth,
                              const char* const *papszOptions) override;
+
+    // Multipart upload
+    virtual bool SupportsParallelMultipartUpload() const { return false; }
+
+    virtual CPLString InitiateMultipartUpload(
+                                const std::string& osFilename,
+                                IVSIS3LikeHandleHelper *poS3HandleHelper,
+                                int nMaxRetry,
+                                double dfRetryDelay,
+                                CSLConstList papszOptions);
+    virtual CPLString UploadPart(const CPLString& osFilename,
+                         int nPartNumber,
+                         const std::string& osUploadID,
+                         vsi_l_offset nPosition,
+                         const void* pabyBuffer,
+                         size_t nBufferSize,
+                         IVSIS3LikeHandleHelper *poS3HandleHelper,
+                         int nMaxRetry,
+                         double dfRetryDelay);
+    virtual bool CompleteMultipart(const CPLString& osFilename,
+                           const CPLString& osUploadID,
+                           const std::vector<CPLString>& aosEtags,
+                           vsi_l_offset nTotalSize,
+                           IVSIS3LikeHandleHelper *poS3HandleHelper,
+                           int nMaxRetry,
+                           double dfRetryDelay);
+    virtual bool AbortMultipart(const CPLString& osFilename,
+                        const CPLString& osUploadID,
+                        IVSIS3LikeHandleHelper *poS3HandleHelper,
+                        int nMaxRetry,
+                        double dfRetryDelay);
 };
 
 /************************************************************************/
@@ -424,7 +511,7 @@ class IVSIS3LikeHandle:  public VSICurlHandle
   public:
     IVSIS3LikeHandle( VSICurlFilesystemHandler* poFSIn,
                       const char* pszFilename,
-                      const char* pszURLIn = nullptr ) :
+                      const char* pszURLIn ) :
         VSICurlHandle(poFSIn, pszFilename, pszURLIn) {}
     ~IVSIS3LikeHandle() override {}
 };
@@ -441,18 +528,16 @@ class VSIS3WriteHandle final : public VSIVirtualHandle
     CPLString           m_osFilename{};
     IVSIS3LikeHandleHelper  *m_poS3HandleHelper = nullptr;
     bool                m_bUseChunked = false;
+    CPLStringList       m_aosOptions{};
 
     vsi_l_offset        m_nCurOffset = 0;
     int                 m_nBufferOff = 0;
     int                 m_nBufferSize = 0;
-    int                 m_nBufferOffReadCallback = 0;
     bool                m_bClosed = false;
     GByte              *m_pabyBuffer = nullptr;
     CPLString           m_osUploadID{};
     int                 m_nPartNumber = 0;
     std::vector<CPLString> m_aosEtags{};
-    CPLString           m_osXML{};
-    int                 m_nOffsetInXML = 0;
     bool                m_bError = false;
 
     CURLM              *m_hCurlMulti = nullptr;
@@ -461,19 +546,13 @@ class VSIS3WriteHandle final : public VSIVirtualHandle
     CPLString           m_osCurlErrBuf{};
     size_t              m_nChunkedBufferOff = 0;
     size_t              m_nChunkedBufferSize = 0;
+    size_t              m_nWrittenInPUT = 0;
 
     int                 m_nMaxRetry = 0;
     double              m_dfRetryDelay = 0.0;
     WriteFuncStruct     m_sWriteFuncHeaderData{};
 
-    static size_t       ReadCallBackBuffer( char *buffer, size_t size,
-                                            size_t nitems, void *instream );
-    bool                InitiateMultipartUpload();
     bool                UploadPart();
-    static size_t       ReadCallBackXML( char *buffer, size_t size,
-                                         size_t nitems, void *instream );
-    bool                CompleteMultipart();
-    bool                AbortMultipart();
     bool                DoSinglePartPUT();
 
     static size_t       ReadCallBackBufferChunked( char *buffer, size_t size,
@@ -488,7 +567,8 @@ class VSIS3WriteHandle final : public VSIVirtualHandle
       VSIS3WriteHandle( IVSIS3LikeFSHandler* poFS,
                         const char* pszFilename,
                         IVSIS3LikeHandleHelper* poS3HandleHelper,
-                        bool bUseChunked );
+                        bool bUseChunked,
+                        CSLConstList papszOptions );
       ~VSIS3WriteHandle() override;
 
       int Seek( vsi_l_offset nOffset, int nWhence ) override;
@@ -544,6 +624,166 @@ class VSIAppendWriteHandle : public VSIVirtualHandle
         bool              IsOK() { return m_pabyBuffer != nullptr; }
 };
 
+/************************************************************************/
+/*                         CurlRequestHelper                            */
+/************************************************************************/
+
+struct CurlRequestHelper
+{
+    WriteFuncStruct sWriteFuncData{};
+    WriteFuncStruct sWriteFuncHeaderData{};
+    char szCurlErrBuf[CURL_ERROR_SIZE+1] = {};
+
+    CurlRequestHelper();
+    ~CurlRequestHelper();
+    long perform(CURL* hCurlHandle,
+                 struct curl_slist* headers, // ownership transferred
+                 VSICurlFilesystemHandler *poFS,
+                 IVSIS3LikeHandleHelper *poS3HandleHelper);
+};
+
+/************************************************************************/
+/*                       NetworkStatisticsLogger                        */
+/************************************************************************/
+
+class NetworkStatisticsLogger
+{
+    static int gnEnabled;
+    static NetworkStatisticsLogger gInstance;
+
+    NetworkStatisticsLogger() = default;
+
+    std::mutex m_mutex{};
+
+    struct Counters
+    {
+        GIntBig nHEAD = 0;
+        GIntBig nGET = 0;
+        GIntBig nPUT = 0;
+        GIntBig nPOST = 0;
+        GIntBig nDELETE = 0;
+        GIntBig nGETDownloadedBytes = 0;
+        GIntBig nPUTUploadedBytes = 0;
+        GIntBig nPOSTDownloadedBytes = 0;
+        GIntBig nPOSTUploadedBytes = 0;
+    };
+
+    enum class ContextPathType
+    {
+        FILESYSTEM,
+        FILE,
+        ACTION,
+    };
+
+    struct ContextPathItem
+    {
+        ContextPathType eType;
+        CPLString       osName;
+
+        ContextPathItem(ContextPathType eTypeIn, const CPLString& osNameIn):
+            eType(eTypeIn), osName(osNameIn) {}
+
+        bool operator< (const ContextPathItem& other ) const
+        {
+            if( static_cast<int>(eType) < static_cast<int>(other.eType) )
+                return true;
+            if( static_cast<int>(eType) > static_cast<int>(other.eType) )
+                return false;
+            return osName < other.osName;
+        }
+    };
+
+    struct Stats
+    {
+        Counters counters{};
+        std::map<ContextPathItem, Stats> children{};
+
+        void AsJSON(CPLJSONObject& oJSON) const;
+    };
+
+    Stats m_stats{};
+    std::map<GIntBig, std::vector<ContextPathItem>> m_mapThreadIdToContextPath{};
+
+    static void ReadEnabled();
+
+    std::vector<Counters*> GetCountersForContext();
+
+public:
+
+    static inline bool IsEnabled()
+    {
+        if( gnEnabled < 0)
+        {
+            ReadEnabled();
+        }
+        return gnEnabled == TRUE;
+    }
+
+    static void EnterFileSystem(const char* pszName);
+
+    static void LeaveFileSystem();
+
+    static void EnterFile(const char* pszName);
+
+    static void LeaveFile();
+
+    static void EnterAction(const char* pszName);
+
+    static void LeaveAction();
+
+    static void LogHEAD();
+
+    static void LogGET(size_t nDownloadedBytes);
+
+    static void LogPUT(size_t nUploadedBytes);
+
+    static void LogPOST(size_t nUploadedBytes,
+                        size_t nDownloadedBytes);
+
+    static void LogDELETE();
+
+    static void Reset();
+
+    static CPLString GetReportAsSerializedJSON();
+};
+
+struct NetworkStatisticsFileSystem
+{
+    inline explicit NetworkStatisticsFileSystem(const char* pszName) {
+        NetworkStatisticsLogger::EnterFileSystem(pszName);
+    }
+
+    inline ~NetworkStatisticsFileSystem()
+    {
+        NetworkStatisticsLogger::LeaveFileSystem();
+    }
+};
+
+struct NetworkStatisticsFile
+{
+    inline explicit NetworkStatisticsFile(const char* pszName) {
+        NetworkStatisticsLogger::EnterFile(pszName);
+    }
+
+    inline ~NetworkStatisticsFile()
+    {
+        NetworkStatisticsLogger::LeaveFile();
+    }
+};
+
+struct NetworkStatisticsAction
+{
+    inline explicit NetworkStatisticsAction(const char* pszName) {
+        NetworkStatisticsLogger::EnterAction(pszName);
+    }
+
+    inline ~NetworkStatisticsAction()
+    {
+        NetworkStatisticsLogger::LeaveAction();
+    }
+};
+
+
 int VSICURLGetDownloadChunkSize();
 
 void VSICURLInitWriteFuncStruct( WriteFuncStruct   *psStruct,
@@ -556,6 +796,8 @@ void MultiPerform(CURLM* hCurlMultiHandle,
                          CURL* hEasyHandle = nullptr);
 void VSICURLResetHeaderAndWriterFunctions(CURL* hCurlHandle);
 
+int VSICurlParseUnixPermissions(const char* pszPermissions);
+
 } // namespace cpl
 
 //! @endcond
