@@ -2,10 +2,10 @@
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Implement VSI large file api for HTTP/FTP files in streaming mode
- * Author:   Even Rouault <even dot rouault at mines dash paris.org>
+ * Author:   Even Rouault <even dot rouault at spatialys.com>
  *
  ******************************************************************************
- * Copyright (c) 2012-2015, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2012-2015, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,7 @@
 #include "cpl_port.h"
 #include "cpl_vsi.h"
 #include "cpl_vsi_virtual.h"
+#include "cpl_vsil_curl_class.h"
 
 #include <algorithm>
 #include <map>
@@ -44,7 +45,7 @@
 #include "cpl_string.h"
 #include "cpl_time.h"
 
-CPL_CVSID("$Id: cpl_vsil_curl_streaming.cpp 5524ee5324f7bd364d391d842a6488c90c0186a7 2018-04-02 16:20:13 +0200 Even Rouault $")
+CPL_CVSID("$Id: cpl_vsil_curl_streaming.cpp 86933038c3926cd4dc3ff37c431b317abb69e602 2021-03-27 23:20:49 +0100 Even Rouault $")
 
 #if !defined(HAVE_CURL) || defined(CPL_MULTIPROC_STUB)
 
@@ -78,38 +79,40 @@ void VSIInstallSwiftStreamingFileHandler(void)
     // Not supported
 }
 
+#ifdef HAVE_CURL
+void VSICurlStreamingClearCache( void )
+{
+    // Not supported
+}
+#endif
+
 #else
 
 //! @cond Doxygen_Suppress
 
 #include <curl/curl.h>
 
-struct curl_slist* VSICurlSetOptions( CURL* hCurlHandle, const char* pszURL,
-                        const char * const* papszOptions );
-struct curl_slist* VSICurlMergeHeaders( struct curl_slist* poDest,
-                                        struct curl_slist* poSrcToDestroy );
-
 #define ENABLE_DEBUG        0
 
 #define N_MAX_REGIONS       10
 
 #define BKGND_BUFFER_SIZE   (1024 * 1024)
 
-void VSICurlStreamingClearCache( void );  // used in cpl_vsil_curl.cpp
-
 /************************************************************************/
 /*                               RingBuffer                             */
 /************************************************************************/
 
 class RingBuffer
 {
-    GByte* pabyBuffer;
-    size_t nCapacity;
-    size_t nOffset;
-    size_t nLength;
+    CPL_DISALLOW_COPY_ASSIGN(RingBuffer)
+
+    GByte* pabyBuffer = nullptr;
+    size_t nCapacity = 0;
+    size_t nOffset = 0;
+    size_t nLength = 0;
 
     public:
-        RingBuffer(size_t nCapacity = BKGND_BUFFER_SIZE);
+        explicit RingBuffer(size_t nCapacity = BKGND_BUFFER_SIZE);
         ~RingBuffer();
 
         size_t GetCapacity() const { return nCapacity; }
@@ -122,9 +125,7 @@ class RingBuffer
 
 RingBuffer::RingBuffer( size_t nCapacityIn ) :
     pabyBuffer(static_cast<GByte*>(CPLMalloc(nCapacityIn))),
-    nCapacity(nCapacityIn),
-    nOffset(0),
-    nLength(0)
+    nCapacity(nCapacityIn)
 {}
 
 RingBuffer::~RingBuffer()
@@ -207,10 +208,12 @@ class VSICurlStreamingHandle;
 
 class VSICurlStreamingFSHandler : public VSIFilesystemHandler
 {
-    std::map<CPLString, CachedFileProp*>   cacheFileSize;
+    CPL_DISALLOW_COPY_ASSIGN(VSICurlStreamingFSHandler)
+
+    std::map<CPLString, CachedFileProp*>   cacheFileSize{};
 
 protected:
-    CPLMutex           *hMutex;
+    CPLMutex           *hMutex = nullptr;
 
     virtual CPLString GetFSPrefix() { return "/vsicurl_streaming/"; }
     virtual VSICurlStreamingHandle* CreateFileHandle(const char* pszURL);
@@ -221,7 +224,8 @@ public:
 
     virtual VSIVirtualHandle *Open( const char *pszFilename,
                                     const char *pszAccess,
-                                    bool bSetError ) override;
+                                    bool bSetError,
+                                    CSLConstList /* papszOptions */ ) override;
     virtual int      Stat( const char *pszFilename, VSIStatBufL *pStatBuf,
                            int nFlags ) override;
 
@@ -244,50 +248,50 @@ public:
 
 class VSICurlStreamingHandle : public VSIVirtualHandle
 {
+    CPL_DISALLOW_COPY_ASSIGN(VSICurlStreamingHandle)
+
   protected:
-    VSICurlStreamingFSHandler* m_poFS;
-    char**          m_papszHTTPOptions;
+    VSICurlStreamingFSHandler* m_poFS = nullptr;
+    char**          m_papszHTTPOptions = nullptr;
 
   private:
-    char*           m_pszURL;
+    char*           m_pszURL = nullptr;
 
 #ifdef notdef
-    unsigned int    nRecomputedChecksumOfFirst1024Bytes;
+    unsigned int    nRecomputedChecksumOfFirst1024Bytes = 0;
 #endif
-    vsi_l_offset    curOffset;
-    vsi_l_offset    fileSize;
-    int             bHasComputedFileSize;
-    ExistStatus     eExists;
-    int             bIsDirectory;
+    vsi_l_offset    curOffset = 0;
+    vsi_l_offset    fileSize = 0;
+    int             bHasComputedFileSize = 0;
+    ExistStatus     eExists = EXIST_UNKNOWN;
+    int             bIsDirectory = 0;
 
-    int             bCanTrustCandidateFileSize;
-    int             bHasCandidateFileSize;
-    vsi_l_offset    nCandidateFileSize;
+    bool            bCanTrustCandidateFileSize = true;
+    bool            bHasCandidateFileSize = false;
+    vsi_l_offset    nCandidateFileSize = 0;
 
-    int             bEOF;
+    bool            bEOF = false;
 
-    size_t          nCachedSize;
-    GByte          *pCachedData;
+    size_t          nCachedSize = 0;
+    GByte          *pCachedData = nullptr;
 
-    CURL*           hCurlHandle;
-
-    volatile int    bDownloadInProgress;
-    volatile int    bDownloadStopped;
-    volatile int    bAskDownloadEnd;
-    vsi_l_offset    nRingBufferFileOffset;
-    CPLJoinableThread *hThread;
-    CPLMutex       *hRingBufferMutex;
-    CPLCond        *hCondProducer;
-    CPLCond        *hCondConsumer;
-    RingBuffer      oRingBuffer;
+    volatile int    bDownloadInProgress = FALSE;
+    volatile int    bDownloadStopped = FALSE;
+    volatile int    bAskDownloadEnd = FALSE;
+    vsi_l_offset    nRingBufferFileOffset = 0;
+    CPLJoinableThread *hThread = nullptr;
+    CPLMutex       *hRingBufferMutex = nullptr;
+    CPLCond        *hCondProducer = nullptr;
+    CPLCond        *hCondConsumer = nullptr;
+    RingBuffer      oRingBuffer{};
     void            StartDownload();
     void            StopDownload();
     void            PutRingBufferInCache();
 
-    GByte          *pabyHeaderData;
-    size_t          nHeaderSize;
-    vsi_l_offset    nBodySize;
-    int             nHTTPCode;
+    GByte          *pabyHeaderData = nullptr;
+    size_t          nHeaderSize = 0;
+    vsi_l_offset    nBodySize = 0;
+    int             nHTTPCode = 0;
 
     void                AcquireMutex();
     void                ReleaseMutex();
@@ -340,16 +344,11 @@ class VSICurlStreamingHandle : public VSIVirtualHandle
 /************************************************************************/
 
 VSICurlStreamingHandle::VSICurlStreamingHandle( VSICurlStreamingFSHandler* poFS,
-                                                const char* pszURL )
+                                                const char* pszURL ):
+    m_poFS(poFS),
+    m_papszHTTPOptions(CPLHTTPGetOptionsFromEnv()),
+    m_pszURL(CPLStrdup(pszURL))
 {
-    m_poFS = poFS;
-    m_pszURL = CPLStrdup(pszURL);
-    m_papszHTTPOptions = CPLHTTPGetOptionsFromEnv();
-
-#ifdef notdef
-    nRecomputedChecksumOfFirst1024Bytes = 0;
-#endif
-    curOffset = 0;
 
     poFS->AcquireMutex();
     CachedFileProp* cachedFileProp = poFS->GetCachedFileProp(pszURL);
@@ -359,32 +358,10 @@ VSICurlStreamingHandle::VSICurlStreamingHandle( VSICurlStreamingFSHandler* poFS,
     bIsDirectory = cachedFileProp->bIsDirectory;
     poFS->ReleaseMutex();
 
-    bCanTrustCandidateFileSize = TRUE;
-    bHasCandidateFileSize = FALSE;
-    nCandidateFileSize = 0;
-
-    nCachedSize = 0;
-    pCachedData = nullptr;
-
-    bEOF = FALSE;
-
-    hCurlHandle = nullptr;
-
-    hThread = nullptr;
     hRingBufferMutex = CPLCreateMutex();
     ReleaseMutex();
     hCondProducer = CPLCreateCond();
     hCondConsumer = CPLCreateCond();
-
-    bDownloadInProgress = FALSE;
-    bDownloadStopped = FALSE;
-    bAskDownloadEnd = FALSE;
-    nRingBufferFileOffset = 0;
-
-    pabyHeaderData = nullptr;
-    nHeaderSize = 0;
-    nBodySize = 0;
-    nHTTPCode = 0;
 }
 
 /************************************************************************/
@@ -396,8 +373,6 @@ VSICurlStreamingHandle::~VSICurlStreamingHandle()
     StopDownload();
 
     CPLFree(m_pszURL);
-    if( hCurlHandle != nullptr )
-        curl_easy_cleanup(hCurlHandle);
     CSLDestroy( m_papszHTTPOptions );
 
     CPLFree(pCachedData);
@@ -470,7 +445,7 @@ int VSICurlStreamingHandle::Seek( vsi_l_offset nOffset, int nWhence )
     {
         curOffset = GetFileSize() + nOffset;
     }
-    bEOF = FALSE;
+    bEOF = false;
     return 0;
 }
 
@@ -560,16 +535,6 @@ vsi_l_offset VSICurlStreamingHandle::GetFileSize()
     }
     ReleaseMutex();
 
-#if LIBCURL_VERSION_NUM < 0x070B00
-    // Curl 7.10.X doesn't manage to unset the CURLOPT_RANGE that would have
-    // been previously set, so we have to reinit the connection handle.
-    if( hCurlHandle )
-    {
-        curl_easy_cleanup(hCurlHandle);
-        hCurlHandle = curl_easy_init();
-    }
-#endif
-
     CURL* hLocalHandle = curl_easy_init();
 
     struct curl_slist* headers =
@@ -714,10 +679,7 @@ vsi_l_offset VSICurlStreamingHandle::GetFileSize()
     const vsi_l_offset nRet = fileSize;
     ReleaseMutex();
 
-    if( hCurlHandle == nullptr )
-        hCurlHandle = hLocalHandle;
-    else
-        curl_easy_cleanup(hLocalHandle);
+    curl_easy_cleanup(hLocalHandle);
 
     return nRet;
 }
@@ -987,7 +949,7 @@ size_t VSICurlStreamingHandle::ReceivedBytesHeader( GByte *buffer, size_t count,
             {
                 const char* pszVal =
                     pszContentLength + strlen("Content-Length: ");
-                bHasCandidateFileSize = TRUE;
+                bHasCandidateFileSize = true;
                 nCandidateFileSize =
                     CPLScanUIntBig(pszVal,
                                    static_cast<int>(pszEndOfLine - pszVal));
@@ -1012,7 +974,7 @@ size_t VSICurlStreamingHandle::ReceivedBytesHeader( GByte *buffer, size_t count,
                         CPLDebug("VSICURL",
                                  "GZip compression enabled --> "
                                  "cannot trust candidate file size");
-                    bCanTrustCandidateFileSize = FALSE;
+                    bCanTrustCandidateFileSize = false;
                 }
             }
         }
@@ -1042,6 +1004,8 @@ VSICurlStreamingHandleReceivedBytesHeader( void *buffer, size_t count,
 
 void VSICurlStreamingHandle::DownloadInThread()
 {
+    CURL* hCurlHandle = curl_easy_init();
+
     struct curl_slist* headers =
         VSICurlSetOptions(hCurlHandle, m_pszURL, m_papszHTTPOptions);
     headers = VSICurlMergeHeaders(headers, GetCurlHeaders("GET", headers));
@@ -1107,6 +1071,8 @@ void VSICurlStreamingHandle::DownloadInThread()
     // Signal to the consumer that the download has ended.
     CPLCondSignal(hCondProducer);
     ReleaseMutex();
+
+    curl_easy_cleanup(hCurlHandle);
 }
 
 static void VSICurlDownloadInThread( void* pArg )
@@ -1125,8 +1091,6 @@ void VSICurlStreamingHandle::StartDownload()
 
     CPLDebug("VSICURL", "Start download for %s", m_pszURL);
 
-    if( hCurlHandle == nullptr )
-        hCurlHandle = curl_easy_init();
     oRingBuffer.Reset();
     bDownloadInProgress = TRUE;
     nRingBufferFileOffset = 0;
@@ -1158,9 +1122,6 @@ void VSICurlStreamingHandle::StopDownload()
 
         CPLJoinThread(hThread);
         hThread = nullptr;
-
-        curl_easy_cleanup(hCurlHandle);
-        hCurlHandle = nullptr;
     }
 
     oRingBuffer.Reset();
@@ -1215,14 +1176,16 @@ size_t VSICurlStreamingHandle::Read( void * const pBuffer, size_t const nSize,
     size_t nRemaining = nBufferRequestSize;
 
     AcquireMutex();
-    const int bHasComputedFileSizeLocal = bHasComputedFileSize;
+    // fileSize might be set wrongly to 0, such as
+    // /vsicurl_streaming/https://query.data.world/s/jgsghstpphjhicstradhy5kpjwrnfy
+    const int bHasComputedFileSizeLocal = bHasComputedFileSize && fileSize > 0;
     const vsi_l_offset fileSizeLocal = fileSize;
     ReleaseMutex();
 
     if( bHasComputedFileSizeLocal && curOffset >= fileSizeLocal )
     {
         CPLDebug("VSICURL", "Read attempt beyond end of file");
-        bEOF = TRUE;
+        bEOF = true;
     }
     if( bEOF )
         return 0;
@@ -1301,7 +1264,7 @@ size_t VSICurlStreamingHandle::Read( void * const pBuffer, size_t const nSize,
         pabyBuffer += nSz;
         curOffset += nSz;
         nRemaining -= nSz;
-        bEOF = TRUE;
+        bEOF = true;
     }
 
     // Has a Seek() being done since the last Read()?
@@ -1361,7 +1324,7 @@ size_t VSICurlStreamingHandle::Read( void * const pBuffer, size_t const nSize,
 
         if( nBytesToSkip != 0 )
         {
-            bEOF = TRUE;
+            bEOF = true;
             return 0;
         }
     }
@@ -1416,7 +1379,7 @@ size_t VSICurlStreamingHandle::Read( void * const pBuffer, size_t const nSize,
                  static_cast<int>(nBufferRequestSize - nRemaining));
     size_t nRet = (nBufferRequestSize - nRemaining) / nSize;
     if( nRet < nMemb )
-        bEOF = TRUE;
+        bEOF = true;
 
     // Give a chance to specialized filesystem to deal with errors to redirect
     // elsewhere.
@@ -1441,7 +1404,7 @@ size_t VSICurlStreamingHandle::Read( void * const pBuffer, size_t const nSize,
         {
             curOffset = 0;
             nRingBufferFileOffset = 0;
-            bEOF = FALSE;
+            bEOF = false;
             AcquireMutex();
             eExists = EXIST_UNKNOWN;
             bHasComputedFileSize = FALSE;
@@ -1552,7 +1515,7 @@ VSICurlStreamingFSHandler::VSICurlStreamingFSHandler()
 
 VSICurlStreamingFSHandler::~VSICurlStreamingFSHandler()
 {
-    ClearCache();
+    VSICurlStreamingFSHandler::ClearCache();
 
     CPLDestroyMutex( hMutex );
     hMutex = nullptr;
@@ -1566,12 +1529,9 @@ void VSICurlStreamingFSHandler::ClearCache()
 {
     CPLMutexHolder oHolder( &hMutex );
 
-    for( std::map<CPLString, CachedFileProp*>::const_iterator
-             iterCacheFileSize = cacheFileSize.begin();
-         iterCacheFileSize != cacheFileSize.end();
-         iterCacheFileSize++ )
+    for( auto& kv: cacheFileSize )
     {
-        CPLFree(iterCacheFileSize->second);
+        CPLFree(kv.second);
     }
     cacheFileSize.clear();
 }
@@ -1637,7 +1597,8 @@ VSICurlStreamingFSHandler::CreateFileHandle( const char* pszURL )
 
 VSIVirtualHandle* VSICurlStreamingFSHandler::Open( const char *pszFilename,
                                                    const char *pszAccess,
-                                                   bool /* bSetError */ )
+                                                   bool /* bSetError */,
+                                                   CSLConstList /* papszOptions */ )
 {
     if( !STARTS_WITH_CI(pszFilename, GetFSPrefix()) )
         return nullptr;
@@ -1677,8 +1638,6 @@ int VSICurlStreamingFSHandler::Stat( const char *pszFilename,
     if( !STARTS_WITH_CI(pszFilename, GetFSPrefix()) )
         return -1;
 
-    CPLString osFilename(pszFilename);
-
     memset(pStatBuf, 0, sizeof(VSIStatBufL));
 
     VSICurlStreamingHandle* poHandle =
@@ -1708,13 +1667,13 @@ int VSICurlStreamingFSHandler::Stat( const char *pszFilename,
 
 const char* VSICurlStreamingFSHandler::GetActualURL(const char* pszFilename)
 {
-    VSICurlStreamingHandle* poHandle = dynamic_cast<VSICurlStreamingHandle*>(
-        Open(pszFilename, "rb", false));
+    if( !STARTS_WITH_CI(pszFilename, GetFSPrefix()) )
+        return pszFilename;
+    auto poHandle = std::unique_ptr<VSICurlStreamingHandle>(
+        CreateFileHandle(pszFilename + GetFSPrefix().size()));
     if( poHandle == nullptr )
         return pszFilename;
-    CPLString osURL(poHandle->GetURL());
-    delete poHandle;
-    return CPLSPrintf("%s", osURL.c_str());
+    return CPLSPrintf("%s", poHandle->GetURL());
 }
 
 /************************************************************************/
@@ -1723,8 +1682,10 @@ const char* VSICurlStreamingFSHandler::GetActualURL(const char* pszFilename)
 
 class IVSIS3LikeStreamingFSHandler: public VSICurlStreamingFSHandler
 {
+        CPL_DISALLOW_COPY_ASSIGN(IVSIS3LikeStreamingFSHandler)
+
 public:
-        IVSIS3LikeStreamingFSHandler() {}
+        IVSIS3LikeStreamingFSHandler() = default;
 
         virtual void UpdateMapFromHandle( IVSIS3LikeHandleHelper * /*poHandleHelper*/ ) {}
         virtual void UpdateHandleFromMap( IVSIS3LikeHandleHelper * /*poHandleHelper*/ ) {}
@@ -1736,21 +1697,27 @@ public:
 
 class VSIS3StreamingFSHandler final: public IVSIS3LikeStreamingFSHandler
 {
-    std::map< CPLString, VSIS3UpdateParams > oMapBucketsToS3Params;
+    CPL_DISALLOW_COPY_ASSIGN(VSIS3StreamingFSHandler)
 
 protected:
     CPLString GetFSPrefix() override { return "/vsis3_streaming/"; }
     VSICurlStreamingHandle* CreateFileHandle( const char* pszURL ) override;
 
 public:
-    VSIS3StreamingFSHandler() {}
-    ~VSIS3StreamingFSHandler() override {}
+    VSIS3StreamingFSHandler() = default;
+    ~VSIS3StreamingFSHandler() override = default;
 
     const char* GetOptions() override
                             { return VSIGetFileSystemOptions("/vsis3/"); }
 
     void UpdateMapFromHandle( IVSIS3LikeHandleHelper * poHandleHelper ) override;
     void UpdateHandleFromMap( IVSIS3LikeHandleHelper * poHandleHelper ) override;
+
+    void ClearCache() override
+    {
+        IVSIS3LikeStreamingFSHandler::ClearCache();
+        VSIS3UpdateParams::ClearCache();
+    }
 };
 
 /************************************************************************/
@@ -1760,16 +1727,7 @@ public:
 void VSIS3StreamingFSHandler::UpdateMapFromHandle(
     IVSIS3LikeHandleHelper * poHandleHelper )
 {
-    CPLMutexHolder oHolder( &hMutex );
-
-    VSIS3HandleHelper * poS3HandleHelper =
-        dynamic_cast<VSIS3HandleHelper *>(poHandleHelper);
-    CPLAssert( poS3HandleHelper );
-    if( !poS3HandleHelper )
-        return;
-
-    oMapBucketsToS3Params[ poS3HandleHelper->GetBucket() ] =
-        VSIS3UpdateParams ( poS3HandleHelper );
+    VSIS3UpdateParams::UpdateMapFromHandle(poHandleHelper);
 }
 
 /************************************************************************/
@@ -1779,20 +1737,7 @@ void VSIS3StreamingFSHandler::UpdateMapFromHandle(
 void VSIS3StreamingFSHandler::UpdateHandleFromMap(
     IVSIS3LikeHandleHelper * poHandleHelper )
 {
-    CPLMutexHolder oHolder( &hMutex );
-
-    VSIS3HandleHelper * poS3HandleHelper =
-        dynamic_cast<VSIS3HandleHelper *>(poHandleHelper);
-    CPLAssert( poS3HandleHelper );
-    if( !poS3HandleHelper )
-        return;
-
-    std::map< CPLString, VSIS3UpdateParams>::iterator oIter =
-        oMapBucketsToS3Params.find(poS3HandleHelper->GetBucket());
-    if( oIter != oMapBucketsToS3Params.end() )
-    {
-        oIter->second.UpdateHandlerHelper(poS3HandleHelper);
-    }
+    VSIS3UpdateParams::UpdateHandleFromMap(poHandleHelper);
 }
 
 /************************************************************************/
@@ -1801,7 +1746,9 @@ void VSIS3StreamingFSHandler::UpdateHandleFromMap(
 
 class VSIS3LikeStreamingHandle final: public VSICurlStreamingHandle
 {
-    IVSIS3LikeHandleHelper* m_poS3HandleHelper;
+    CPL_DISALLOW_COPY_ASSIGN(VSIS3LikeStreamingHandle)
+
+    IVSIS3LikeHandleHelper* m_poS3HandleHelper = nullptr;
 
   protected:
     struct curl_slist* GetCurlHeaders(
@@ -1966,15 +1913,17 @@ VSIAzureStreamingFSHandler::CreateFileHandle( const char* pszURL )
 
 class VSIOSSStreamingFSHandler final: public IVSIS3LikeStreamingFSHandler
 {
-    std::map< CPLString, VSIOSSUpdateParams > oMapBucketsToOSSParams;
+    CPL_DISALLOW_COPY_ASSIGN(VSIOSSStreamingFSHandler)
+
+    std::map< CPLString, VSIOSSUpdateParams > oMapBucketsToOSSParams{};
 
   protected:
     CPLString GetFSPrefix() override { return "/vsioss_streaming/"; }
     VSICurlStreamingHandle* CreateFileHandle( const char* pszURL ) override;
 
   public:
-    VSIOSSStreamingFSHandler() {}
-    ~VSIOSSStreamingFSHandler() override {}
+    VSIOSSStreamingFSHandler() = default;
+    ~VSIOSSStreamingFSHandler() override = default;
 
     const char* GetOptions() override
                         { return VSIGetFileSystemOptions("/vsioss/"); }
@@ -1993,11 +1942,7 @@ void VSIOSSStreamingFSHandler::UpdateMapFromHandle(
     CPLMutexHolder oHolder( &hMutex );
 
     VSIOSSHandleHelper * poOSSHandleHelper =
-        dynamic_cast<VSIOSSHandleHelper *>(poHandleHelper);
-    CPLAssert( poOSSHandleHelper );
-    if( !poOSSHandleHelper )
-        return;
-
+        cpl::down_cast<VSIOSSHandleHelper *>(poHandleHelper);
     oMapBucketsToOSSParams[ poOSSHandleHelper->GetBucket() ] =
         VSIOSSUpdateParams ( poOSSHandleHelper );
 }
@@ -2012,11 +1957,7 @@ void VSIOSSStreamingFSHandler::UpdateHandleFromMap(
     CPLMutexHolder oHolder( &hMutex );
 
     VSIOSSHandleHelper * poOSSHandleHelper =
-        dynamic_cast<VSIOSSHandleHelper *>(poHandleHelper);
-    CPLAssert( poOSSHandleHelper );
-    if( !poOSSHandleHelper )
-        return;
-
+        cpl::down_cast<VSIOSSHandleHelper *>(poHandleHelper);
     std::map< CPLString, VSIOSSUpdateParams>::iterator oIter =
         oMapBucketsToOSSParams.find(poOSSHandleHelper->GetBucket());
     if( oIter != oMapBucketsToOSSParams.end() )
@@ -2204,14 +2145,12 @@ void VSICurlStreamingClearCache( void )
     // FIXME ? Currently we have different filesystem instances for
     // vsicurl/, /vsis3/, /vsigs/ . So each one has its own cache of regions,
     // file size, etc.
-    const char* const apszFS[] = { "/vsicurl_streaming/", "/vsis3_streaming/",
-                                   "/vsigs_streaming/", "vsiaz_streaming/",
-                                   "/vsioss_streaming/", "/vsiswift_streaming/" };
-    for( size_t i = 0; i < CPL_ARRAYSIZE(apszFS); ++i )
+    CSLConstList papszPrefix = VSIFileManager::GetPrefixes();
+    for( size_t i = 0; papszPrefix && papszPrefix[i]; ++i )
     {
-        VSICurlStreamingFSHandler *poFSHandler =
+        auto poFSHandler =
             dynamic_cast<VSICurlStreamingFSHandler*>(
-                VSIFileManager::GetHandler( apszFS[i] ));
+                VSIFileManager::GetHandler( papszPrefix[i] ));
 
         if( poFSHandler )
             poFSHandler->ClearCache();
