@@ -32,6 +32,8 @@
 #include "cpl_vsil_curl_priv.h"
 #include "cpl_vsil_curl_class.h"
 
+#include <errno.h>
+
 #include <algorithm>
 #include <set>
 #include <map>
@@ -39,7 +41,7 @@
 
 #include "cpl_alibaba_oss.h"
 
-CPL_CVSID("$Id: cpl_vsil_webhdfs.cpp 78ebfa67fe512cc98e3452fc307a0a17e88d0b68 2019-06-21 00:26:51 +0200 Even Rouault $")
+CPL_CVSID("$Id: cpl_vsil_webhdfs.cpp 818288a58e026803402172eaa5a53bcc31ebf7aa 2021-03-14 17:20:00 +0300 drons $")
 
 #ifndef HAVE_CURL
 
@@ -51,6 +53,11 @@ void VSIInstallWebHdfsHandler( void )
 
 #else
 
+#if !CURL_AT_LEAST_VERSION(7,18,2)
+// Needed for CURLINFO_REDIRECT_URL
+#error Need libcurl version 7.18.2 or newer
+#endif
+
 //! @cond Doxygen_Suppress
 #ifndef DOXYGEN_SKIP
 
@@ -86,12 +93,13 @@ public:
 
         VSIVirtualHandle *Open( const char *pszFilename,
                                 const char *pszAccess,
-                                bool bSetError ) override;
+                                bool bSetError,
+                                CSLConstList papszOptions ) override;
         int Unlink( const char *pszFilename ) override;
         int Rmdir( const char *pszFilename ) override;
         int Mkdir( const char *pszDirname, long nMode ) override;
 
-        CPLString GetFSPrefix() override { return "/vsiwebhdfs/"; }
+        CPLString GetFSPrefix() const override { return "/vsiwebhdfs/"; }
 
         const char* GetOptions() override;
 };
@@ -108,7 +116,7 @@ class VSIWebHDFSHandle final : public VSICurlHandle
     CPLString       m_osUsernameParam{};
     CPLString       m_osDelegationParam{};
 
-    bool            DownloadRegion(vsi_l_offset startOffset, int nBlocks) override;
+   std::string      DownloadRegion(vsi_l_offset startOffset, int nBlocks) override;
 
   public:
     VSIWebHDFSHandle( VSIWebHDFSFSHandler* poFS,
@@ -129,7 +137,6 @@ class VSIWebHDFSHandle final : public VSICurlHandle
 /*                           PatchWebHDFSUrl()                          */
 /************************************************************************/
 
-#ifdef HAVE_CURLINFO_REDIRECT_URL
 static CPLString PatchWebHDFSUrl(const CPLString& osURLIn,
                                  const CPLString& osNewHost)
 {
@@ -150,7 +157,6 @@ static CPLString PatchWebHDFSUrl(const CPLString& osURLIn,
     }
     return osURL;
 }
-#endif
 
 /************************************************************************/
 /*                       GetWebHDFSDataNodeHost()                       */
@@ -159,23 +165,9 @@ static CPLString PatchWebHDFSUrl(const CPLString& osURLIn,
 static CPLString GetWebHDFSDataNodeHost()
 {
     CPLString osDataNodeHost = CPLGetConfigOption("WEBHDFS_DATANODE_HOST", "");
-#ifndef HAVE_CURLINFO_REDIRECT_URL
-    if( !osDataNodeHost.empty() )
-    {
-        static bool bFirstTime = true;
-        if( bFirstTime )
-        {
-            CPLError(CE_Warning, CPLE_AppDefined,
-                 "WEBHDFS_DATANODE_HOST not honored due to too old libcurl");
-            bFirstTime = false;
-        }
-    }
-#endif
     return osDataNodeHost;
 }
 
-#ifdef HAVE_CURLINFO_REDIRECT_URL
-
 /************************************************************************/
 /*                         VSIWebHDFSWriteHandle                        */
 /************************************************************************/
@@ -232,9 +224,10 @@ VSIWebHDFSWriteHandle::VSIWebHDFSWriteHandle( VSIWebHDFSFSHandler* poFS,
                                               const char* pszFilename) :
         VSIAppendWriteHandle(poFS, poFS->GetFSPrefix(), pszFilename,
                              GetWebHDFSBufferSize()),
-        m_osURL( pszFilename + poFS->GetFSPrefix().size() )
+        m_osURL( pszFilename + poFS->GetFSPrefix().size() ),
+        m_osDataNodeHost( GetWebHDFSDataNodeHost() )
 {
-    m_osDataNodeHost = GetWebHDFSDataNodeHost();
+    // cppcheck-suppress useInitializationList
     m_osUsernameParam = CPLGetConfigOption("WEBHDFS_USERNAME", "");
     if( !m_osUsernameParam.empty() )
         m_osUsernameParam = "&user.name=" + m_osUsernameParam;
@@ -297,6 +290,10 @@ bool VSIWebHDFSWriteHandle::CreateFile()
         return false;
     }
 
+    NetworkStatisticsFileSystem oContextFS(m_poFS->GetFSPrefix());
+    NetworkStatisticsFile oContextFile(m_osFilename);
+    NetworkStatisticsAction oContextAction("Write");
+
     CPLString osURL = m_osURL + "?op=CREATE&overwrite=true" +
         m_osUsernameParam + m_osDelegationParam;
 
@@ -336,6 +333,8 @@ retry:
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogPUT(0);
+
     long response_code = 0;
     curl_easy_getinfo(hCurlHandle, CURLINFO_HTTP_CODE, &response_code);
 
@@ -390,6 +389,10 @@ retry:
 
 bool VSIWebHDFSWriteHandle::Append()
 {
+    NetworkStatisticsFileSystem oContextFS(m_poFS->GetFSPrefix());
+    NetworkStatisticsFile oContextFile(m_osFilename);
+    NetworkStatisticsAction oContextAction("Write");
+
     CPLString osURL = m_osURL + "?op=APPEND" +
                       m_osUsernameParam + m_osDelegationParam;
 
@@ -412,6 +415,8 @@ bool VSIWebHDFSWriteHandle::Append()
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogPOST(0, 0);
+
     long response_code = 0;
     curl_easy_getinfo(hCurlHandle, CURLINFO_HTTP_CODE, &response_code);
 
@@ -472,6 +477,8 @@ bool VSIWebHDFSWriteHandle::Append()
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogPOST(m_nBufferOff, 0);
+
     response_code = 0;
     curl_easy_getinfo(hCurlHandle, CURLINFO_HTTP_CODE, &response_code);
 
@@ -493,40 +500,46 @@ bool VSIWebHDFSWriteHandle::Append()
     return response_code == 200;
 }
 
-#endif // #ifdef HAVE_CURLINFO_REDIRECT_URL
-
-
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
 VSIVirtualHandle* VSIWebHDFSFSHandler::Open( const char *pszFilename,
                                         const char *pszAccess,
-                                        bool bSetError)
+                                        bool bSetError,
+                                        CSLConstList papszOptions )
 {
     if( !STARTS_WITH_CI(pszFilename, GetFSPrefix()) )
         return nullptr;
 
     if( strchr(pszAccess, 'w') != nullptr || strchr(pszAccess, 'a') != nullptr )
     {
-#ifdef HAVE_CURLINFO_REDIRECT_URL
+        if( strchr(pszAccess, '+') != nullptr &&
+            !CPLTestBool(CPLGetConfigOption("CPL_VSIL_USE_TEMP_FILE_FOR_RANDOM_WRITE", "NO")) )
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                        "w+ not supported for /vsiwebhdfs, unless "
+                        "CPL_VSIL_USE_TEMP_FILE_FOR_RANDOM_WRITE is set to YES");
+            errno = EACCES;
+            return nullptr;
+        }
+
         VSIWebHDFSWriteHandle* poHandle =
             new VSIWebHDFSWriteHandle(this, pszFilename);
         if( !poHandle->IsOK() )
         {
             delete poHandle;
-            poHandle = nullptr;
+            return nullptr;
+        }
+        if( strchr(pszAccess, '+') != nullptr)
+        {
+            return VSICreateUploadOnCloseFile(poHandle);
         }
         return poHandle;
-#else
-        CPLError(CE_Failure, CPLE_AppDefined,
-                 "libcurl >= 7.18.1 for /vsiwebhdfs support");
-        return nullptr;
-#endif
     }
 
     return
-        VSICurlFilesystemHandler::Open(pszFilename, pszAccess, bSetError);
+        VSICurlFilesystemHandler::Open(pszFilename, pszAccess, bSetError, papszOptions);
 }
 
 /************************************************************************/
@@ -585,6 +598,9 @@ char** VSIWebHDFSFSHandler::GetFileList( const char *pszDirname,
         CPLDebug("WEBHDFS", "GetFileList(%s)" , pszDirname);
     *pbGotFileList = false;
 
+    NetworkStatisticsFileSystem oContextFS(GetFSPrefix());
+    NetworkStatisticsAction oContextAction("ListBucket");
+
     CPLAssert( strlen(pszDirname) >= GetFSPrefix().size() );
     CPLString osDirnameWithoutPrefix = pszDirname + GetFSPrefix().size();
 
@@ -621,6 +637,8 @@ char** VSIWebHDFSFSHandler::GetFileList( const char *pszDirname,
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogGET(sWriteFuncData.nSize);
+
     long response_code = 0;
     curl_easy_getinfo(hCurlHandle, CURLINFO_HTTP_CODE, &response_code);
 
@@ -683,6 +701,9 @@ int VSIWebHDFSFSHandler::Unlink( const char *pszFilename )
     if( !STARTS_WITH_CI(pszFilename, GetFSPrefix()) )
         return -1;
 
+    NetworkStatisticsFileSystem oContextFS(GetFSPrefix());
+    NetworkStatisticsAction oContextAction("Unlink");
+
     CPLString osBaseURL = GetURLFromFilename(pszFilename);
 
     CURLM* hCurlMultiHandle = GetCurlMultiHandleFor(osBaseURL);
@@ -716,6 +737,8 @@ int VSIWebHDFSFSHandler::Unlink( const char *pszFilename )
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogDELETE();
+
     long response_code = 0;
     curl_easy_getinfo(hCurlHandle, CURLINFO_HTTP_CODE, &response_code);
 
@@ -760,6 +783,9 @@ int VSIWebHDFSFSHandler::Unlink( const char *pszFilename )
 
 int VSIWebHDFSFSHandler::Rmdir( const char *pszFilename )
 {
+    NetworkStatisticsFileSystem oContextFS(GetFSPrefix());
+    NetworkStatisticsAction oContextAction("Rmdir");
+
     return Unlink(pszFilename);
 }
 
@@ -790,6 +816,9 @@ int VSIWebHDFSFSHandler::Mkdir( const char *pszDirname, long nMode )
         return -1;
     }
 
+    NetworkStatisticsFileSystem oContextFS(GetFSPrefix());
+    NetworkStatisticsAction oContextAction("Mkdir");
+
     CPLString osBaseURL = GetURLFromFilename(osDirnameWithoutEndSlash);
 
     CURLM* hCurlMultiHandle = GetCurlMultiHandleFor(osBaseURL);
@@ -828,6 +857,8 @@ int VSIWebHDFSFSHandler::Mkdir( const char *pszDirname, long nMode )
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogPUT(0);
+
     long response_code = 0;
     curl_easy_getinfo(hCurlHandle, CURLINFO_HTTP_CODE, &response_code);
 
@@ -874,9 +905,10 @@ int VSIWebHDFSFSHandler::Mkdir( const char *pszDirname, long nMode )
 
 VSIWebHDFSHandle::VSIWebHDFSHandle( VSIWebHDFSFSHandler* poFSIn,
                           const char* pszFilename, const char* pszURL ) :
-        VSICurlHandle(poFSIn, pszFilename, pszURL)
+        VSICurlHandle(poFSIn, pszFilename, pszURL),
+        m_osDataNodeHost(GetWebHDFSDataNodeHost())
 {
-    m_osDataNodeHost = GetWebHDFSDataNodeHost();
+    // cppcheck-suppress useInitializationList
     m_osUsernameParam = CPLGetConfigOption("WEBHDFS_USERNAME", "");
     if( !m_osUsernameParam.empty() )
         m_osUsernameParam = "&user.name=" + m_osUsernameParam;
@@ -895,14 +927,18 @@ vsi_l_offset VSIWebHDFSHandle::GetFileSize( bool bSetError )
     if( oFileProp.bHasComputedFileSize )
         return oFileProp.fileSize;
 
+    NetworkStatisticsFileSystem oContextFS(poFS->GetFSPrefix());
+    NetworkStatisticsFile oContextFile(m_osFilename);
+    NetworkStatisticsAction oContextAction("GetFileSize");
+
     oFileProp.bHasComputedFileSize = true;
 
     CURLM* hCurlMultiHandle = poFS->GetCurlMultiHandleFor(m_pszURL);
 
     CPLString osURL(m_pszURL);
 
-    if( osURL.find("/webhdfs/v1") ==
-        osURL.size() - strlen("/webhdfs/v1") &&
+    if( osURL.size() > strlen("/webhdfs/v1") &&
+        osURL.find("/webhdfs/v1") == osURL.size() - strlen("/webhdfs/v1") &&
         std::count(osURL.begin(),
                    osURL.end(),'/') == 4 )
     {
@@ -935,6 +971,8 @@ vsi_l_offset VSIWebHDFSHandle::GetFileSize( bool bSetError )
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogGET(sWriteFuncData.nSize);
+
     long response_code = 0;
     curl_easy_getinfo(hCurlHandle, CURLINFO_HTTP_CODE, &response_code);
 
@@ -997,15 +1035,19 @@ vsi_l_offset VSIWebHDFSHandle::GetFileSize( bool bSetError )
 /*                          DownloadRegion()                            */
 /************************************************************************/
 
-bool VSIWebHDFSHandle::DownloadRegion( const vsi_l_offset startOffset,
-                                       const int nBlocks )
+std::string VSIWebHDFSHandle::DownloadRegion( const vsi_l_offset startOffset,
+                                              const int nBlocks )
 {
     if( bInterrupted && bStopOnInterruptUntilUninstall )
-        return false;
+        return std::string();
 
     poFS->GetCachedFileProp(m_pszURL, oFileProp);
     if( oFileProp.eExists == EXIST_NO )
-        return false;
+        return std::string();
+
+    NetworkStatisticsFileSystem oContextFS(poFS->GetFSPrefix());
+    NetworkStatisticsFile oContextFile(m_osFilename);
+    NetworkStatisticsAction oContextAction("Read");
 
     CURLM* hCurlMultiHandle = poFS->GetCurlMultiHandleFor(m_pszURL);
 
@@ -1014,11 +1056,7 @@ bool VSIWebHDFSHandle::DownloadRegion( const vsi_l_offset startOffset,
     WriteFuncStruct sWriteFuncData;
     int nRetryCount = 0;
     double dfRetryDelay = m_dfRetryDelay;
-
-#ifdef HAVE_CURLINFO_REDIRECT_URL
     bool bInRedirect = false;
-#endif
-
     const vsi_l_offset nEndOffset =
         startOffset + nBlocks * VSICURLGetDownloadChunkSize() - 1;
 
@@ -1032,10 +1070,7 @@ retry:
     curl_easy_setopt(hCurlHandle, CURLOPT_WRITEFUNCTION,
                      VSICurlHandleWriteFunc);
 
-
-#ifdef HAVE_CURLINFO_REDIRECT_URL
     if( !bInRedirect )
-#endif
     {
         osURL += "?op=OPEN&offset=";
         osURL += CPLSPrintf(CPL_FRMT_GUIB, startOffset);
@@ -1047,12 +1082,10 @@ retry:
     struct curl_slist* headers =
         VSICurlSetOptions(hCurlHandle, osURL, m_papszHTTPOptions);
 
-#ifdef HAVE_CURLINFO_REDIRECT_URL
     if( !m_osDataNodeHost.empty() )
     {
         curl_easy_setopt(hCurlHandle, CURLOPT_FOLLOWLOCATION, 0);
     }
-#endif
 
     if( ENABLE_DEBUG )
         CPLDebug("WEBHDFS", "Downloading %s...", osURL.c_str());
@@ -1069,6 +1102,8 @@ retry:
 
     curl_slist_free_all(headers);
 
+    NetworkStatisticsLogger::LogGET(sWriteFuncData.nSize);
+
     if( sWriteFuncData.bInterrupted )
     {
         bInterrupted = true;
@@ -1076,7 +1111,7 @@ retry:
         CPLFree(sWriteFuncData.pBuffer);
         curl_easy_cleanup(hCurlHandle);
 
-        return false;
+        return std::string();
     }
 
     long response_code = 0;
@@ -1085,7 +1120,6 @@ retry:
     if( ENABLE_DEBUG )
         CPLDebug("WEBHDFS", "Got response_code=%ld", response_code);
 
-#ifdef HAVE_CURLINFO_REDIRECT_URL
     if( !bInRedirect )
     {
         char *pszRedirectURL = nullptr;
@@ -1108,7 +1142,6 @@ retry:
             goto retry;
         }
     }
-#endif
 
     if( response_code != 200 )
     {
@@ -1147,7 +1180,7 @@ retry:
         }
         CPLFree(sWriteFuncData.pBuffer);
         curl_easy_cleanup(hCurlHandle);
-        return false;
+        return std::string();
     }
 
     oFileProp.eExists = EXIST_YES;
@@ -1157,10 +1190,13 @@ retry:
                               sWriteFuncData.pBuffer,
                               sWriteFuncData.nSize);
 
+    std::string osRet;
+    osRet.assign(sWriteFuncData.pBuffer, sWriteFuncData.nSize);
+
     CPLFree(sWriteFuncData.pBuffer);
     curl_easy_cleanup(hCurlHandle);
 
-    return true;
+    return osRet;
 }
 
 
